package Klava;

import java.io.* ;
import org.mikado.imc.mobility.*;

public class DefaultNodeRemoteProcessHandler implements NodeRemoteProcessHandler
{
  protected NetNode node ;
  protected KlavaProcess P ;
  protected boolean MessagesOn = false ;
  protected KlavaProcessPacket pack;
  protected JavaByteCodeUnMarshaller unmarshaller;
  
  public DefaultNodeRemoteProcessHandler(NetNode node, KlavaProcessPacket pack, JavaByteCodeUnMarshaller unmarshaller) {
    this.node = node;
    this.P = null;
    this.pack = pack;
    this.unmarshaller = unmarshaller;
  }

  public void start() {
    (new Thread(this)).start() ;
  }

  public void run() {
    try {
      System.out.println( "Starting a Remote Process ..." ) ;      
      P = (KlavaProcess) unmarshaller.unmarshal(pack) ;
      if ( pack.environment != null ) {
	  P.setEnvironment( pack.environment ) ;
      }
      if ( pack.caller != null ) {
        P.setCaller( pack.caller );
      }
      P.mutex = pack.mutex ;
      P.otherProcess = pack.otherProcess ;
      if ( P.otherProcess != null ) {
        synchronized( P.otherProcess ) {
          if ( ! P.otherProcess.raceWinner ) {
            System.out.println( "Remote Process lost before starting." ) ;
            synchronized ( this ) {
              notifyAll() ;
            }
          } else {
            P.otherProcess.otherProcess = P ;
            System.out.println( "... Remote Process started" ) ;
            startProcess( P ) ;
          }
        }
      } else {
        startProcess( P ) ;
      }
    } catch ( ClassNotFoundException enf ) {
      System.err.println( "Node (getProcessFromPacket): " + enf ) ;
      enf.printStackTrace() ;
    } catch ( SecurityException es ) {
      System.err.println( "Node (getProcessFromPacket): " + es ) ;
      es.printStackTrace() ;
    } catch ( IOException e ) {
      e.printStackTrace() ;
    } catch ( InstantiationException e2 ) {
      e2.printStackTrace() ;
    } catch ( IllegalAccessException e3 ) {
      e3.printStackTrace() ;
    }
  }

  synchronized protected void startProcess( KlavaProcess P ) {
    synchronized( P ) {
      try {
        node.addProcess( P ) ;
        P.wait() ;
      } catch ( InterruptedException e ) {
        e.printStackTrace() ;
      }
    }
    notifyAll() ;
  }

  public void setMessages( boolean on ) { MessagesOn = on ; }

  protected void PrintMessage( String s ) {
    if ( MessagesOn )
      System.out.println( s ) ;
  }
}
