package Klava ;

import java.io.* ;
import java.util.StringTokenizer ;
import java.util.Hashtable ;
import java.util.Enumeration ;

public class Environment implements Serializable, KlavaMonitorable {
  protected Hashtable environment ;
  protected KlavaMonitorable monitorable ;

  public Environment() throws KlavaMalformedPhyLocalityException {
      this( null, null ) ;
  }
  public Environment( KlavaMonitorable m ) throws KlavaMalformedPhyLocalityException {
      this( null, m ) ;
  }

  public Environment( String filename ) throws KlavaMalformedPhyLocalityException
    { this( filename, null ) ; }

  public Environment( String filename, KlavaMonitorable m ) throws KlavaMalformedPhyLocalityException {
    environment = new Hashtable() ;

    if ( m == null )
        monitorable = newMonitorable() ;
    else
        monitorable = m ;
    if ( filename != null )
      readEnvironment (filename);
  }

  protected void readEnvironment( String filename ) throws KlavaMalformedPhyLocalityException
  {
    try {
      System.out.println( "Reading Environment " + filename + "..." ) ;
      FileInputStream file = new FileInputStream( filename ) ;
      BufferedReader ifile = new BufferedReader( new InputStreamReader(file) );
      StringTokenizer st ;
      String line ;
      LogicalLocality lLoc ;
      PhysicalLocality fLoc ;
      try {
        while( ( line = ifile.readLine() ) != null ) {
          st = new StringTokenizer( line, "=:" ) ;
          lLoc = new LogicalLocality( st.nextToken() );
          fLoc = new PhysicalLocality( st.nextToken() ) ;
          add( lLoc, fLoc ) ;
        }
        ifile.close() ;
      } catch ( IOException ioe ) {
        ioe.printStackTrace() ;
      }
    } catch ( FileNotFoundException e ) {
      System.err.println( "No file " + filename +
                          " for the environment found... skipping." ) ;
    } catch ( SecurityException ex ) {
      System.err.println( "Skipping reading environment, due to Security:" ) ;
      System.err.println( ex ) ;
      // this catches SecurityException in Applets as well
    }
  }

  public KlavaMonitorable newMonitorable()  // Factory Method
    { return new KlavaMonitorableDefault() ; }

  // KlavaMonitorable interface
  public void addMonitor( KlavaMonitor m )
    {
        monitorable.addMonitor( m ) ;
        updateMonitor( m ) ;
    }
  public void removeMonitor( KlavaMonitor m )
    { monitorable.removeMonitor( m ) ; }
  public void notifyAddElement( Object o )
    { monitorable.notifyAddElement( o ) ; }
  public void notifyRemoveElement( Object o )
    { monitorable.notifyRemoveElement( o ) ; }
  public void notifyRemoveAll() {
    monitorable.notifyRemoveAll();
  }

  synchronized protected void updateMonitor( KlavaMonitor m ) {
    m.removeAll() ;
    Enumeration keys = environment.keys() ;
    String key ;
    while( keys.hasMoreElements() ) {
        key = (String) keys.nextElement() ;
        m.addElem( key + "=" + toPhysical( key ) ) ;
    }
  }

  synchronized public PhysicalLocality toPhysical( Locality l ) {
    return ( PhysicalLocality ) environment.get( l.toString() ) ;
  }

  synchronized public PhysicalLocality toPhysical( String l ) {
    return ( PhysicalLocality ) environment.get( l ) ;
  }

  synchronized public void add( LogicalLocality ll, PhysicalLocality pl ) {
    add( ll.toString(), pl ) ;
  }

  synchronized public void add( String ll, PhysicalLocality pl ) {
    environment.put( ll, pl ) ;
    notifyAddElement( ll + "=" + pl ) ;
  }

  synchronized public boolean try_add(LogicalLocality ll, PhysicalLocality pl) {
    boolean ret = (toPhysical(ll) == null);

    if (ret)
      add(ll, pl);

    return ret;
  }

  synchronized void remove(LogicalLocality l) {
    environment.remove(l.toString());
  }

  public String toString() {
    return environment.toString() ;
  }

  synchronized public Enumeration keys() {
    return environment.keys() ;
  }
}
