package Klava;

import java.io.*;
import java.util.TreeSet;
import java.util.Iterator;

public class FileMessagePrinter extends MessagePrinter {
  protected FileWriter file;
  protected String file_name;

  public FileMessagePrinter(String name, String prefix, String suffix,
                            MessagePrinter p) {
    super(name, p);
    try {
      file_name = prefix + "-" + name + "." +
        (suffix.length() == 0 ? "msgout" : suffix);
      file = new FileWriter(file_name);
    } catch (java.io.IOException e) {
      e.printStackTrace();
    }
  }

  public String getFileName() {
    return file_name;
  }

  synchronized public void Print(String s) {
    super.Print(s);
    try {
      file.write(s + "\n");
      file.flush();
    } catch (java.io.IOException e) {
      e.printStackTrace();
    }
  }

  synchronized public void close() throws IOException {
    if (file != null) {
      file.flush();
      file.close();
      file = null;
    }
  }

  synchronized public void sort() throws IOException {
    close();

    TreeSet ordered = new TreeSet();
    BufferedReader reader =
      new BufferedReader(new FileReader(file_name));

    String line = reader.readLine();
    while (line != null) {
      ordered.add(line);
      line = reader.readLine();
    }

    reader.close();

    file = new FileWriter(file_name);

    Iterator it = ordered.iterator();
    while (it.hasNext())
      file.write(it.next().toString() + "\n");

    file.close();
    file = null;
  }
}