package Klava;

/**
 * Title:
 * Description:  Implement an abstraction for an application that uses
 * Nodes, so that at the end of main it can wait for all the nodes
 * to terminate and call System.exit
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

import java.util.Vector;
import java.util.Enumeration;

public class KlavaApplication {
  protected boolean set_security_manager;
  protected Vector nodes = new Vector ();
  protected Vector processes = new Vector ();

  public KlavaApplication() {
    this(true);
  }

  public KlavaApplication(boolean set_security_manager) {
    if (set_security_manager)
      System.setSecurityManager(newSecurityManager());

    this.set_security_manager = set_security_manager;
  }

  public void setSecurityManager(boolean b) {
    set_security_manager = b;
  }

  protected SecurityManager newSecurityManager() {
    return new KlavaSecurityManager();
  }

  public boolean addNode(NetNode node) {
    if (nodes.contains(node))
      return false;

    nodes.addElement(node);
    node.setApplication(this);
    return true;
  }

  public void removeNode(NetNode node) {
    nodes.remove(node);
    node.setApplication(null);
  }

  public boolean addProcess(KlavaProcess P) {
    if (processes.contains(P))
      return false;

    processes.addElement(P);
    return true;
  }

  public void removeProcess(KlavaProcess P) {
    processes.remove(P);
  }

  public void startNodes() {
    for (int i = 0; i < nodes.size(); ++i)
      ((NetNode)(nodes.elementAt(i))).start ();
  }

  public void waitForNodesAndTerminate() {
    waitForElementsAndTerminate(nodes.elements());
  }

  public void waitForProcessesAndTerminate() {
    waitForElementsAndTerminate(processes.elements());
  }

  protected void waitForElementsAndTerminate(Enumeration elements) {
    while (elements.hasMoreElements())
      try {
        ((Thread)(elements.nextElement())).join();
      } catch (InterruptedException e) {
        e.printStackTrace();
        break;
      }

    appTerminate();

    System.err.println ("KlavaApplication terminated.");
    System.exit (0);
  }

  // can be redefined by subclasses
  protected void appTerminate() {}
}