package Klava ;

import java.io.* ;
import java.util.Vector ;
import java.util.Enumeration ;
import org.mikado.imc.mobility.*;

public abstract class KlavaProcess extends JavaMigratingCode implements java.io.Serializable, Callable
{
  transient protected NodeInterface node ; // where we're executing
  transient protected boolean is_bound = false; // bound to a node decorator
  protected Vector environment = new Vector() ; // process' env.
  protected boolean executing = false ; // are we executing?
  protected String procName ; // this process' name
  protected boolean MessagesOn = false ;
  protected boolean notifyOnNonExistentLocality = false;
  // whether we have to receive an exception for non-existent phys loc

  public final static int NOT_MIGRATED = 0;
  public final static int MIGRATED = 1;
  public final static int CONTINUED = 2;
  public final static int ARRIVED = 3;
  public final static int MIGRATING = 4;

  transient protected KlavaProcess caller = null;
  transient protected MessagePrinter message_printer = null;

  protected KlavaProcess migratableCaller = null;
  protected KlavaProcess called = null;
  protected KInteger CurrentMark = new KInteger(0);
  protected int MigrationStatus = NOT_MIGRATED;
  protected boolean previouslyArrived = false;

  public KlavaProcess otherProcess = null ; // for + operator
  public Object mutex = null ; // "          "
  public boolean raceWinner = true ; // "        "

  static private int ProcID = 0 ;

  protected LogicalLocality self = new LogicalLocality( "self" ) ;

  public KlavaProcess( NodeInterface n ) {
    super( "KP-" + nextProcID() ) ;
    node = n ;
    procName = getClass().getName() ;
    addExcludePackages();
  }

  public KlavaProcess() {
    super( "KP-" + nextProcID() ) ;
    procName = getClass().getName() ;
    addExcludePackages();
  }

  protected void addExcludePackages() {
    addExcludePackage("Klava.");
    addExcludePackage("momi.");
  }

  static private synchronized int nextProcID() {
    return ProcID++ ;
  }


  final public String KlavaProcName() {
    return procName ;
  }

  final public void setKlavaProcName(String s) {
    procName = s;
  }

  final public void setNode( NodeInterface n ) {
    node = n ;
  }

  final public void in( TupleItem o1, Locality l, long TimeOut ) throws KlavaException {
    Tuple t = new Tuple( o1 ) ;
    readin( t, l, NodeMessage.IN, TimeOut, true ) ;
  }
  final public void in( TupleItem o1, TupleItem o2, Locality l, long TimeOut )
    throws KlavaException {
      Tuple t = new Tuple( o1, o2 ) ;
      readin( t, l, NodeMessage.IN, TimeOut, true ) ;
  }
  final public void in( TupleItem o1, TupleItem o2, TupleItem o3, Locality l, long TimeOut )
    throws KlavaException {
      Tuple t = new Tuple( o1, o2, o3 ) ;
      readin( t, l, NodeMessage.IN, TimeOut, true ) ;
  }
  final public void in( TupleItem o1, TupleItem o2, TupleItem o3, TupleItem o4, Locality l, long TimeOut )
    throws KlavaException {
      Tuple t = new Tuple( o1, o2, o3, o4 ) ;
      readin( t, l, NodeMessage.IN, TimeOut, true ) ;
  }
  final public void in( TupleItem o1, Locality l ) throws KlavaException {
    Tuple t = new Tuple( o1 ) ;
    readin( t, l, NodeMessage.IN, -1, true ) ;
  }
  final public void in( TupleItem o1, TupleItem o2, Locality l )
    throws KlavaException {
      Tuple t = new Tuple( o1, o2 ) ;
      readin( t, l, NodeMessage.IN, -1, true ) ;
  }
  final public void in( TupleItem o1, TupleItem o2, TupleItem o3, Locality l )
    throws KlavaException {
      Tuple t = new Tuple( o1, o2, o3 ) ;
      readin( t, l, NodeMessage.IN, -1, true ) ;
  }
  final public void in( TupleItem o1, TupleItem o2, TupleItem o3, TupleItem o4, Locality l )
    throws KlavaException {
      Tuple t = new Tuple( o1, o2, o3, o4 ) ;
      readin( t, l, NodeMessage.IN, -1, true ) ;
  }

  final public void in( Tuple t, Locality l ) throws KlavaException {
    node.in( t, l ) ;
  }
  final public boolean in_nb(Tuple t, Locality l) throws KlavaException {
    return node.in_nb(t, l);
  }
  final public void in( Tuple t ) throws KlavaException {
    node.in( t ) ;
  }
  final public void in( Tuple t, Locality l, long TimeOut ) throws KlavaException {
    node.in( t, l, TimeOut ) ;
  }
  final public boolean in_t(Tuple t, Locality l, long TimeOut) throws KlavaException
  {
     try {
       in(t, l, TimeOut);
       return true;
     }  catch (KlavaTimeOutException e) {
            return false;
     }
  }
  final public void in( Tuple t, long TimeOut ) throws KlavaException {
    node.in( t, TimeOut ) ;
  }

  final public void out( Object o1, Locality l ) throws KlavaException {
    out( new Tuple( o1 ), l );
  }
  final public void out( Object o1, Object o2, Locality l  ) throws KlavaException {
    out( new Tuple( o1, o2 ), l ) ;
  }
  final public void out( Object o1, Object o2, Object o3, Locality l  ) throws KlavaException {
    out( new Tuple( o1, o2, o3 ), l ) ;
  }
  final public void out( Object o1, Object o2, Object o3, Object o4, Locality l  ) throws KlavaException {
    out( new Tuple( o1, o2, o3, o4 ), l ) ;
  }
  final public void out( Tuple t, Locality l ) throws KlavaException {
    node.out( t, l ) ;
  }
  final public void out( Tuple t ) throws KlavaException {
    node.out( t ) ;
  }

  final public void read( TupleItem o1, Locality l, long TimeOut )
    throws KlavaException {
      Tuple t = new Tuple( o1 ) ;
      readin( t, l, NodeMessage.READ, TimeOut, true ) ;
  }
  final public void read( TupleItem o1, TupleItem o2, Locality l, long TimeOut )
    throws KlavaException {
      Tuple t = new Tuple( o1, o2 ) ;
      readin( t, l, NodeMessage.READ, TimeOut, true ) ;
  }
  final public void read( TupleItem o1, TupleItem o2, TupleItem o3, Locality l, long TimeOut )
    throws KlavaException {
      Tuple t = new Tuple( o1, o2, o3 ) ;
      readin( t, l, NodeMessage.READ, TimeOut, true ) ;
  }
  final public void read( TupleItem o1, TupleItem o2, TupleItem o3, TupleItem o4, Locality l, long TimeOut )
    throws KlavaException {
      Tuple t = new Tuple( o1, o2, o3, o4 ) ;
      readin( t, l, NodeMessage.READ, TimeOut, true ) ;
  }
  final public void read( TupleItem o1, Locality l )
    throws KlavaException {
      Tuple t = new Tuple( o1 ) ;
      readin( t, l, NodeMessage.READ, -1, true ) ;
  }
  final public void read( TupleItem o1, TupleItem o2, Locality l )
    throws KlavaException {
      Tuple t = new Tuple( o1, o2 ) ;
      readin( t, l, NodeMessage.READ, -1, true ) ;
  }
  final public void read( TupleItem o1, TupleItem o2, TupleItem o3, Locality l )
    throws KlavaException {
      Tuple t = new Tuple( o1, o2, o3 ) ;
      readin( t, l, NodeMessage.READ, -1, true ) ;
  }
  final public void read( TupleItem o1, TupleItem o2, TupleItem o3, TupleItem o4, Locality l )
    throws KlavaException {
      Tuple t = new Tuple( o1, o2, o3, o4 ) ;
      readin( t, l, NodeMessage.READ, -1, true ) ;
  }
  final public void read( Tuple t, Locality l ) throws KlavaException {
    node.read( t, l ) ;
  }
  final public boolean read_nb(Tuple t, Locality l) throws KlavaException {
    return node.read_nb(t, l);
  }
  final public void read( Tuple t ) throws KlavaException {
    node.read( t ) ;
  }
  final public void read( Tuple t, Locality l, long TimeOut ) throws KlavaException {
    node.read( t, l, TimeOut ) ;
  }
  final public boolean read_t(Tuple t, Locality l, long TimeOut) throws KlavaException
  {
     try {
        read(t, l, TimeOut);
        return true;
     } catch (KlavaTimeOutException e) {
            return false;
     }
  }
  final public void read( Tuple t, long TimeOut ) throws KlavaException {
    node.read( t, TimeOut ) ;
  }

  final protected boolean readin( Tuple t, Locality l, int OpCode, long TimeOut,
                               boolean blocking )
    throws KlavaException {
      if( OpCode == NodeMessage.IN ) {
        if (blocking)
          node.in( t, l, TimeOut ) ;
        else
          return node.in_nb(t, l);
      } else if ( OpCode == NodeMessage.READ ) {
        if (blocking)
          node.read( t, l, TimeOut ) ;
        else
          return node.read_nb(t, l);
      }

      return true;
  }

  final public void eval( KlavaProcessVar P, Locality l )
    throws KlavaException {
      node.eval( P, l ) ;
  }
  final public void eval( KlavaProcessVar P )
    throws KlavaException {
      node.eval( P ) ;
  }

  final public void eval( KlavaProcess P, Locality l ) throws KlavaException {
    node.eval( P, l ) ;
  }
  final public void eval( KlavaProcess P ) throws KlavaException {
    node.eval( P ) ;
  }

  final public Locality newloc() throws KlavaException {
    return node.newloc() ;
  }

  final public void removeloc( Locality l ) throws KlavaException {
    node.removeloc( l ) ;
  }

  final public void or( KlavaProcess P1, KlavaProcess P2 )
    throws KlavaException {
      P1.setEnvironment( environment ) ;
      P2.setEnvironment( environment ) ;
      node.or( P1, P2 ) ;
  }

  final public void procCall( Callable P ) throws KlavaException {
    P.setEnvironment( environment ) ;
    P.setCaller( this );
    node.procCall( P ) ;
  }

  final public int procCall( KlavaProcess P ) throws KlavaException
  {
      P.setMigratableCaller( this );
      called = P;
      procCall( (Callable)P );
      MigrationStatus = P.getMigrationStatus();
      return MigrationStatus ;
  }

  protected void setMigratableCaller( KlavaProcess P )
  {
    migratableCaller = P ;
  }

  public KlavaProcess getMigratableCaller()
  {
    return migratableCaller ;
  }

  final void setBound()
  {
    is_bound = true;
  }

  final boolean isBound()
  {
      return is_bound;
  }

  public KlavaProcess duplicate()
  {
      try {
          ByteArrayOutputStream bostream = new ByteArrayOutputStream();
          ObjectOutputStream ostream = new ObjectOutputStream(bostream);

          ostream.writeObject(this);

          ByteArrayInputStream bistream = new ByteArrayInputStream
              (bostream.toByteArray());
          ObjectInputStream istream = new ObjectInputStream(bistream);

          return (KlavaProcess) istream.readObject();
      } catch (IOException ioe) {
          ioe.printStackTrace();
          return null;
      } catch (ClassNotFoundException ce) {
          ce.printStackTrace();
          return null;
      }
  }

  final public int getMigrationStatus()
  {
    return MigrationStatus;
  }


  final public void migrate( Locality loc )
  {
    try
    {
      PhysicalLocality phyloc = node.getPhysicalLocality(loc);
      if ( phyloc.equals(node.ThisLocality()) )
          {
              MigrationStatus = CONTINUED;
          }
      else
          {
              MigrationStatus = MIGRATING;
              eval(this, loc);
              MigrationStatus = MIGRATED;
          }
    }
    catch ( KlavaException ke )
    {
      MigrationStatus = NOT_MIGRATED;
      ke.printStackTrace ();
      System.err.println ("Migration to " + loc + " FAILED!");
    }
  }

    /**
     * retrieve the contents of the .class file for a specific class
     * @param  className the name of the class whose contents are retrieved
     * @return the byte array with the contents of .class file
     */
    synchronized public byte[] getClassBytes( String className ) {
        byte[] ClassBytes = null ;

        if ( usedClasses != null )
            ClassBytes = (byte[])usedClasses.get( className ) ;

        if ( ClassBytes != null )
            return ClassBytes ;
        else
            ClassBytes = getClassBytesFromClassLoader( className ) ;

        return ClassBytes ;
    }

    /**
     * Retrieve the code of a class from the class loader table
     * @param  className the name of the class whose contents are retrieved
     * @return the contents of the class
     */
    public byte[] getClassBytesFromClassLoader( String className ) {
        ClassLoader classLoader = getClass().getClassLoader() ;

        if ( classLoader instanceof NodeClassLoader ) {
            byte[] ClassBytes =
                ((NodeClassLoader)classLoader).getClassBytes( className ) ;
            if ( ClassBytes != null )
                return ClassBytes ;
        }

        try {
            byte result[] = ClassBytesLoader.loadClassBytes(className) ;
            return result ;
        } catch ( FileNotFoundException nof ) {
            nof.printStackTrace() ;
            return null ;
        } catch ( Exception e ) {
            e.printStackTrace() ;
        }
        return null ;
    }

  final public void setNotifyOnNonExistentLocalities(boolean b) {
    notifyOnNonExistentLocality = b;
  }
  final public boolean getNotifyOnNonExistentLocalities() {
    return notifyOnNonExistentLocality;
  }

  final public void run() {
    executing = true ;
    if ( MigrationStatus == MIGRATING )
      setArrived() ;

    try {
      pre_execution();
      execute() ;
    } catch ( KlavaException e ) {
      System.err.println( "Uncaught " + e ) ;
      e.printStackTrace() ;
    } catch ( NoSuchMethodException ne ) {
      System.err.println( "Uncaught " + ne ) ;
      ne.printStackTrace() ;
    }
    executing = false ;
    if ( migratableCaller != null &&
         MigrationStatus != MIGRATED &&
         previouslyArrived )
    {
         node.addProcess ( migratableCaller );
         migratableCaller = null;
    }

    node.removeProcess( this ) ;
  }

  final void setArrived() {
    MigrationStatus = ARRIVED ;
    if ( ! previouslyArrived ) {
      previouslyArrived = true ;
    }
    if ( migratableCaller != null ) {
      migratableCaller.setArrived() ;
    }
  }


  final public boolean isPreviouslyArrived() {
    return previouslyArrived ;
  }


  // template method
  protected void pre_execution()
  {
  }

  // to be defined in the derived classes
  abstract public void execute() throws KlavaException, NoSuchMethodException ;

  // Callable interface

  final public void setCaller( KlavaProcess P ) {
    caller = P ;
  }

  final public KlavaProcess getCaller() {
    return caller;
  }

  final public void setEnvironment( Environment env ) {
    if ( env != null ) {
      environment.addElement( env ) ;
    }
  }
  final public void setEnvironment( Vector env ) {
    if ( env == null )
      return ;
    Enumeration en = env.elements() ;
    while ( en.hasMoreElements() )
      setEnvironment( (Environment)(en.nextElement()) ) ;
  }

  final public PhysicalLocality toPhysical( Locality l ) {
    Enumeration en = environment.elements() ;
    Environment env ;
    PhysicalLocality pl ;
    while ( en.hasMoreElements() ) {
      env = (Environment)en.nextElement() ;
      pl = env.toPhysical( l ) ;
      if ( pl != null )
        return pl ;
    }
    return null ;
  }

  final public Vector getEnvironment() {
    return environment ;
  }

  final protected PhysicalLocality getNodeLocality() {
    if (node != null)
      return node.ThisLocality();
    else
      return null;
  }

  public PhysicalLocality getPhysicalLocality(Locality loc)
        throws KlavaLogicalLocalityException {
    return node.getPhysicalLocality(loc);
  }

  public KlavaProcess closeProcess(KlavaProcess P) {
    return node.closeProcess(P);
  }

  public KlavaProcessVar closeProcess(KlavaProcessVar P) {
    return node.closeProcess(P);
  }

  final public void addToEnvironment( LogicalLocality ll, PhysicalLocality pl )
         throws KlavaMalformedPhyLocalityException {
    synchronized( environment ) {
      Environment env = (Environment)environment.firstElement() ;
      if ( env != null )
        env.add( ll, pl ) ;
      else {
        env = new Environment() ;
        env.add( ll, pl ) ;
        environment.addElement( env ) ;
      }
    }
  }

  final public void addToEnvironment( String ll, PhysicalLocality pl )
          throws KlavaMalformedPhyLocalityException
  {
    addToEnvironment( new LogicalLocality( ll ), pl ) ;
  }

  final public boolean isExecuting() {
    return executing ;
  }

  synchronized public void Print( String s ) {
    if (message_printer == null) {
      if (node != null)
        node.Print( s ) ;
      else
        System.out.println(s);
    } else
      message_printer.Print(s);
  }

  public void Print( KString s ) {
    Print( s + "");
  }

  public void setMessagePrinter(MessagePrinter msg) {
    if (message_printer == null)
      message_printer = msg;
  }

  public void pause( int TimeToPause ) {
    try {
      Thread.sleep( TimeToPause ) ;
    } catch ( InterruptedException e ) {
      System.err.println( e ) ;
    }
  }

  public void setMessages( boolean on ) { MessagesOn = on ; }

  protected void PrintMessage( String s ) {
    if ( MessagesOn )
      Print( s ) ;
  }
}
