package Klava;

/**
 * Estend KlavaApplication and possibly logs to files all the strings
 * printed by processes
 */

import java.util.*;

public class KlavaTestApplication extends KlavaApplication {
  protected FileMessagePrinterFactory msg_printer_factory = null;
  protected String name;
  protected Vector file_name_patterns = new Vector();
  // patterns for file names to sort

  public KlavaTestApplication(String name, String suffix,
                              boolean log_to_file, boolean set_security) {
    super(set_security);

    NetUtils.use_localhost_address = log_to_file;
    // for tests stored on files, always use localhost address
    // so that tests will work
    // and have the same result, when connected to Internet and when isolated

    if (name != null)
      this.name = name;
    else
      this.name = getClass().getName();

    if (log_to_file) {
      msg_printer_factory = new FileMessagePrinterFactory();
      msg_printer_factory.setPrefix(this.name);
      msg_printer_factory.setSuffix(suffix);
    }
  }

  public KlavaTestApplication(String name, String suffix,
                              boolean log_to_file) {
    this(name, suffix, log_to_file, true);
  }

  public boolean addNode(NetNode node) {
    if (super.addNode(node)) {
      node.setMessagePrinterFactory(msg_printer_factory);
      return true;
    } else
      return false;
  }

  public void addProcessToNode(KlavaProcess P, NetNode n, boolean log_proc) {
    addNode(n);
    addProcess(P);
    n.addProcess(P, log_proc);
  }

  public void addProcessToNode(KlavaProcess P, NetNode n) {
    addProcessToNode(P, n, true);
  }

  public void start() {
    startNodes();
  }

  public FileMessagePrinterFactory getFileMessagePrinterFactory() {
    return msg_printer_factory;
  }

  protected void appTerminate() {
    if (msg_printer_factory == null)
      return;

    try {
      Enumeration patterns = file_name_patterns.elements();
      while (patterns.hasMoreElements())
        msg_printer_factory.sortFiles((String) patterns.nextElement());

      msg_printer_factory.closeAllFiles();
    } catch (java.io.IOException e) {
      e.printStackTrace();
    }
  }

  // sort all files that contains this pattern in their name
  public void sortFiles(String pattern) {
    file_name_patterns.addElement(pattern);
  }
}