package Klava;

import java.net.*;
import java.io.*;

public class LoginHandler extends Thread implements LoginHandlerInterface {
  protected final static int defaultPort = 9999 ;
  protected ServerSocket serversocket ;
  protected RoutingTable routing_table;
  protected int port;
  protected NetNode node; // the node we belong to
  protected ProtocolTable proto_table = new ProtocolTable();

  public LoginHandler(NetNode node, int port_, RoutingTable routing_table)
        throws IOException {
    this.node = node;
    port = port_;
    this.routing_table = routing_table;

    if (port == -1) {
      serversocket = new ServerSocket(0);
      port = serversocket.getLocalPort();
    } else
      serversocket = new ServerSocket(port);
    Print("opened");
  }

  protected int getPort() {
    return port;
  }

  protected void accept(PhysicalLocality loc) {
    LogicalLocality dummy_loc = new LogicalLocality(); // we don't use this
    boolean accepted = false;

    while (! accepted)
      accepted = accept_registration("LOGIN", loc, dummy_loc, false);
  }

  // accept a direct connection
  protected ObjectInputStream acceptconn(PhysicalLocality loc) {
    TupleSpace ts = proto_table.get_protocol_ts("CONNECT");
    ObjectInputStream objIStream = null;

    try {
      Tuple tuple = new Tuple
        (Socket.class, ObjectInputStream.class, ObjectOutputStream.class);

      ts.in(tuple);

      Socket socket = (Socket) tuple.getItem(0) ;
      objIStream = (ObjectInputStream) tuple.getItem(1) ;
      ObjectOutputStream objOStream = (ObjectOutputStream) tuple.getItem(2) ;

      try {
        PhysicalLocality my_phy_loc;

        Integer login_port = null;
        Object next_token = objIStream.readObject();

        login_port = (Integer) next_token;
        my_phy_loc =
            new PhysicalLocality(NetUtils.getNodeIPAddress(socket.getInetAddress()) + ":" +
            login_port);

        objOStream.writeObject( new Boolean( true ) ) ;
        objOStream.flush() ;

        loc.setValue(my_phy_loc);
      } catch (ClassNotFoundException ce) {
        ce.printStackTrace();
      } catch (KlavaMalformedPhyLocalityException mfl) {
          mfl.printStackTrace();
          objOStream.writeObject( new Boolean( false ) ) ;
          objOStream.flush() ;
      }
    } catch ( IOException e ) {
      e.printStackTrace();
    }

    return objIStream;
  }

  protected ObjectOutputStream dirconnect(PhysicalLocality loc)
            throws KlavaException {
    boolean result = false;
    ObjectOutputStream objOStream = null;
    try {
      Print( "Connecting directly to " + loc + " ...") ;
      String host = NetUtils.retrieveHostFromLocality(loc);
      int peer_port = NetUtils.retrieveConnectionPortFromLocality(loc);
      Socket s = new Socket( host, peer_port ) ;
      Print( "Direct socket obtained..." ) ;
      InputStream iStream = s.getInputStream() ;
      OutputStream oStream = s.getOutputStream() ;

      objOStream = new ObjectOutputStream(oStream);
      objOStream.flush();
      objOStream.writeObject(new String("CONNECT"));
      // my physical locality
      objOStream.writeObject( new Integer(port) ) ;
      objOStream.flush() ;
      ObjectInputStream objIStream = new ObjectInputStream( iStream ) ;
      result = ((Boolean)objIStream.readObject()).booleanValue();

      Print( "Connected directly to " + loc) ;
    } catch (Exception e) {
      Print( "Failed to directly connect to " + loc) ;
      e.printStackTrace();
      return null;
    }

    return objOStream;
  }

  protected boolean register(PhysicalLocality ploc,
                             LogicalLocality lloc) {
    return accept_registration("SUBSCRIBE", ploc, lloc, true);
  }

  protected Socket accept_connection() throws IOException {
    Socket socket = serversocket.accept() ;
    Print( "New request from " +
           NetUtils.getNodeAddress(socket.getInetAddress())+
           ":" + socket.getPort() ) ;
    return socket;
  }

  public void run() {
    Socket socket ;

    while (true) {
      try {
        socket = accept_connection();

        InputStream iStream = socket.getInputStream() ;
        OutputStream oStream = socket.getOutputStream() ;

        ObjectInputStream objIStream = new ObjectInputStream( iStream ) ;
        ObjectOutputStream objOStream = new ObjectOutputStream( oStream ) ;

        Object next_token = objIStream.readObject();

        boolean handled_protocol = false;

        // read protocol identification string
        if (next_token instanceof String) {
          TupleSpace ts = proto_table.handle_protocol((String) next_token);
          if (ts != null) {
            Tuple tuple = new Tuple(socket, objIStream, objOStream);
            ts.out(tuple);
            handled_protocol = true;
          }
        }

        if (! handled_protocol)
          socket.close(); // FIXME: comunicate an error?
      } catch (IOException ioe) {
        ioe.printStackTrace();
        break;
      } catch (ClassNotFoundException ce) {
        ce.printStackTrace();
      }
    }
  }

  void stop_accept() throws IOException {
    serversocket.close();
  }

  protected boolean accept_registration(String proto,
                                        PhysicalLocality ploc,
                                        LogicalLocality lloc,
                                        boolean handle_log_loc) {
    Socket socket ;
    NodeHandler nhandler ;
    boolean accepted = false;

    try {
      TupleSpace ts = proto_table.get_protocol_ts(proto);

      Tuple tuple = new Tuple
        (Socket.class, ObjectInputStream.class, ObjectOutputStream.class);

      ts.in(tuple);

      socket = (Socket) tuple.getItem(0) ;
      ObjectInputStream objIStream = (ObjectInputStream) tuple.getItem(1) ;
      ObjectOutputStream objOStream = (ObjectOutputStream) tuple.getItem(2) ;

      String my_phy_loc_str =
              NetUtils.getNodeIPAddress(socket.getInetAddress());
      PhysicalLocality my_phy_loc = null;

      NodeHandler node_handler = null;

      boolean communicated_port = false;

      try {
        Integer login_port = null;
        Object next_token = objIStream.readObject();

        // read port for login
        if (next_token instanceof PhysicalLocality) {
          my_phy_loc = new PhysicalLocality(next_token.toString());
        } else { // must be an Integer
          communicated_port = true;
          login_port = (Integer) next_token;
          my_phy_loc = new PhysicalLocality(my_phy_loc_str + ":" + login_port);
        }

        synchronized (routing_table) {
          LogicalLocality log_loc = null;
          if (handle_log_loc) {
            // read logical locality
            log_loc = (LogicalLocality)
            objIStream.readObject();

            accepted = node.try_AddToEnv(log_loc, my_phy_loc);

            if (! accepted)
              System.err.println("Logical Locality " + log_loc +
                                 " already present!");
          } else {
            accepted = ! isNode(my_phy_loc);

            if (! accepted)
              System.err.println("Physical Locality " + my_phy_loc +
                                 " already present!");
          }

          if (accepted) {
            node_handler = newNodeHandler (this, socket,
                                              log_loc, my_phy_loc);
            addNode(my_phy_loc, node_handler);
            ploc.setValue(my_phy_loc);
            lloc.setValue(log_loc);
          }
        }
      } catch (ClassNotFoundException ce) {
        ce.printStackTrace();
      } catch (KlavaMalformedPhyLocalityException mfl) {
          mfl.printStackTrace();
          accepted = false;
      }

      if (communicated_port)
        objOStream.writeObject(my_phy_loc);

      if (accepted)
        node_handler.setStreams(objOStream, objIStream);

      objOStream.writeObject( new Boolean( accepted ) ) ;
      objOStream.flush() ;

      if (accepted)
        node_handler.start();
    } catch ( IOException e ) {
      System.err.println( e ) ;
    }

    return accepted;
  }

  protected NodeHandler newNodeHandler(LoginHandler n, Socket s,
                                       LogicalLocality ll, PhysicalLocality pl)
      throws IOException {
    return new NodeHandler(n, node, s, ll, pl);
  }

  public boolean isNode( Locality l ) {
    return routing_table.hasRoute( l ) ;
  }

  public void addNode( Locality l, NodeHandler nh ) {
    routing_table.addRoute( l, nh ) ;
    PrintNewNode( l ) ;
  }

  public void removeNode( Locality l ) {
    routing_table.removeRoute( l ) ;
    PrintRemovedNode( l ) ;
    node.propagateRemoveLocality(l);
  }

  void removeNode( LogicalLocality ll, Locality pl ) {
    removeNode(pl);
    if (ll != null)
      node.removeFromEnv(ll);
  }

  // called when a connection is lost (stream is close)
  void lostConnection( LogicalLocality ll, Locality pl ) {
    removeNode(ll, pl);
    if (ll == null)
      node.lostConnection(pl);
    else
      node.lostConnection(ll, (PhysicalLocality)pl);
  }

  public NodeHandler getNodeHandler( Locality l ) {
    return routing_table.getNodeHandler( l ) ;
  }

  synchronized protected void Print( String s ) {
    System.out.println( "LoginHandler(" + port + ") " + s) ;
  }

  synchronized protected void PrintNewNode( Locality l ) {
    Print( "New Node at locality : " + l ) ;
  }

  synchronized protected void PrintRemovedNode( Locality l ) {
    Print( "Node at " + l + " removed." ) ;
  }
}