package Klava;

import java.io.*;

public class MessageDeliverer extends Thread {
  protected Thread sibling;
  protected Queue messageQueue ;
  protected ObjectOutputStream oObjStream ;
  protected Locality loc ; // if we are connected directly to a node
  // if null it means that we are connected to a Net
  protected boolean closing_connection = false;
  protected int verbosity = 1;

  public MessageDeliverer (Thread sibling, Queue q, ObjectOutputStream o,
                           Locality l ) {
    super ();
    this.sibling = sibling;
    messageQueue = q ;
    oObjStream = o ;
    loc = l ;
    if (sibling instanceof Node)
      verbosity = ((Node)sibling).getVerbosity();
  }

  public void run () {
    NodeMessage message = null;

    try {
      while ( true ) {
        message = (NodeMessage) messageQueue.remove() ;
        Print( "Delivering: " + message ) ;
        oObjStream.reset ();
        oObjStream.writeObject( message ) ;
        oObjStream.flush() ;
      }
    } catch (IOException ex) {
      if (message != null)
        messageQueue.addOnTop(message); // we put it back on top of the
      // queue since it probably hasn't been correctly delivered.
      if ( ! closing_connection )
      {
        // it's a stream problem
        ex.printStackTrace() ;
        if( sibling != null && sibling instanceof NetNode ) {
          if ( loc != null )
            ((NetNode)sibling).lostConnection( loc ) ;
          else
            ((NetNode)sibling).lostNetConnection ();
        }
      }
    } catch (InterruptedException ie) {
    } finally {
      System.err.println("MessageDeliverer (" +
        (sibling != null ? sibling.getName() : "null") + ") stopped.");
    }
  }

  public void close_connection()
  {
    closing_connection = true;
    try {
      oObjStream.close ();
    } catch (IOException ioe) { ioe.printStackTrace (); }
  }

  protected void Print(String s) {
    if (verbosity == 0)
      return;

    if (sibling != null)
      System.out.print("(" + sibling.getName() + ") ");

    System.out.println(s);
  }
}
