package Klava ;

import java.util.Hashtable ;
import java.util.Enumeration ;

public class MonitorableHashtable extends Hashtable
    implements KlavaMonitorable {

  protected KlavaMonitorable monitorable ;

  public MonitorableHashtable() { this( null ) ; }
  public MonitorableHashtable( KlavaMonitorable m ) {
      super() ;
      if ( m == null )
          monitorable = newMonitorable() ;
      else
          monitorable = m ;
  }

  public KlavaMonitorable newMonitorable()  // Factory Method
    { return new KlavaMonitorableDefault() ; }

  // KlavaMonitorable interface
  public void addMonitor( KlavaMonitor m )
    {
        monitorable.addMonitor( m ) ;
        updateMonitor( m ) ;
    }
  public void removeMonitor( KlavaMonitor m )
    { monitorable.removeMonitor( m ) ; }
  public void notifyAddElement( Object o )
    { monitorable.notifyAddElement( o ) ; }
  public void notifyRemoveElement( Object o )
    { monitorable.notifyRemoveElement( o ) ; }

  public void notifyRemoveAll() {
    monitorable.notifyRemoveAll();
  }

  synchronized protected void updateMonitor( KlavaMonitor m ) {
    m.removeAll() ;
    Enumeration keys = keys() ;
    String key ;
    while( keys.hasMoreElements() ) {
        key = (String) keys.nextElement() ;
        m.addElem( key + "=" + get( key ) ) ;
    }
  }

    public synchronized Object put(Object key, Object value) {
        notifyAddElement( key ) ;
        return super.put( key, value ) ;
    }

    public synchronized Object remove(Object key) {
        notifyRemoveElement( key ) ;
        return super.remove( key ) ;
    }
}
