package Klava ;

import java.io.* ;
import java.net.* ;
import java.util.Vector ;
import java.util.Hashtable ;
import java.util.Enumeration ;
import java.util.TreeSet;
import java.lang.reflect.Constructor;

import org.mikado.imc.mobility.JavaByteCodeUnMarshaller;
import org.mikado.imc.mobility.NodeClassLoader;

public class NetNode extends Thread implements NetNodeInterface {
  // ( process name, KlavaProcess )
  protected MonitorableHashtable processes ; // processes running in this node
  // ( process name, Tuple )
  protected Hashtable waitingProcesses ;
  // ( PhysLoc, NetNode )
  protected Hashtable newLocalities ; // new localities
  protected Hashtable subNodes; // nodes in the subnet
  protected Hashtable LocalitiesTable ; // localities for direct connections
  protected Environment environment ;
  protected String name ;
  protected PhysicalLocality here ; // phys loc of this node
  protected LogicalLocality self ;
  protected Queue outMessageQueue = new Queue(); // outgoing messages queue
  protected Queue inMessageQueue = new Queue(); // incoming messages queue
  protected InputStream iStream ; // for networking comunication
  protected OutputStream oStream ;
  protected ObjectInputStream objIStream ;
  protected ObjectOutputStream objOStream ;
  protected TupleSpace TS ;
  protected String netHostName ;
  protected int netPort ;
  protected InetAddress local_address ;
  protected int local_port;
  protected boolean DirectConnections = false ;
  protected NodeClassLoader classLoader = null;
  protected JavaByteCodeUnMarshaller unmarshaller = null;
  protected LocalMatcher local_matcher = null;
  protected LocalTupleSpaceMatcher local_ts_matcher = null;
  protected KlavaProcess nodeCreator ; // The process that creates this node with newloc

  protected boolean MessagesOn = false ;
  protected boolean stopped = false ;
  protected boolean closing = false ;
  protected boolean killing_processes = false;
  protected InfoMessages infoMessages ;
  protected int verbosity = 1; // verbosity of messages
  protected boolean ack_non_blocking = true; // whether non-blocking actions
  // have to be acknowledged by receiving sites
  protected boolean automatic_evaluate_localities = true;
  // whether logical localities within tuples have to be always evaluated
  protected boolean default_log_processes = true;
  // whether to keep track of classes sent along with a process
  protected boolean log_recorded_classes = false;
  // whether remote processes are accepted
  protected boolean accept_remote_processes = true;
  // whether to shut the node down when the connection to a server is lost
  protected boolean close_on_lost_net_connection = false;

  protected MessageDeliverer deliverer ;
  protected MessageReceiver receiver ;

  protected MessagePrinterFactory msg_printer_factory;
  protected TreeSet procnames_to_log = new TreeSet();
  protected TreeSet procnames_to_wait = new TreeSet();
  //protected Vector processes_to_wait = new Vector();

  // the application we are running in
  protected KlavaApplication application;

  public final static int TO_BE_CONNECTED = 0;
  public final static int CONNECTED = 1;
  public final static int NOT_CONNECTED = 2;
  protected int connection_status = TO_BE_CONNECTED;

  static private int ProcID = 0 ; // for processes names

  protected final static int defaultPort = 9999 ;

  protected LoginHandler handle_login = null;
  protected int my_port;
  protected PhysicalLocality logon_server; // the server we are logged at
  protected TupleSpace unsubscribing_nodes = new TupleSpace();
  // for notifying about unsubscriptions
  protected TupleSpace lost_connections = new TupleSpace();
  // for notifying about lost connections
  protected RoutingTable routing_table = new RoutingTable(this);
  // for nodes that want to unsubscribe
  protected boolean no_logon_server = false;
  // whether this node can connect to another server

  protected boolean locality_propagation = true;
  // whether localities in our subnet must be propagated to the server

  public NetNode( String Name, String Here, String NetHostName, int NetPort )
    throws IOException, KlavaException {
      super( "Node - " + Name ) ;
      initNode( Name, buildMyPhyLoc( Here ), NetHostName, NetPort ) ;
  }

  public NetNode( String Name, String Here, String NetHostName, Integer NetPort )
    throws IOException, KlavaException {
      super( "Node - " + Name ) ;
      initNode( Name, buildMyPhyLoc( Here ), NetHostName, NetPort.intValue() ) ;
  }

  public NetNode( String Name, PhysicalLocality Here, String NetHostName, int NetPort )
    throws IOException, KlavaException {
      super( "Node - " + Name ) ;
      initNode( Name, Here, NetHostName, NetPort ) ;
  }

  public NetNode( String loc, int my_port_, String NetHostName, int NetPort )
      throws IOException, KlavaException {
    this(loc,
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), my_port_)),
      NetHostName, NetPort);
  }

  public NetNode( String loc, int my_port ) throws IOException, KlavaException {
    this(loc, my_port, null, -1);
  }

  public NetNode( String loc, Integer my_port ) throws IOException, KlavaException {
    this(loc, my_port.intValue());
  }

  protected PhysicalLocality buildMyPhyLoc(String loc)
      throws KlavaMalformedPhyLocalityException {
    return NetUtils.buildMyPhyLoc(loc);
  }

  protected void initNode( String Name, PhysicalLocality Here, String NetHostName, int NetPort )
    throws IOException, KlavaException {
      name = Name ;
      here = Here ;
      infoMessages = newInfoMessages() ;
      infoMessages.addMonitor( new InfoMessagesMonitorDefault() ) ;
      self = new LogicalLocality( "self" ) ;
      processes = newProcTable() ;
      waitingProcesses = new Hashtable() ;
      newLocalities = new Hashtable() ;
      subNodes = new Hashtable ();
      LocalitiesTable = new Hashtable() ;
      environment = newEnvironment ( NetUtils.canonizeLocality(Here) + ".loc" );

      environment.add( self, here ) ;
      TS = newTupleSpace() ;
      TS.setNode( this ) ;
      netHostName = NetHostName ;
      netPort = NetPort ;

      local_matcher = createLocalMatcher ();
      local_ts_matcher = createLocalTupleSpaceMatcher (TS);

      classLoader = newNodeClassLoader() ;
      unmarshaller = newJavaByteCodeUnMarshaller();

      my_port = NetUtils.retrieveConnectionPortFromLocality(Here);
      setAutomaticLocalityEvaluation(false);
      handle_login = new LoginHandler(this, my_port, routing_table);
      handle_login.start();
      my_port = handle_login.getPort();
      here.setValue(new PhysicalLocality(NetUtils.createNodeAddress
          (NetUtils.getLocalIPAddress(), my_port)));
      connection_status = NOT_CONNECTED;
  }

  // factory method
  protected Environment newEnvironment( String name )
          throws KlavaMalformedPhyLocalityException
  {
    return new Environment (name);
  }

  // factory method
  protected NodeClassLoader newNodeClassLoader()
  {
    return new NodeClassLoader ();
  }
  
  protected JavaByteCodeUnMarshaller newJavaByteCodeUnMarshaller() {
  	return new JavaByteCodeUnMarshaller(classLoader);
  }

  public void addToEnv( String logLoc, String phyLoc )
          throws KlavaMalformedPhyLocalityException
  {
    addToEnv( new LogicalLocality( logLoc ),
              new PhysicalLocality( phyLoc ) ) ;
  }

  public void addToEnv( String logLoc, PhysicalLocality phyLoc )
  {
    addToEnv( new LogicalLocality( logLoc ), phyLoc ) ;
  }

  public void addToEnv( String logLoc, int phyLoc )
  {
    addToEnv( new LogicalLocality( logLoc ),
              NetUtils.createLocalPhyLocAddress(phyLoc) ) ;
  }

  public void addToEnv( LogicalLocality logLoc, PhysicalLocality phyLoc ) {
    environment.add( logLoc, phyLoc ) ;
  }

  public void addToEnv( Environment env ) {
    Enumeration en = env.keys() ;
    String l ;
    while ( en.hasMoreElements() ) {
      l = (String)en.nextElement() ;
      if ( ! l.equals(self.toString()) ) // skip self
        environment.add( new LogicalLocality(l), (PhysicalLocality)env.toPhysical(l) ) ;
    }
  }

  public void removeFromEnv(LogicalLocality loc) {
    environment.remove(loc);
  }

  boolean try_AddToEnv(LogicalLocality ll, PhysicalLocality pl) {
    return environment.try_add(ll, pl);
  }

  protected MonitorableHashtable newProcTable()
    { return new MonitorableHashtable() ; }

  protected InfoMessages newInfoMessages() { return new InfoMessages() ; }

  public void setNoLogonServer(boolean b) {
    no_logon_server = b;
  }

  public void logRoutes() {
    logRoutes(ThisLocality().toString());
  }

  public void logRoutes(String name) {
    if (msg_printer_factory != null)
      routing_table.setMessagePrinter
      (msg_printer_factory.createMessagePrinter
    ("RoutingTable-" + name, null));
  }

  public void logLoadedClasses() {
    logLoadedClasses(NetUtils.canonizeLocality(ThisLocality()));
  }

  public void logLoadedClasses(String name) {
    if (msg_printer_factory != null)
      classLoader.setMessagePrinter
        (msg_printer_factory.createMessagePrinter
    ("LoadedClasses-" + name, null));
  }

  public void setDirectConnections() throws IOException {
    DirectConnections = true ;
    addProcess(new AcceptConnNodeCoordinator());
  }

    public void setLocalMatcher( LocalMatcher matcher ) {
        local_matcher = matcher;
    }

    // factory method
    protected LocalMatcher createLocalMatcher() {
        return new LocalMatcher ();
    }

    public void setLocalTupleSpaceMatcher( LocalTupleSpaceMatcher matcher ) {
        local_ts_matcher = matcher;
        local_ts_matcher.setTupleSpace (TS);
    }

    public void setMessagePrinterFactory(MessagePrinterFactory factory) {
      msg_printer_factory = factory;
    }

    public MessagePrinterFactory getMessagePrinterFactory() {
      return msg_printer_factory;
    }

    public void setApplication(KlavaApplication app) {
      application = app;
    }

    public void setLocalityPropagation(boolean prop) {
      locality_propagation = prop;
    }

    // factory method
    protected LocalTupleSpaceMatcher
        createLocalTupleSpaceMatcher(TupleSpace ts)
    {
        return new LocalTupleSpaceMatcher (ts);
    }

  // in case notify a listener for a lost connection
  protected void lostConnection( Locality loc ) {
    _lostConnection(loc);
    Tuple lost_conn_tuple = new Tuple(loc);
    lost_connections.out(lost_conn_tuple);
  }

  // in case notify a listener for a lost connection also
  // for a logical locality
  protected void lostConnection( LogicalLocality lloc, PhysicalLocality loc ) {
    _lostConnection(loc);
    Tuple lost_conn_tuple = new Tuple(lloc, loc);
    lost_connections.out(lost_conn_tuple);
  }

  protected void _lostConnection( Locality loc ) {
    synchronized ( LocalitiesTable ) {
      Queue q = (Queue)LocalitiesTable.get( loc ) ;
      if ( q != null ) {
        Print( "Direct connection lost with " + loc ) ;
        synchronized ( q ) {
          q.notifyAll() ;
        }
        LocalitiesTable.remove( loc ) ;
      }
    }
  }

  public void setMessages( boolean on ) { MessagesOn = on ; }

  public void setVerbosity(int verb) { verbosity = verb; }
  public int getVerbosity() { return verbosity; }

  public boolean getAutomaticLocalityEvaluation() {
    return automatic_evaluate_localities;
  }

  public void setAutomaticLocalityEvaluation(boolean b) {
    automatic_evaluate_localities = b;
  }

  public void setCloseOnLostNetConnection(boolean b) {
    close_on_lost_net_connection = b;
  }

  public void setAckNonBlocking(boolean b) {
    ack_non_blocking = b;
  }

  protected TupleSpace newTupleSpace() {
    return new TupleSpace() ;
  }

  static private synchronized int nextProcID() {
    return ProcID++ ;
  }

  synchronized protected void connect( String host, int port ) throws IOException
  {
    PrintSystemOutMessage( "Connecting to " + host + ":" + port + " ...") ;
    Socket s = new Socket( host, port ) ;
    PrintSystemOutMessage( "Socket obtained..." ) ;
    iStream = s.getInputStream() ;
    oStream = s.getOutputStream() ;
    PrintSystemOutMessage( "Connected." ) ;
    local_address = NetUtils.getNodeAddress(s.getLocalAddress ());
    local_port = s.getLocalPort ();
  }

  synchronized public int getConnectionStatus()
  {
    while ( connection_status == TO_BE_CONNECTED )
      {
        try
        {
          wait ();
        }
        catch ( InterruptedException ie )
        {
          ie.printStackTrace ();
        }
      }
    return connection_status;
  }

  synchronized public void waitToBeConnected() throws InterruptedException {
    while (getConnectionStatus() != CONNECTED)
      wait();
  }

  synchronized protected void connect_login( Locality name ) throws IOException {
    try {
      boolean registered = try_login(name);
      if ( ! registered )
        throw new IOException( "Locality " + here + " already used in this net" ) ;
      PrintSystemOutMessage( "Login successful!" ) ;
    } catch ( IOException ex ) {
      try {
        ex.printStackTrace() ;
        oStream.close() ;
      } catch ( IOException ex2 ) {
        ex.printStackTrace() ;
      }
      throw ex ;
    } catch ( Exception e ) {
      e.printStackTrace() ;
      throw new IOException( "Exception! Connection aborted") ;
    }
  }

  // responsible of initializing object stream objOStream and objIStream
  protected boolean try_login(Locality loc)
                    throws IOException, ClassNotFoundException {
      boolean send_log_loc = (loc instanceof LogicalLocality);
      String proto_string;

      if (send_log_loc) {
        PrintSystemOutMessage( "Subscribe as " + loc + " ..." ) ;
        proto_string = "SUBSCRIBE";
      } else {
        PrintSystemOutMessage( "Login as " + loc + " ..." ) ;
        proto_string = "LOGIN";
      }

      objOStream = new ObjectOutputStream(oStream);

      objOStream.writeObject(proto_string);

      objOStream.writeObject(new Integer(my_port));

      if (send_log_loc)
        objOStream.writeObject(loc);

      objOStream.flush();

      objIStream = new ObjectInputStream( iStream ) ;
      PhysicalLocality my_loc = (PhysicalLocality) objIStream.readObject();
      PrintSystemOutMessage("Physical Locality: " + my_loc);
      return ((Boolean)objIStream.readObject()).booleanValue();
  }

  // package-friendly access
  // used to set the creator process when
  // creation is triggered by a newloc action
  void setCreator( KlavaProcess nodeCreator )
  {
    this.nodeCreator = nodeCreator ;
  }

  synchronized InetAddress getLocalAddress()
  {
    return local_address;
  }

  synchronized int getLocalPort()
  {
    return local_port;
  }

  String getNetHostName()
  {
    return netHostName;
  }

  int getNetPort()
  {
    return netPort;
  }

  // this ia available only in this package
  PhysicalLocality getHereLoc()
  {
    return here;
  }

  public void run()
  {
    try {
      show();
      listen();
    } catch ( Exception e ) {
      e.printStackTrace() ;
    }
  }

  public void accept(PhysicalLocality loc) {
    handle_login.accept(loc);
    propagateNewLocalityToServer(loc);
  }

  public boolean login(Locality loc) throws KlavaException {
    PhysicalLocality ploc = getPhysicalLocality(loc);
    boolean result = login_or_subscribe(ploc, here);

    if (result)
      propagateLocalities(ploc);

    return result;
  }

  public boolean login_or_subscribe(PhysicalLocality ploc, Locality my_loc)
      throws KlavaException {
    String netHostName = NetUtils.retrieveHostFromLocality(ploc);
    int netPort = NetUtils.retrieveConnectionPortFromLocality(ploc);

    try {
      synchronized (this) {
        connect_and_login(my_loc, netHostName, netPort);
        logon_server = new PhysicalLocality
        (NetUtils.createNodeAddress(netHostName, netPort));
      }
    } catch (IOException ioe) {
      ioe.printStackTrace();
      return false;
    }

    return true;
  }

  protected void propagateLocalities(Locality loc) {
    if (! locality_propagation)
      return;

    Vector locs = new Vector();
    synchronized (routing_table) {
      Enumeration localities = routing_table.getLocalities();
      while (localities.hasMoreElements()) {
        locs.addElement(localities.nextElement());
      }
    }
    propagateLocalities(loc, locs);
  }

  protected void propagateLocality(Locality server, Locality loc) {
    Vector locality = new Vector(1);
    locality.addElement(loc);
    propagateLocalities(server, locality);
  }

  protected void propagateLocalities(Locality loc, Vector locs) {
    SendMessage(ThisLocality(), loc, Thread.currentThread().getName(),
                NodeMessage.PROPAGATE_LOCALITY, locs, -1, false, false);
  }

  protected void propagateNewLocalityToServer(Locality loc) {
    if (! locality_propagation)
      return;

    synchronized (this) {
      if (logon_server != null) {
        propagateLocality(logon_server, loc);
      }
    }
  }

  protected void propagateNewLocalitiesToServer(Vector locs) {
    if (! locality_propagation)
      return;

    synchronized (this) {
      if (logon_server != null) {
        propagateLocalities(logon_server, locs);
      }
    }
  }

  protected void propagateRemoveLocality(Locality loc) {
    synchronized (this) {
      if (logon_server != null)
        propagateRemoveLocality(logon_server, loc);
    }
  }

  protected void propagateRemoveLocality(Locality server, Locality loc) {
    SendMessage(ThisLocality(), server, Thread.currentThread().getName(),
                NodeMessage.PROPAGATE_REMOVE_LOCALITY, loc, -1, false, false);
  }

  public boolean register(PhysicalLocality ploc, LogicalLocality lloc) {
    boolean result = handle_login.register(ploc, lloc);

    if (result)
      propagateNewLocalityToServer(ploc);

    return result;
  }

  public boolean subscribe(PhysicalLocality ploc, LogicalLocality lloc)
      throws KlavaException {
    boolean result = login_or_subscribe(ploc, lloc);

    if (result)
      propagateLocalities(ploc);

    return result;
  }

  public void logout(Locality loc) {
    terminateDeliverReceiver();
  }

  public void unsubscribe(Locality loc, LogicalLocality myloc)
      throws KlavaException{
    Tuple t = new Tuple(myloc);
    try {
      waitToBeConnected();
      goWaiting(t, logon_server, NodeMessage.UNSUBSCRIBE, -1, true, false);
      // what kind of response could we get?
      } catch (InterruptedException ie) {}

      terminateDeliverReceiver();
  }

  // it is a binder for loc
  public void unregister(LogicalLocality loc) {
    LogicalLocality unsub_loc = new LogicalLocality();
    PhysicalLocality source = new PhysicalLocality();
    KString proc_name = new KString();
    Tuple unsub_tuple = new Tuple(unsub_loc, source, proc_name);
    unsubscribing_nodes.in(unsub_tuple);
    removeFromEnv(unsub_loc);
    SendMessage( ThisLocality(), source, proc_name.toString(),
                 NodeMessage.UNREGISTERED, new Tuple(unsub_loc), -1, true, false);
    loc.setValue(unsub_loc);
  }

  // wait for a disconnection to take place
  public void disconnected(PhysicalLocality loc) {
    PhysicalLocality discloc = new PhysicalLocality();
    lost_connections.in(new Tuple(discloc));
    loc.setValue(discloc);
  }

  // wait for a disconnection to take place also for a logical locality
  public void disconnected(PhysicalLocality loc, LogicalLocality lloc) {
    LogicalLocality ldiscloc = new LogicalLocality();
    PhysicalLocality discloc = new PhysicalLocality();
    lost_connections.in(new Tuple(ldiscloc, discloc));
    lloc.setValue(ldiscloc);
    loc.setValue(discloc);
  }

  public void bindloc(LogicalLocality ll, PhysicalLocality pl)
      throws KlavaException {
    addToEnv(ll, pl);
  }

  public void unbindloc(LogicalLocality ll) throws KlavaException {
    removeFromEnv(ll);
  }

  // accept a direct connection
  public void acceptconn(PhysicalLocality loc) {
    ObjectInputStream is = null;
    while (is == null) {
      is = handle_login.acceptconn(loc);
      if (is != null)
        new MessageReceiver( this, inMessageQueue, is, loc ).start() ;
    }
  }

  // try to establish a directo connection to loc
  public boolean dirconnect(Locality loc) throws KlavaException {
    boolean result = false;
    PhysicalLocality ploc = getPhysicalLocality(loc);
    synchronized ( LocalitiesTable ) {
      Queue outQueue = (Queue)LocalitiesTable.get( ploc ) ;
      if ( outQueue == null ) {
        ObjectOutputStream objOStream = handle_login.dirconnect(ploc);
        result = (objOStream != null);
        if (result) {
          outQueue = new Queue() ;
          LocalitiesTable.put( ploc, outQueue ) ;
          new MessageDeliverer( this, outQueue, objOStream, ploc ).start() ;
        }
      }
    }
    return result;
  }

  protected synchronized void connect_and_login(Locality name,
          String netHostName, int netPort) throws IOException {
      connect( netHostName, netPort ) ;
      connect_login( name ) ;

      deliverer = new MessageDeliverer( this, outMessageQueue, objOStream, null ) ;
      receiver = new MessageReceiver( this, inMessageQueue, objIStream, null ) ;

      deliverer.start() ;
      receiver.start() ;
      show() ;

      connection_status = CONNECTED;

      notifyAll ();
  }

  /*
  This does not seem to work due to synchronization issues
  protected void startWaitingForProcesses() {
    if (! processes_to_wait.isEmpty()) {
      PrintSystemOutMessage ("Starting WaitForProcesses...");
      (new WaitForProcesses(this, processes_to_wait.elements())).start();
    }
  }
  */

  synchronized protected void Stop() {
    if ( ! stopped )
    {
      stopped = true ; // to avoid multiple calls

      PrintSystemOutMessage ("Releasing Resources");
      releaseResources ();
      PrintSystemOutMessage ("Node " + here + " Terminated");
      interrupt() ;
    }
  }

  // template method to be redefined by derived classes
  protected void releaseResources()
  {}

  // shows the Node graphically: to be implemented by derived classes
  protected void show() {}

  public synchronized void Close()
  {
    if ( nodeCreator != null &&
         nodeCreator != Thread.currentThread() )
    {
      throw new SecurityException( "Only " + nodeCreator + "is allowed to close this node" ) ;
    }
    if ( ! closing )
    {
      closing = true;
      closeConnection ();
      terminateNode ();
    }
  }

  protected void closeConnection() {
    connection_status = NOT_CONNECTED;

    PrintSystemOutMessage( "Closing connection..." ) ;

    try {
      if (handle_login != null)
        handle_login.stop_accept();
    } catch (IOException ioe) {
      ioe.printStackTrace();
    }

    routing_table.release_handlers();
    removeNewLocalities ();
    removeSubNodes ();

    PrintSystemOutMessage( "Connection Closed" ) ;
  }

  void terminateDeliverReceiver() {
    // receiver and deliverer must be checked, since the node might be
    // stopped before they are actually created.
    if (receiver != null)
      receiver.close_connection ();

    if (deliverer != null) {
      deliverer.close_connection ();
      deliverer.interrupt ();
    }
  }

  protected void lostNetConnection()
  {
    connection_status = NOT_CONNECTED;
    PrintSystemOutMessage ("Lost connection to Net");

    terminateDeliverReceiver ();

    if (close_on_lost_net_connection) {
      removeNewLocalities ();
      removeSubNodes ();

      Close ();
    }
  }

  protected void terminateNode()
  {
    killProcesses ();
    terminateDeliverReceiver ();

    Stop ();
  }

  protected void killProcesses()
  {
    if ( killing_processes )
      return ;

    killing_processes = true;

    synchronized( processes )
    {
      Enumeration en = processes.elements() ;
      while ( en.hasMoreElements() )
        killProcess( ( KlavaProcess ) en.nextElement() ) ;
    }
  }

  // this is brutal, but we cannot wait!
  protected void killProcess( KlavaProcess P )
  {
    PrintSystemOutMessage ("Killing process " + P.getName ());
    P.stop ();
  }

  protected void removeNewLocalities ()
  {
    synchronized ( newLocalities )
    {
      if (newLocalities.isEmpty())
        return;

      PrintSystemOutMessage ("Removing created localities");

      Enumeration en = newLocalities.elements ();
      while ( en.hasMoreElements () )
        removeNewLocality ( (NetNode)en.nextElement () );
    }
  }

  protected void removeNewLocality( NetNode n )
  {
    PrintSystemOutMessage ("Removing locality " + n);
    n.Close ();
    try {
      n.join() ;
    } catch (InterruptedException e) {}
  }

  protected void removeSubNodes ()
  {
    synchronized ( subNodes )
    {
      if (subNodes.isEmpty())
        return;

      PrintSystemOutMessage ("Removing sub nodes");

      Enumeration en = subNodes.elements ();
      while ( en.hasMoreElements () )
        removeSubNode ( (NetNode)en.nextElement () );
    }
  }

  protected void removeSubNode( NetNode n )
  {
    PrintSystemOutMessage ("Removing sub node" + n);
    n.Close ();
    try {
      n.join();
    } catch (InterruptedException e) {}
  }

  protected void listen() throws IOException, KlavaException {
    NodeMessage message ;
    KlavaProcess process ;

    PrintEnvironment() ;
    Print( "Starting listening..." ) ;

    while ( true ) {
      try {
        message = ( NodeMessage )inMessageQueue.remove() ;
        PrintMessage( "Received: " + message ) ;
      } catch (InterruptedException ie) {
        ie.printStackTrace();
        return;
      } catch ( Exception e ) {
        PrintSystemErrMessage( "Node Exception :" + e ) ;
        e.printStackTrace() ;
        continue ;
      }

      PrintAction( "", message.OpCode, message.Content,
                   message.Dest, " From " + message.Source ) ;

      if (! here.equals(message.Dest)) {
        handleForeignMessage(message);
        continue;
      }

      switch( message.OpCode ) {
      case NodeMessage.OUT :
        if (message.ack_requested)
          sendResponse(message, NodeMessage.RESPONSEOUT);
        handleOUT( message ) ;
        break ;
      case NodeMessage.IN :
        handleIN( message ) ;
        break ;
      case NodeMessage.READ :
        handleREAD( message ) ;
        break ;
      case NodeMessage.EVAL :
        handleEVAL( message ) ;
        break ;
      case NodeMessage.RESPONSE :
      case NodeMessage.RESPONSEIN :
      case NodeMessage.RESPONSEOUT :
      case NodeMessage.RESPONSEEVAL :
      case NodeMessage.NOPHYSLOC :
      case NodeMessage.TUPLE_ABSENT :
        handleRESPONSE( message ) ;
        break ;
      case NodeMessage.EVALUATE_LOCALITY:
        (new EvaluateLocalityWaitingThread
          (this, message)).start();
        break;
      case NodeMessage.RESPONSE_LOCALITY:
        try {
          wakeUpProcess( message, message.OpCode ) ;
        } catch (KlavaException ke) {
          ke.printStackTrace();
        }
        break;
      case NodeMessage.UNSUBSCRIBE:
        LogicalLocality loc =
          ((LogicalLocality)((Tuple)message.Content).getItem(0));
        unsubscribing_nodes.out( new Tuple(loc, message.Source,
          new KString(message.ProcessName)));
        break;
      case NodeMessage.UNREGISTERED:
        try {
          wakeUpProcess( message, message.OpCode ) ;
        } catch (KlavaException ke) {
          ke.printStackTrace();
        }
        break;
      case NodeMessage.PROPAGATE_LOCALITY:
        try {
          Vector locs = (Vector) message.Content;
          Enumeration en = locs.elements();
          Locality subloc;
          while (en.hasMoreElements()) {
            subloc = (Locality) en.nextElement();
            if (! routing_table.addRoute(subloc, message.Source))
              PrintSystemOutMessage ("Failed adding route " + subloc +
                " -> " + message.Source);
          }
          propagateNewLocalitiesToServer(locs);
        } catch (ClassCastException ce) {
          ce.printStackTrace();
        }
        break;
      case NodeMessage.PROPAGATE_REMOVE_LOCALITY:
        try {
          Locality subloc = (Locality) message.Content;
          routing_table.removeRoute(subloc);
          propagateRemoveLocality(subloc);
        } catch (ClassCastException ce) {
          ce.printStackTrace();
        }
        break;
      default:
        unhandledMessage( message );
      }
    }
  }

  protected void unhandledMessage(NodeMessage message) {}

  // called if the message is not destined to this node
  protected void handleForeignMessage(NodeMessage message) {
    PrintSystemOutMessage("Forwarding message: " + message);
    // forward this message to the right recipient
    enqueueMessage(message);
  }

  protected void handleOUT( NodeMessage message ) throws KlavaException {
    TS.out( ( Tuple ) message.Content ) ;
  }

  protected void sendResponse(NodeMessage message, int OpCode) {
    sendResponse( message, OpCode, new Tuple () ) ;
  }

  protected void sendResponse(NodeMessage message, int OpCode, Tuple t) {
    SendMessage( here, message.Source, message.ProcessName,
                 OpCode, t ) ;
  }

  protected void handleIN( NodeMessage message ) throws KlavaException {
    Tuple t = ( Tuple ) message.Content ;
    if ( TS.in_nb( t ) ) {
      SendMessage( here, message.Source, message.ProcessName,
                   NodeMessage.RESPONSEIN, t ) ;
    } else {
      if (message.blocking)
        (new InReadThread( this, message )).start() ;
      else
        SendMessage( here, message.Source, message.ProcessName,
                     NodeMessage.TUPLE_ABSENT, t );
    }
  }

  protected void handleREAD( NodeMessage message ) throws KlavaException {
    Tuple t = ( Tuple ) message.Content ;
    if ( TS.read_nb( t ) ) {
      SendMessage( here, message.Source, message.ProcessName,
                   NodeMessage.RESPONSE, t ) ;
    } else {
      if (message.blocking)
        (new InReadThread( this, message )).start() ;
      else
        SendMessage( here, message.Source, message.ProcessName,
                     NodeMessage.TUPLE_ABSENT, t );
    }
  }

  protected void handleEVAL( NodeMessage message ) throws KlavaException {
    if (message.Content instanceof Tuple) {
      if (accept_remote_processes) {
        Tuple t = (Tuple)message.Content;

        handleRemoteProcess( (KlavaProcessPacket)t.getItem(0), false ) ;
        sendResponse(message, NodeMessage.RESPONSEEVAL);
      } else {
        sendResponse(message, NodeMessage.RESPONSEEVAL,
                     new Tuple(
                     new KlavaRemoteProcessNotAllowed(here.toString())));
      }
    } else {
      PrintSystemErrMessage("Invalid format for content: " +
        message.Content);
      sendResponse(message, NodeMessage.RESPONSEEVAL,
                     new Tuple(
                     new KlavaRemoteProcessNotAllowed(
                     message.Content.getClass().toString())));
    }
  }

  protected void handleRESPONSE( NodeMessage message ) throws KlavaException {
    wakeUpProcess( message, message.OpCode ) ;
  }

  protected void handleRemoteProcess( KlavaProcessPacket pack, boolean block )
      throws KlavaException {
    if (! accept_remote_processes)
      throw new KlavaRemoteProcessNotAllowed(here.toString());

    NodeRemoteProcessHandler remProcHandl =
        new DefaultNodeRemoteProcessHandler(this, pack, unmarshaller);
    synchronized( remProcHandl ) {
      remProcHandl.start() ;
      if ( block ) {
        try {
          remProcHandl.wait() ;
        } catch ( InterruptedException ie ) {
          ie.printStackTrace() ;
        }
      }
    }
  }

  protected void wakeUpProcess( NodeMessage message, int OpCode ) throws KlavaException {
    Tuple t  ;

    synchronized ( waitingProcesses ) {
      if ( waitingProcesses.containsKey( message.ProcessName ) ) {
        t = ( Tuple ) waitingProcesses.get( message.ProcessName ) ;
        KlavaProcess requester =
            (KlavaProcess) processes.get (message.ProcessName);
        // requester will be null when a blocking operation has been performed
        // from within the node itself

        synchronized( t ) {
            Tuple received = ( Tuple ) message.Content;

            if (wakeUpProcess(t, received, message, OpCode)) {
              waitingProcesses.remove( message.ProcessName ) ;
              t.notifyAll();
              return;
            }

            if ( OpCode == NodeMessage.RESPONSEOUT ||
                 OpCode == NodeMessage.RESPONSEEVAL ||
                 OpCode == NodeMessage.NOPHYSLOC) {

                if (OpCode == NodeMessage.NOPHYSLOC ||
                    OpCode == NodeMessage.RESPONSEEVAL)
                  t.setMatched(received);

                t.notify ();
                waitingProcesses.remove( message.ProcessName ) ;
            } else if ( local_matcher.localMatch (t, received, requester) ) {
                PrintMessage( "waking up " + message.ProcessName ) ;
                waitingProcesses.remove( message.ProcessName ) ;
                if (OpCode != NodeMessage.TUPLE_ABSENT) {
                  if (! isException(received))
                    t.copy_tuple( received ) ;
                  else
                    t.setMatched(received);
                } else
                  t.setMatched(null); // indicate tuple absent
                t.notifyAll() ;
            } else {
                putTupleBack (message);
                t.updateAlreadyRetrieved ((Tuple) message.Content);
                SendMessage( here, (PhysicalLocality)message.Source,
                             message.ProcessName,
                             ( OpCode == NodeMessage.RESPONSEIN ?
                               NodeMessage.IN :
                               NodeMessage.READ ), t ) ;
                // FIXME: do not assume that if not IN then READ
            }
        }
      } else {
        PrintSystemErrMessage( message.ProcessName + " doesn't exist!" ) ;
        putTupleBack (message);

        // check if it failed because the process had already been removed
        if ( waitingProcesses.containsKey( message.ProcessName ) )
          waitingProcesses.remove( message.ProcessName ) ;
      }
    }
  }

  // this can be redefined by subclasses in order to deal with special
  // message's response
  protected boolean wakeUpProcess(Tuple t, Tuple received,
        NodeMessage message, int opCode) {
    switch (opCode) {
      case NodeMessage.RESPONSE_LOCALITY:
        t.setItem(0, ((Tuple)message.Content).getItem(0));
        return true;
      case NodeMessage.UNREGISTERED:
        return true;
      default:
        return false;
    }
  }

  protected boolean isException(Tuple t) {
    return (t.length() > 0 && t.getItem(0) instanceof KlavaException);
  }

  protected KlavaException getException(Tuple t) {
    return (KlavaException) t.getItem(0);
  }

  protected void rethrowTupleException(Tuple t) throws KlavaException {
    throw (KlavaException)t.getItem(0);
  }

  protected void putTupleBack(NodeMessage message) throws KlavaException {
    if ( message.OpCode == NodeMessage.RESPONSEIN ) // back to the sender
      SendMessage( here, (PhysicalLocality)message.Source,
                   Thread.currentThread().getName(), NodeMessage.OUT,
                   ((Tuple)message.Content).getMatched (), -1, false, false ) ;
    // the put back must not wait for an ack otherwise we deadlock.
  }

  public void out( Tuple t ) throws KlavaException {
    out( t, self ) ;
  }
  public void out( Tuple t, Locality l ) throws KlavaException {
    PrintAction( "", NodeMessage.OUT, t, l, "" ) ;
    PhysicalLocality pl = getPhysicalLocality( l ) ;
    // evaluates localities and processes
    evaluateTuple( t, false ) ;
    PrintAction( ">", NodeMessage.OUT, t, pl, "" ) ;
    if ( pl == null ) {
      Print( "Locality " + l + " not found!" ) ;
      return ;
    }

    NetNode local_node = getLocalNode(pl) ;
    if (local_node == null) {
      String procName = Thread.currentThread().getName() ;
      if (! ack_non_blocking)
        SendMessage( here, pl, procName, NodeMessage.OUT, t, -1, false, true ) ;
      else
        goWaiting( t, pl, NodeMessage.OUT, -1, false, true );
    } else
      handleLocalNode( local_node, t, NodeMessage.OUT, -1, true );
  }

  // the version with _ is used by InReadThread
  public void _in( Tuple t, long timeout ) throws KlavaTimeOutException {
    TS.in( t, timeout ) ;
  }
  public void in( Tuple t ) throws KlavaException {
    ReadIn( t, self, NodeMessage.IN, -1, true ) ;
  }
  public void in( Tuple t, Locality l ) throws KlavaException {
    ReadIn( t, l, NodeMessage.IN, -1, true ) ;
  }
  public boolean in_nb( Tuple t, Locality l ) throws KlavaException {
    return ReadIn( t, l, NodeMessage.IN, -1, false );
  }
  // time out versions
  public void in( Tuple t, long TimeOut )
    throws KlavaException {
      ReadIn( t, self, NodeMessage.IN, TimeOut, true ) ;
  }
  public void in( Tuple t, Locality l, long TimeOut )
    throws KlavaException {
      ReadIn( t, l, NodeMessage.IN, TimeOut, true ) ;
  }
    public boolean in_t(Tuple t, Locality l, long TimeOut)
	throws KlavaException
    {
	try {
	    in( t, l, TimeOut);
	    return true;
	} catch (KlavaTimeOutException e) {
	    return false;
	}
    }

  public void _read( Tuple t, long timeout ) throws KlavaTimeOutException {
    TS.read( t, timeout ) ;
  }
  public void read( Tuple t ) throws KlavaException {
    ReadIn( t, self, NodeMessage.READ, -1, true ) ;
  }
  public void read( Tuple t, Locality l ) throws KlavaException {
    ReadIn( t, l, NodeMessage.READ, -1, true ) ;
  }
  public boolean read_nb( Tuple t, Locality l ) throws KlavaException {
    return ReadIn( t, l, NodeMessage.READ, -1, false );
  }
  // time out versions
  public void read( Tuple t, long TimeOut )
    throws KlavaException {
      ReadIn( t, self, NodeMessage.READ, TimeOut, true ) ;
  }
  public void read( Tuple t, Locality l, long TimeOut )
    throws KlavaException {
      ReadIn( t, l, NodeMessage.READ, TimeOut, true ) ;
  }
    public boolean read_t(Tuple t, Locality l, long TimeOut)
	throws KlavaException
    {
	try {
	    read(t, l, TimeOut);
	    return true;
	} catch (KlavaTimeOutException e) {
	    return false;
	}
    }

  protected boolean ReadIn( Tuple t, Locality l, int OpCode, long TimeOut,
                            boolean blocking )
    throws KlavaException {

      PrintAction( "", OpCode, t, l, "" ) ;
      PhysicalLocality pl = getPhysicalLocality( l ) ;
      // evaluates only processes
      evaluateTuple( t, true ) ;
      PrintAction( "> ", OpCode, t, pl, "" ) ;
      boolean result = false;

      try {
        NetNode local_node = getLocalNode (pl);
        if (local_node == null)
          result = goWaiting( t, pl, OpCode, TimeOut, blocking, false ) ;
        else
          result = handleLocalNode( local_node, t, OpCode, TimeOut, blocking );

        PrintAction( "-> ", OpCode, t, pl, (result ? "" : " FAILED!") ) ;
      } catch ( KlavaTimeOutException eto ) {
        PrintAction( "TIME OUT: ", OpCode, t, pl, "" ) ;
        throw eto ;
      }

      Thread currentThread = Thread.currentThread() ;
      if ( currentThread instanceof KlavaProcess ) {
        KlavaProcess thisProcess = (KlavaProcess)currentThread ;
        if( thisProcess.mutex != null ) {
          synchronized( thisProcess.mutex ) {
            if( thisProcess.raceWinner ) {
              thisProcess.otherProcess.raceWinner = false ;
              removeProcess( thisProcess.otherProcess ) ;
            } else {
              if ( OpCode == NodeMessage.IN ) {
                // back to the owner
                out( t, pl ) ;
              }
              removeProcess( thisProcess ) ;
            }
          }
        }
      }

      return result;
  }

  protected void PrintAction( String pre, int OpCode, Object t, Locality l, String post ) {
    if (verbosity == 0)
      return;

    if ( OpCode == NodeMessage.IN )
      Print( pre + "IN( " + t + " )@" + l + post) ;
    else if ( OpCode == NodeMessage.READ )
      Print( pre + "READ( " + t + " )@" + l + post ) ;
    else if ( OpCode == NodeMessage.RESPONSE ||
              OpCode == NodeMessage.RESPONSEIN ||
              OpCode == NodeMessage.NOPHYSLOC )
      Print( pre + "RESPONSE( " + t + " )@" + l + post ) ;
    else if ( OpCode == NodeMessage.EVAL )
      Print( pre + "EVAL( " + t + " )@" + l + post ) ;
    else if ( OpCode == NodeMessage.OUT )
      Print( pre + "OUT( " + t + " )@" + l + post ) ;
  }

  protected boolean handleLocalNode( NetNode n, Object t,
                                     int OpCode, long TimeOut,
                                     boolean blocking )
    throws KlavaException {
      switch ( OpCode ) {
        case NodeMessage.OUT :
          n.TS.out( (Tuple)t ) ;
          break ;
        case NodeMessage.IN :
        case NodeMessage.READ :
            Tuple tuple = (Tuple) t;
            return handle_local_retrieve (n, tuple, OpCode, TimeOut, blocking);
        case NodeMessage.EVAL :
          try {
            if ( t instanceof KlavaProcess )
            {
              KlavaProcess P = (KlavaProcess) t;
              if (P.getMigrationStatus() == KlavaProcess.MIGRATING)
                  P = P.duplicate();

              n.addProcess( P ) ;
            }
            else if ( t instanceof KlavaProcessVar )
              n.handleRemoteProcess( ((KlavaProcessVar)t).getProcPacket(), false ) ;
          } catch ( IllegalThreadStateException e ) {
            e.printStackTrace() ;
          }
          break ;
        }
      return true ;
  }

  // return the node if it's a new locality created by this node or if
  // it's a node in the sub net of this node
  protected NetNode isLocalNode(PhysicalLocality n) {
    NetNode res = (NetNode)newLocalities.get(n);
    if (res == null)
      res = (NetNode)subNodes.get(n);

    return res;
  }

  protected NetNode getLocalNode(PhysicalLocality n) {
    if ( n.equals( here ) )
      return this;

    return isLocalNode( n );
  }

    protected boolean handle_local_retrieve(NetNode n, Tuple tuple, int opCode,
                                            long Timeout, boolean blocking)
        throws KlavaException {
        boolean succeed = false;

        KlavaProcess requester = null;
        Thread curr_thread = Thread.currentThread ();
        if (curr_thread instanceof KlavaProcess)
          requester = (KlavaProcess) curr_thread;
        // the current thread could be a Node

        while (! succeed) {
            if (opCode == NodeMessage.IN) {
                if (Timeout < 0) {
                    succeed = n.local_ts_matcher.in (tuple, blocking,
                                                     requester);
                } else {
                    succeed = n.local_ts_matcher.in (tuple, Timeout,
                                                     requester);
                }
                if (! succeed && blocking)
                    put_local_tuple_back (n, tuple);
                // blocking must be tested: if it's blocking, no tuple
                // was withdrawn, and it may be dangerous to call
                // put_local_tuple_back: Tuple.matched could non null!
            } else { // it is a READ
                if (Timeout < 0) {
                    succeed = n.local_ts_matcher.read (tuple, blocking,
                                                       requester);
                } else {
                    succeed = n.local_ts_matcher.read (tuple, Timeout,
                                                       requester);
                }
            }

            if (! succeed ) {
                tuple.resetOriginalTemplate ();
                if ( local_ts_matcher.should_retry() )
                  System.err.println ("Next try: " + tuple);
                else
                  return false;
            }
        }

        return succeed;
    }

    protected void put_local_tuple_back(NetNode n, Tuple t)
        throws KlavaException {
        Tuple original = t.getMatched ();
        if ( original != null ) {
          n.out (original);
        }
    }

  protected boolean goWaiting( Tuple t, PhysicalLocality pl, int OpCode,
                               long TimeOut, boolean blocking, boolean ack )
    throws KlavaException {
      String procName = Thread.currentThread().getName() ;
      waitingProcesses.put( procName, t ) ;
      synchronized ( t ) {
        SendMessage( here, pl, procName, OpCode, t, TimeOut, blocking, ack ) ;
        try {
          if ( TimeOut < 0 ) {
            t.wait() ;
          } else {
            long startTime = System.currentTimeMillis() ;
            t.wait( TimeOut ) ;
            long elapsedTime = System.currentTimeMillis() - startTime ;
            synchronized( waitingProcesses ) {
              if( waitingProcesses.get( procName ) != null ) {
                // if it's still there it means: time out
                waitingProcesses.remove( procName ) ;
                // but do not trust elapsedTime only: Java is not reliable
                // for this, i.e. elapsedTime could be < TimeOut even
                // the specified time expired; thus check also the
                // condition t.getMatched() == null
                if (elapsedTime >= TimeOut || t.getMatched() == null)
                  throw new KlavaTimeOutException( "tuple : " + t ) ;
              }
            }
          }

          Thread thread = Thread.currentThread() ;
          Tuple matched = t.getMatched();
          if (matched != null && isException(matched) &&
              thread instanceof KlavaProcess &&
              (((KlavaProcess)thread).getNotifyOnNonExistentLocalities() ||
              OpCode == NodeMessage.EVAL)
              )
            throw getException(matched);

        } catch ( InterruptedException e ) {
          e.printStackTrace() ;
        }
      }
      return (t.getMatched() != null &&
              ! isException(t.getMatched())); // otherwise tuple absent
  }

  protected void evaluateTuple( Tuple t, boolean onlyProcesses )
      throws KlavaException {
    Environment env = (automatic_evaluate_localities ? environment : null);
    // if localities are not to be evaluated automatically we do not
    // include the environment in the process packet

      synchronized( t ) {
        Object TupleItem ;
        for ( int i = 0 ; i < t.length() ; i++ ) {
          TupleItem = t.getItem( i ) ;
          if ( TupleItem instanceof LogicalLocality &&
               automatic_evaluate_localities) {
            PhysicalLocality pl = getPhysicalLocality( (LogicalLocality)TupleItem ) ;
            // it's better not to put null in a tuple space
            if ( pl != null )
              t.setItem( i, pl ) ;
            else
              t.setItem( i, "" ) ;
          } else if ( TupleItem instanceof KlavaProcess ) {
            KlavaProcess P = (KlavaProcess)TupleItem ;
            Thread currentProc = Thread.currentThread() ;
            try {
              t.setItem(i, new KlavaProcessVar(
                  createProcessPacket(P, env,
                                      (currentProc instanceof KlavaProcess ?
                                       ( (KlavaProcess) currentProc).
                                       getEnvironment() : null))));
            } catch (IOException e) {
              throw new KlavaException(e);
            }
          } else if ( TupleItem instanceof KlavaProcessVar ) {
            KlavaProcessVar pV = (KlavaProcessVar)TupleItem ;
            KlavaProcess P = pV.getProc() ;
            Thread currentProc = Thread.currentThread() ;
            if ( P != null ) {
              try {
                pV.setProcPack(createProcessPacket(P, env,
                                                   (currentProc instanceof
                                                    KlavaProcess ?
                                                    ( (KlavaProcess) currentProc).
                                                    getEnvironment() : null)));
              } catch (IOException e) {
                throw new KlavaException(e);
              }
            }
          } else if ( TupleItem instanceof Tuple ) // recursive call
	    evaluateTuple( (Tuple)TupleItem, onlyProcesses ) ;
        }
      }
  }

  public KlavaProcess closeProcess(KlavaProcess P) {
    P.setEnvironment(environment);
    return P;
  }

  public KlavaProcessVar closeProcess(KlavaProcessVar P) {
    P.setEnvironment(environment);
    return P;
  }

  public void eval( KlavaProcess P ) throws KlavaException {
    eval( P, self ) ;
  }
  public void eval( KlavaProcess P, Locality loc ) throws KlavaException {
    PhysicalLocality pl = getPhysicalLocality( loc ) ;
    if ( pl == null )
      Print( "Locality " + loc + " not found!" ) ;
    else {
      NetNode local_node = getLocalNode(pl);
      if (local_node != null) {
        handleLocalNode( local_node, P, NodeMessage.EVAL, -1, true );
      } else {
        Thread currentThread = Thread.currentThread() ;
        String procName ;
        if ( currentThread instanceof KlavaProcess ) {
          KlavaProcess proc = (KlavaProcess)currentThread ;
          // only procName is to be sent
          procName = proc.KlavaProcName() ;
        } else {
          procName = currentThread.getName() ;
        }

        KlavaProcessPacket pack = null;
        try {
          pack = createProcessPacket(P, null, null);
        } catch (IOException e) {
          throw new KlavaException(e);
        }

        if (! ack_non_blocking)
          SendMessage( here, pl, procName, NodeMessage.EVAL,
            new Tuple(pack), -1, false, true );
        else
          goWaiting( new Tuple(pack), pl, NodeMessage.EVAL,
            -1, false, true );
      }
    }
  }

  public void eval( KlavaProcessVar P ) throws KlavaException {
    eval( P, self ) ;
  }
  public void eval( KlavaProcessVar P, Locality loc ) throws KlavaException {
    KlavaProcess proc = P.getProc() ;
    if ( proc != null ) {
      eval( proc, loc ) ;
      return ;
    }
    PhysicalLocality pl = getPhysicalLocality( loc ) ;
    if ( pl == null )
      Print( "Locality " + loc + " not found!" ) ;
    else {
      NetNode local_node = getLocalNode(pl);
      if (local_node != null) {
        handleLocalNode( local_node, P, NodeMessage.EVAL, -1, true );
      } else {
        Thread currentThread = Thread.currentThread() ;
        String procName ;
        if ( currentThread instanceof KlavaProcess ) {
          proc = (KlavaProcess)currentThread ;
          // only procName is to be sent
          procName = proc.KlavaProcName() ;
        } else {
          procName = currentThread.getName() ;
        }
        if (! ack_non_blocking)
          SendMessage( here, pl, procName, NodeMessage.EVAL,
            new Tuple (P.getProcPacket()),
            -1, false, true );
        else
          goWaiting( new Tuple(P.getProcPacket()), pl, NodeMessage.EVAL,
            -1, false, true );
      }
    }
  }

  protected KlavaProcessPacket createProcessPacket( KlavaProcess P, Environment env, Vector procEnv )
  throws IOException {
    if (P.hasToDeliverCode())
      if (log_recorded_classes && msg_printer_factory != null)
        P.setMessagePrinter
            (msg_printer_factory.createMessagePrinter(P.KlavaProcName(), null));

    return new KlavaProcessPacket( P, env, procEnv ) ;
  }

  public PhysicalLocality newloc(NodeCoordinator P, int port,
                                 String classname) throws KlavaException {
    try {
      NetNode newNode;
      if (classname != null) {
        Class c = Class.forName(classname);
        Class parameters[] = build_params_for_NewNode();
        Constructor cons = c.getConstructor(parameters);
        Object args[] = build_args_for_NewNode(P, port, classname);
        newNode = (NetNode) cons.newInstance(args);
      } else {
        PhysicalLocality newp = new PhysicalLocality(NetUtils.createNodeAddress
            (NetUtils.getLocalIPAddress(), port));
        newNode = new NetNode( "NetNode_" + newp, newp, netHostName, netPort ) ;
      }

      PhysicalLocality newl = newNode.ThisLocality();
      newNode.setApplication(application);
      newLocalities.put( newl, newNode ) ;

      Thread currentThread = Thread.currentThread() ;
      if ( currentThread instanceof KlavaProcess ) {
        newNode.setCreator( ( KlavaProcess ) currentThread ) ;
      }
      newNode.start() ;

      if (P != null)
        newNode.addProcess(P);

      return newl;
    } catch (Exception ioe) {
      throw new KlavaException(ioe.getMessage());
    }
  }

  protected Object[] build_args_for_NewNode(NodeCoordinator P, int port,
      String classname) {
    Object[] args = {"Node", new Integer(port)};
    return args;
  }

  protected Class[] build_params_for_NewNode() {
    Class parameters[] = {java.lang.String.class, java.lang.Integer.class};
    return parameters;
  }

  public PhysicalLocality newloc(NodeCoordinator P) throws KlavaException {
    return newloc(P, -1, null);
  }

  public PhysicalLocality newloc() throws KlavaException {
    return newloc(null, -1, null ) ;
  }

  public PhysicalLocality newloc(int port) throws KlavaException {
    return newloc(null, port, null ) ;
  }

  public void removeloc( Locality loc ) throws KlavaException {
    NetNode node = ( NetNode ) newLocalities.get( loc ) ;
    if ( node != null ) {
     Print( "Removing locality " + loc ) ;
     node.Close() ;
     newLocalities.remove( loc ) ;
    }
  }

  // add a node in the subnet of this node
  public void addSubNode(NetNode n) {
    synchronized(subNodes) {
      subNodes.put(n.getHereLoc(), n);
    }
    addProcess(new AcceptNodeCoordinator(true));
    n.start();
    try {
      n.waitToBeConnected();
    } catch (InterruptedException ie) {
      ie.printStackTrace();
    }
  }

  public NetNode isSubNode(Locality loc) {
    synchronized(subNodes) {
      return (NetNode) subNodes.get(loc);
    }
  }

  public void removeSubNode(Locality loc) {
    synchronized (subNodes) {
      subNodes.remove(loc);
    }
  }

  public void requestLocality( Locality l, Tuple t )
    throws KlavaException {
      String procName = Thread.currentThread().getName() ;

      waitingProcesses.put( procName, t ) ;
      NodeMessage mesg = new NodeMessage( here, l, procName,
                                          NodeMessage.REQUESTLOC, t,
                                          -1, true, false ) ;
      synchronized ( t ) {
	enqueueMessage( mesg ) ;
        try {
          t.wait() ;
          Tuple matched = t.getMatched();
          // if we received an exception it is stored in the matched field
          if ( matched != null && isException (matched) )
	    throw (KlavaException)matched.getItem(0) ;
        } catch ( InterruptedException e ) {
          e.printStackTrace() ;
        }
      }
  }

  public void or( KlavaProcess P1, KlavaProcess P2 ) {
    // any object will do
    Object mutex = new Integer(nextProcID()) ;
    synchronized ( mutex ) {
      P1.mutex = mutex ;
      P2.mutex = mutex ;
      P1.otherProcess = P2 ;
      P2.otherProcess = P1 ;
      addProcess( P1 ) ;
      addProcess( P2 ) ;
      try {
        mutex.wait() ;
      } catch ( InterruptedException e ) {
        e.printStackTrace() ;
      }
    }
  }

  public void procCall( Callable P ) throws KlavaException {
    if ( P instanceof KlavaProcess ) {
      startProcAndWait( (KlavaProcess)P ) ;
    } else if ( P instanceof KlavaProcessVar ) {
      KlavaProcessVar pv = (KlavaProcessVar)P ;
      KlavaProcess proc = pv.getProc() ;
      if ( proc != null )
        startProcAndWait( proc ) ;
      else {
        Thread curThread = Thread.currentThread() ;
        if ( curThread instanceof KlavaProcess ) {
          KlavaProcess curProc = (KlavaProcess)curThread ;
          pv.setMutex( curProc.mutex, curProc.otherProcess ) ;
        }
        handleRemoteProcess( pv.getProcPacket(), true ) ;
      }
    }
  }

  protected void startProcAndWait( KlavaProcess P ) {
    synchronized ( P ) {
      try {
        Thread curThread = Thread.currentThread() ;
        addProcess( P ) ;
        P.wait() ;
      } catch ( InterruptedException e ) {
        e.printStackTrace() ;
      }
    }
  }


  protected NetNode createNewNode( String Name, PhysicalLocality Here, String NetHostName, int NetPort )
    throws IOException, KlavaException {
      return (new NetNode( Name, Here, NetHostName, NetPort )) ;
  }

  protected void SendMessage( Locality source, Locality dest,
                              String processname, int opcode,
                              Object content ) {
    SendMessage( source, dest, processname, opcode, content, -1, true, false ) ;
  }

  protected void SendMessage( Locality source, Locality dest,
                              String processname, int opcode, Object content,
                              long timeout, boolean block, boolean ack ) {
    NodeMessage mesg = new NodeMessage( // duplicate is for a Java Bug
      source, (PhysicalLocality)dest.duplicate(), processname, opcode,
      content, timeout, block, ack ) ;
    PrintMessage( "Sending : " + mesg ) ;
    if ( ! DirectConnections || dest.equals(here) ) {
      enqueueMessage( mesg ) ;
    } else {
      Queue outQueue ;
      synchronized ( LocalitiesTable ) {
        outQueue = (Queue)LocalitiesTable.get( dest ) ;
        if ( outQueue == null ) {
          Print( "Need to create connection to " + dest );
          outQueue = establishDirConn(dest);
        }
      }
      outQueue.add( mesg ) ;
    }
  }

  // called when a direct connection has to be establish if
  // automatic direct connections are enabled.
  protected Queue establishDirConn(Locality dest) {
    Queue outQueue = null;
    boolean result = false;

    try {
      result = dirconnect(dest);
      if (result)
        outQueue = (Queue)LocalitiesTable.get(dest);
    } catch (KlavaException ke) {
      ke.printStackTrace();
    }

    if (! result) {
      Print( "Revert to indirect connection for " + dest );
      outQueue = new Queue();
      new MessageDeliverer( this, outQueue, objOStream, dest ).start() ;
    }

    return outQueue;
  }

  protected void enqueueMessage(NodeMessage msg) {
    Locality dest = msg.Dest;
    NodeHandler nhandler = handle_login.getNodeHandler(dest);
    if (nhandler != null)
      nhandler.Enqueue(msg);
    else {
      if (LocalitiesTable.containsKey(dest))
        ((Queue) LocalitiesTable.get(dest)).add(msg);
      else
        outMessageQueue.add(msg);
    }
  }

  boolean deliverToServer(NodeMessage msg) {
    if (no_logon_server)
      return false;

    outMessageQueue.add(msg);
    return true;
  }

  protected void enqueueInMessage( NodeMessage msg ) {
    inMessageQueue.add(msg);
  }

  public PhysicalLocality getPhysicalLocality( Locality loc )
    throws KlavaLogicalLocalityException {
      if ( loc instanceof PhysicalLocality )
        return (PhysicalLocality) loc ;

      Thread currentThread = Thread.currentThread() ;
      if ( currentThread instanceof KlavaProcess ) {
        KlavaProcess currentProc = (KlavaProcess)currentThread ;
        PhysicalLocality pl = currentProc.toPhysical( loc ) ;
        if ( pl != null ) {
          Print( "Using Process Environment for " + loc + " = " + pl ) ;
          return pl ;
        }
      }

      PhysicalLocality ploc = environment.toPhysical( loc ) ;

      if ( ploc == null ) {
        // throw new KlavaLogicalLocalityException( loc.toString() ) ;
        try {
          return translateLogicalLocality((LogicalLocality)loc);
        } catch (KlavaException ke) {
          ke.printStackTrace();
          return null;
        }
      }

      return ploc ;
  }

  protected PhysicalLocality translateLogicalLocality(LogicalLocality loc)
      throws KlavaException {
    Tuple t = new Tuple(loc);
    try {
      waitToBeConnected();
      goWaiting( t, logon_server, NodeMessage.EVALUATE_LOCALITY, -1, true, false ) ;
      PhysicalLocality phy_loc = (PhysicalLocality) t.getItem(0) ;
      return phy_loc;
    } catch (InterruptedException ie) {
      return null;
    }
  }

  // also specify whether you want to log this process
  public void addProcess( KlavaProcess P, boolean log_proc ) {
    if( P == Thread.currentThread() ) {
      PrintSystemErrMessage( "addProcess of this" ) ;
      return ;
    } else {
      P.setName( P.KlavaProcName() + "-" + here + "-" + nextProcID()) ;

      if (log_proc ||
          procnames_to_log.contains(excludePackage(P.KlavaProcName())))
        setProcessMessagePrinter(P);

      if (application != null &&
          procnames_to_wait.contains(excludePackage(P.KlavaProcName())))
        application.addProcess(P);

      if (! P.isBound())
          P.setNode( createProxy() ) ;
    }

    synchronized( processes ) {
      processes.put( P.getName(), P ) ;
    }

    PrintNewProcess( P.getName() ) ;

    P.start() ;
  }

  // given a class name retrieve only the name, discarding a possible
  // package name
  protected String excludePackage(String name) {
    int index = name.lastIndexOf('.');

    if (index == -1)
      return name;

    return name.substring(index + 1);
  }

  // log all processes given a specific name
  public void logForName(String name) {
    procnames_to_log.add(name);
  }

  public void logRecordedClasses(boolean b) {
    log_recorded_classes = b;
  }

  public void acceptRemoteProcesses(boolean b) {
    accept_remote_processes = b;
  }

  // wait all processes given a specific name
  public void waitForName(String name) {
    procnames_to_wait.add(name);
  }

  // default is to log the process
  public void addProcess( KlavaProcess P ) {
    addProcess(P, default_log_processes);
  }

  public void setDefaultLogProcesses(boolean def) {
    default_log_processes = def;
  }

  protected void setProcessMessagePrinter(KlavaProcess P) {
    if (msg_printer_factory != null)
        P.setMessagePrinter
          (msg_printer_factory.createMessagePrinter(P.KlavaProcName(), null));
  }

  protected NodeProxy createProxy() {
    return new NetNodeProxy(this);
  }

  public void removeProcess( KlavaProcess P ) {
    synchronized ( P ) {
      PrintRemovedProcess( P.getName() ) ;
      if ( P.isExecuting() )
        P.stop() ;
      P.notifyAll() ;
      if ( P.mutex != null ) {
        synchronized( P.mutex ) {
          if ( P.raceWinner )
            P.mutex.notifyAll() ;
        }
      }
    }

    synchronized( processes ) {
      processes.remove( P.getName() ) ;
    }
  }

  public PhysicalLocality ThisLocality() {
    return here ;
  }

  public void Print( String s ) {
    infoMessages.PrintMessage( "(" + ProcName() + ") " + s + "\n" ) ;
  }

  synchronized protected String ProcName() {
    Thread curThread = Thread.currentThread() ;
    if ( curThread instanceof KlavaProcess )
      return ((KlavaProcess)curThread).KlavaProcName() ;
    else
      return curThread.getName() ;
  }

  protected void PrintMessage( String s ) {
    if ( MessagesOn )
      PrintSystemOutMessage( s ) ;
  }

  protected void PrintSystemOutMessage( String s )
  {
    System.out.println ("<" + here + ">" + s);
  }

  protected void PrintSystemErrMessage( String s )
  {
    System.err.println ("<" + here + ">" + s);
  }

  synchronized protected void PrintEnvironment() {
    Print( "Starting Environment : " + environment ) ;
  }

  synchronized public void PrintQueue() {
    Print( "outgoing messages queue : " ) ;
    Print( "" + outMessageQueue ) ;
  }

  synchronized protected void PrintNewProcess( String s ) {
    Print( "Starting Process " + s ) ;
  }

  synchronized protected void PrintRemovedProcess( String s ) {
    Print( "Removing Process " + s ) ;
  }

  public String toString() {
    return ("NetNode : " + name) ;
  }

  public static void main( String args[] ) throws IOException, KlavaException {
    if ( args.length < 4 || args.length > 5 )
      throw new IOException( "Syntax : NetNode <name> <physLocHere> <hostname> <port> {DirConnect}" ) ;

    NetNode n = new NetNode( args[0], args[1], args[2], Integer.parseInt( args[3] ) ) ;
    if ( args.length == 5 )
      n.setDirectConnections() ;
    n.start() ;
  }
}

class EvaluateLocalityWaitingThread extends Thread {
  private NetNode node;
  protected NodeMessage message;

  public EvaluateLocalityWaitingThread(NetNode node, NodeMessage message) {
    this.node = node;
    this.message = message;
  }

  public void run() {
    PhysicalLocality ploc = null;
    try {
      ploc = node.getPhysicalLocality
        ((LogicalLocality)((Tuple)message.Content).getItem(0));
    } catch (KlavaLogicalLocalityException ke) {
      ke.printStackTrace();
    }
    node.SendMessage( node.ThisLocality(),
                 message.Source, message.ProcessName,
                 NodeMessage.RESPONSE_LOCALITY, new Tuple(ploc),
                 -1, true, false );
  }
}

