package Klava;

import java.io.*;
import java.util.*;

public class Node extends NetNode {
  public Node( String loc, int my_port_, String NetHostName, int NetPort )
      throws IOException, KlavaException {
    super(loc, my_port_, NetHostName, NetPort);
  }

  public Node( String loc, Integer my_port_, String NetHostName, Integer NetPort )
      throws IOException, KlavaException {
    super(loc, my_port_.intValue(), NetHostName, NetPort.intValue());
  }

  public Node( String loc, PhysicalLocality ploc, String NetHostName, int NetPort )
      throws IOException, KlavaException {
    super(loc, ploc, NetHostName, NetPort);
  }

  public Node( String loc, String ploc, String NetHostName, int NetPort )
      throws IOException, KlavaException {
    super(loc, ploc, NetHostName, NetPort);
  }

  public Node( String Name, String Here, String NetHostName, Integer NetPort )
    throws IOException, KlavaException {
      super(Name, Here, NetHostName, NetPort.intValue());
  }

  protected void initNode( String Name, PhysicalLocality Here, String NetHostName, int NetPort )
      throws IOException, KlavaException {
    super.initNode(Name, Here, NetHostName, NetPort);
    setAutomaticLocalityEvaluation(true);
    // in the original Klaim model the evaluation of localities and
    // process closure was automatic
    setCloseOnLostNetConnection(true);
    // for backward compatibility, when a node is not connected to a Net
    // it terminates its execution
  }

  public void run() {
    // for backward compatibility
    try {
      addProcess(
          new LoginNodeCoordinator(
          new PhysicalLocality(
          NetUtils.createNodeAddress(netHostName, netPort))));
    } catch (KlavaMalformedPhyLocalityException e) {
      e.printStackTrace();
      return;
    }
    super.run();
  }

  // called if the message is not destined to this node
  protected void handleForeignMessage(NodeMessage message) {
    PrintSystemErrMessage("Message wrongly delivered here: " + message);
  }

  protected void propagateLocalities(Locality loc, Vector locs) {
    // in the flat model there's no need to propagate localities
  }

  public PhysicalLocality newloc(NodeCoordinator P, int port,
                                 String classname) throws KlavaException {
    // make sure that we create a Klava.Node (for backward compatibility)
    return super.newloc(P, port,
                        (classname == null ? "Klava.Node" : classname));
  }

  protected Object[] build_args_for_NewNode(NodeCoordinator P, int port,
      String classname) {
    Object[] args = {"Node", new Integer(port),
      getNetHostName(), new Integer(getNetPort())};
    return args;
  }

  protected Class[] build_params_for_NewNode() {
    Class parameters[] = {java.lang.String.class, java.lang.Integer.class,
      java.lang.String.class, java.lang.Integer.class};
    return parameters;
  }

  public void addSubNode(NetNode n) {
    synchronized(subNodes) {
      subNodes.put(n.getHereLoc(), n);
    }
    // this is not very clean: it redefines the NetNode implementation
    // for backward compatibility
  }

  public static void main( String args[] ) throws IOException, KlavaException {
    if ( args.length < 4 || args.length > 5 )
      throw
      new IOException( "Syntax : Node <name> <localport>(or physloc) <hostname> <port> {DirConnect}" ) ;

    NetNode n;
    n = new Node( args[0], args[1],
                    args[2], Integer.parseInt( args[3] ) ) ;

    if ( args.length == 5 )
      n.setDirectConnections() ;

    KlavaApplication app = new KlavaApplication ();
    app.addNode (n);
    app.startNodes ();

    app.waitForNodesAndTerminate();
  }
}