package Klava;

/**
 * This is used as the interface for NodeApplet, i.e. a node that is
 * created by a KlavaApplet when connecting to a server.
 * If you provide your own NodeApplet, you have to implement these
 * methods accordingly.  Typically the setKlavaApplet should simply
 * store the reference to the KlavaApplet and lostNetConnection should
 * invoke stopNode on the KlavaApplet.  See NodeApplet implementation
 * as an example.
 * @see Klava.NodeApplet
 *
 * @author $author$
 * @version $Revision: 1.1.2.1 $
 */
public interface NodeAppletInt extends NetNodeInterface {
    /**
     * called by KlavaApplet for storing the reference to it
     *
     * @param applet reference to KlavaApplet
     */
    void setKlavaApplet(KlavaApplet applet);

    /**
     * called when the connection to the server is lost (e.g. due to
     * a net failure).  This should be implemented by simply calling
     * the method stopNode on the store KlavaApplet:
     *
     * <pre>
     * klava_applet.stopNode();
     * </pre>
     *
     * @see Klava.NodeApplet#lostNetConnection
     */
    void lostNetConnection();

    /**
     * Usually inherited by NetNode
     */
    void start();

    /**
     * Usually inherited by Thread
     *
     * @return whether this thread is alive
     * @see java.lang.Thread
     */
    boolean isAlive();

    /**
     * Usually inherited by NetNode
     */
    void Close();
}
