// Node that outs tuples to a graphycal window

package Klava ;

import java.awt.* ;
import java.io.* ;

public class NodeConsole extends NodeG {
  public NodeConsole( String Name, String Here, String NetHostName, int NetPort )
    throws IOException, KlavaException {
      super( Name, Here, NetHostName, NetPort  ) ;
  }

  public NodeConsole( String Name, String Here, String NetHostName, Integer NetPort )
    throws IOException, KlavaException {
      super( Name, Here, NetHostName, NetPort ) ;
  }

  public NodeConsole( String Name, PhysicalLocality Here, String NetHostName, int NetPort )
    throws IOException, KlavaException {
      super( Name, Here, NetHostName, NetPort  ) ;
  }

  public NodeConsole( String Name, int Here, String NetHostName, int NetPort )
    throws IOException, KlavaException {
      super( Name, Here, NetHostName, NetPort  ) ;
    }

  public void addToEnv( LogicalLocality logL, PhysicalLocality phyLoc )
  {
    String logLoc = logL.toString();
    if ( isSpecialLoc( logLoc ) ) {
      try {
        Node n = handleSpecialLoc( logLoc, phyLoc ) ;
        newLocalities.put( new PhysicalLocality(phyLoc), n ) ;

        if ( n != null ) {
            if ( DirectConnections )
                n.setDirectConnections() ;

            show() ;
            n.start() ;
        }
      } catch ( IOException e ) {
      	e.printStackTrace() ;
      } catch (KlavaException ke) {
          ke.printStackTrace();
      }
    }
    super.addToEnv( logL, phyLoc ) ;
  }

  protected boolean isSpecialLoc( String logLoc ) {
    return ( logLoc != null && (
      logLoc.equals( "screen" ) ||
      logLoc.equals( "keyb" ) )
    ) ;
  }

  protected Node handleSpecialLoc( String logLoc, PhysicalLocality phyLoc )
          throws IOException, KlavaException
  {
    Node n = null ;

    if ( logLoc.equals( "screen" ) ) {
      n = new NodeScreen( "screen", phyLoc, netHostName, netPort ) ;
      nodeFrame.addRow( ((withPanel)n).getPanel() ) ;
    } else if ( logLoc.equals( "keyb" ) ) {
      n = new NodeKeyboard( "keyboard", phyLoc, netHostName, netPort ) ;
      nodeFrame.addRow( ((withPanel)n).getPanel(), BorderLayout.SOUTH ) ;
    }

    return n ;
  }

  public static void main( String args[] ) throws IOException, KlavaException {
    if ( args.length < 4 || args.length > 5 )
      throw new IOException( "Syntax : Node <name> <localport> <hostname> <port> {DirConnect}" ) ;

    int port = Integer.parseInt(args[1]);
    NodeConsole n = new NodeConsole( args[0], port + "", args[2], Integer.parseInt( args[3] ) ) ;
    if ( args.length == 5 )
      n.setDirectConnections() ;

    n.addToEnv("screen", port + 1);
    n.addToEnv("keyb", port + 2);

    KlavaApplication app = new KlavaApplication ();
    app.addNode (n);
    app.startNodes ();

    app.waitForNodesAndTerminate();
  }
}
