package Klava ;

import java.io.* ;
import java.net.* ;

public class NodeHandler extends Thread {
  protected Queue messageQueue = new Queue() ;; // messages queue
  // streams for networking communication
  protected InputStream iStream ;
  protected OutputStream oStream ;
  protected ObjectInputStream objIStream ;
  protected ObjectOutputStream objOStream ;
  protected Socket socket;
  protected LoginHandler login_handler ; // Net we belong to
  protected String RemoteHostAddress ;
  protected int RemoteHostPort = 0;
  protected Locality LocHandled ;
  protected LogicalLocality my_log_loc;
  protected NetNode node;
  protected PhysicalLocality node_phy_loc;

  public NodeHandler( LoginHandler login, NetNode n, Socket s,
                      LogicalLocality lloc, PhysicalLocality ploc )
      throws IOException {
    login_handler = login ;
    socket = s;
    iStream = s.getInputStream() ;
    oStream = s.getOutputStream() ;
    RemoteHostAddress = NetUtils.getNodeIPAddress(s.getInetAddress());
    my_log_loc = lloc;
    node = n;
    node_phy_loc = node.ThisLocality();
    setName(node_phy_loc + "'s " + getName());
    LocHandled = ploc;
  }

  void setStreams(ObjectOutputStream os, ObjectInputStream is) {
    objOStream = os;
    objIStream = is;
  }

  public void run() {
    try {
      execute();
    } catch (IOException ioe) {
      System.err.println(ioe);
    } finally {
      try {
        login_handler.lostConnection(my_log_loc, LocHandled);
        oStream.close();
      } catch (IOException ioe) {
        ioe.printStackTrace();
      }
    }
  }

  protected Locality retrieve_locality(ObjectInputStream objIStream)
        throws ClassNotFoundException, IOException {
    PhysicalLocality loc;
    loc = ( PhysicalLocality ) objIStream.readObject() ;
    return loc;
  }

  protected void execute() throws IOException {
    MessageDeliverer deliverer = new MessageDeliverer(
	this, messageQueue, objOStream, null ) ;

    try {
      deliverer.start() ;
      route() ;
    } finally {
      deliverer.interrupt() ;
    }
  }

  public int getPort() { return RemoteHostPort ; }
  public String getRemoteAddress() { return RemoteHostAddress ; }

  protected void stop_route() {
    try {
      objIStream.close();
    } catch (IOException ioe) {
      ioe.printStackTrace();
    }
  }

  protected void route() throws IOException {
    NodeMessage message ;
    Locality loc ;
    NodeHandler nHandler ;

    while( true ) {
      try {
	message = ( NodeMessage ) objIStream.readObject() ;
	loc = message.Dest ;
	Print( " got Message " + message ) ;

        deliver_msg(loc, message);
      } catch ( ClassNotFoundException e ) {
        e.printStackTrace() ;
      }
    }
  }

  protected void deliver_msg(Locality loc, NodeMessage message) {
    if (loc.equals(node_phy_loc))
      node.enqueueInMessage(message);
    else {
      NodeHandler nHandler = resolve_locality(loc);
      if ( nHandler == null ) {
        handle_unresolved_dest(loc, message);
      } else {
        nHandler.Enqueue( message ) ;
      }
    }
  }

  protected NodeHandler resolve_locality(Locality loc) {
    return login_handler.getNodeHandler( loc ) ;
  }

  protected void handle_unresolved_dest(Locality loc, NodeMessage message) {
    if (node.deliverToServer(message))
      return;

    // sends an exception to the sender only if it's a blocking op
    // or if ack is required
    Print( loc + " not found on Net!" ) ;
    if( message.OpCode == NodeMessage.IN ||
        message.OpCode == NodeMessage.READ ||
        message.ack_requested ) {
      message.OpCode = NodeMessage.NOPHYSLOC ;
      message.Content = new Tuple(
          new KlavaPhysicalLocalityException( loc.toString() ) ) ;
      message.Dest = message.Source ;
      Enqueue( message ) ;
    }
  }

  public void Enqueue( NodeMessage m ) {
    messageQueue.add( m ) ;
  }

  protected void Print( String s ) {
    System.out.println(getName() + ": " + s ) ;
  }
}
