package Klava;


public abstract class NodeInterfaceDecorator implements NodeInterface {

  protected NodeInterface node ;
  protected LogicalLocality self = new LogicalLocality( "self" ) ;

  protected NodeInterfaceDecorator ( NodeInterface node ) {
    this.node = node ;
  }


  public void out( Tuple t ) throws KlavaException {
    node.out( t ) ;
  }

  public void out( Tuple t, Locality l ) throws KlavaException {
    node.out( t, l ) ;
  }

  public void in( Tuple t ) throws KlavaException {
    node.in( t ) ;
  }

  public void in( Tuple t, Locality l) throws KlavaException {
    node.in( t, l );
  }

    // time out versions
  public void in( Tuple t, long TimeOut )  throws KlavaException {
    node.in( t , TimeOut ) ;
  }

  public void in( Tuple t, Locality l, long TimeOut )  throws KlavaException {
    node.in( t, l, TimeOut );
  }

  public boolean in_t( Tuple t, Locality l, long TimeOut ) throws KlavaException {
    return node.in_t( t, l, TimeOut );
  }

  // non blocking version
  public boolean in_nb(Tuple t,Locality l) throws  KlavaException {
    return node.in_nb( t, l ) ;
  }


  public void read( Tuple t ) throws KlavaException {
    node.read( t ) ;
  }

  public void read( Tuple t, Locality l ) throws KlavaException {
    node.read( t, l ) ;
  }

   // time out versions
  public void read( Tuple t, long TimeOut ) throws KlavaException {
    node.read( t, TimeOut ) ;
  }

  public void read( Tuple t, Locality l, long TimeOut )  throws KlavaException {
    node.read( t, l, TimeOut ) ;
  }

  public boolean read_t( Tuple t, Locality l, long TimeOut ) throws KlavaException {
    return node.read_t( t, l, TimeOut ) ;
  }

  // non blocking version
  public boolean read_nb( Tuple t, Locality l ) throws KlavaException {
    return node.read_nb( t, l ) ;
  }

  // To set the right node reference in case of decoration.
  // This solution doesn't handle proc closures wrapped in KlavaProcessVar
  protected void setNode( KlavaProcess P ) {
    if ( ! P.isBound() ) {
      P.setNode( this );
    	P.setBound();
    }
  }

  public void eval( KlavaProcess P ) throws KlavaException {
    eval( P, self ) ;
  }

  public void eval( KlavaProcess P, Locality loc ) throws KlavaException {
    if ( ThisLocality().equals( getPhysicalLocality( loc ) ) ) {
      setNode( P ) ;
    }
    node.eval( P, loc ) ;
  }

  public void eval( KlavaProcessVar P ) throws KlavaException {
     eval( P, self ) ;
  }

  public void eval( KlavaProcessVar P, Locality loc ) throws KlavaException {
    KlavaProcess proc ;
    if ( ThisLocality().equals( getPhysicalLocality( loc ) )
         && ( proc = P.getProc() ) != null  ) {
      setNode( proc ) ;
    }
    node.eval( P, loc ) ;
  }

  public PhysicalLocality newloc() throws KlavaException {
    return node.newloc() ;
  }

  public void removeloc( Locality l ) throws KlavaException {
    node.removeloc( l ) ;
  }

  public void or( KlavaProcess P1, KlavaProcess P2 ) {
    setNode( P1 ) ;
    setNode( P2 ) ;
    node.or( P1, P2 ) ;
  }

  public void procCall( Callable P ) throws KlavaException {
    if ( P instanceof KlavaProcess ) {
      setNode( ( KlavaProcess ) P ) ;
    } else if ( P instanceof KlavaProcessVar ) {
      KlavaProcess proc = ( ( KlavaProcessVar ) P ).getProc() ;
      if ( proc != null ) {
        setNode( proc ) ;
      }
    }
    node.procCall( P ) ;
  }

  public void addProcess( KlavaProcess P ) {
    setNode( P ) ;
    node.addProcess( P ) ;
  }

  public void removeProcess( KlavaProcess P ) {
    node.removeProcess( P ) ;
  }

  public void requestLocality( PhysicalLocality l, Tuple t ) throws KlavaException {
    node.requestLocality( l, t ) ;
  }

  public PhysicalLocality ThisLocality() {
    return node.ThisLocality() ;
  }

  public PhysicalLocality getPhysicalLocality( Locality loc )
      throws KlavaLogicalLocalityException
  {
     return node.getPhysicalLocality( loc ) ;
  }

  public void Print( String s ) {
    node.Print( s ) ;
  }

  // To get the wrapped node
  public NodeInterface getDecoratedNode() {
    return node ;
  }

  public String toString() {
    return getClass().getName() + "( " + node.toString() + " )" ;
  }

}

