/**
 * Base class for a generic NetNode that waits for subscriptions
 */

package Klava;

import java.io.*;

public class RegisterNetNode extends NetNode {
  protected RegisterNodeCoordinator register_nodecoord;
  protected UnregisterNodeCoordinator unregister_nodecoord;
  protected DisconnectedLogLocNodeCoordinator disconnected_nodecoord;

  public RegisterNetNode( String loc, int my_port, String NetHostName, int NetPort )
      throws IOException, KlavaException {
    super(loc, my_port, NetHostName, NetPort);
    register_nodecoord = newRegisterNodeCoordinator();
    addProcess(register_nodecoord);
    unregister_nodecoord = newUnregisterNodeCoordinator();
    addProcess(unregister_nodecoord);
    disconnected_nodecoord = newDisconnectedLogLocNodeCoordinator();
    addProcess(disconnected_nodecoord);
  }

  public RegisterNetNode( String loc, int my_port )
          throws IOException, KlavaException
  {
    this(loc, my_port, null, -1);
  }

  protected RegisterNodeCoordinator newRegisterNodeCoordinator() {
    return new RegisterNodeCoordinator();
  }

  protected UnregisterNodeCoordinator newUnregisterNodeCoordinator() {
    return new UnregisterNodeCoordinator();
  }

  protected DisconnectedLogLocNodeCoordinator
    newDisconnectedLogLocNodeCoordinator() {
      return new DisconnectedLogLocNodeCoordinator();
  }

  public void logRegister() {
    setProcessMessagePrinter(register_nodecoord);
  }

  public void logUnregister() {
    setProcessMessagePrinter(unregister_nodecoord);
  }

  public void logDisconnected() {
    setProcessMessagePrinter(disconnected_nodecoord);
  }

  public static void main( String args[] ) throws java.io.IOException, KlavaException {
    if ( args.length < 2 || args.length > 5 ||
          (args.length > 2 && args.length < 4 ) )
      throw new
      java.io.IOException
      ( "Syntax : RegisterNetNode <logLoc> <my_port> {<hostname> <port> {DirConnect}}" ) ;

    NetNode n;
    if (args.length > 2)
      n = new RegisterNetNode( args[0], Integer.parseInt(args[1]),
        args[2], Integer.parseInt( args[3] ) ) ;
    else
      n = new RegisterNetNode( args[0], Integer.parseInt(args[1]));

    if ( args.length == 5 )
      n.setDirectConnections() ;

    n.start() ;
  }
}