package Klava;

/**
 * store routes for physical localities
 */

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;

public class RoutingTable {
  // Nodes in the Net : ( Loc, NetNodeHandler )
  protected Hashtable node_handlers = new Hashtable();
  // Nodes to which messages for other nodes are forwarded
  // ( ProxyNode, Vector of nodes connected to proxy )
  protected Hashtable proxies = new Hashtable();
  private NetNode node;
  protected MessagePrinter message_printer = null;

  public RoutingTable(NetNode node) {
    this.node = node;
  }

  void setMessagePrinter(MessagePrinter msg) {
    message_printer = msg;
  }

  synchronized void addRoute(Locality loc, NodeHandler nhandler) {
    node_handlers.put(loc, nhandler);
    printAddRoute(nhandler, loc);
  }

  synchronized boolean addRoute(Locality subloc, Locality suploc) {
    NodeHandler nhandler = (NodeHandler) getNodeHandler(suploc);

    if (nhandler == null)
      return false;

    addRoute(subloc, nhandler);
    addProxy(suploc, subloc);

    return true;
  }

  synchronized void removeRoute(Locality loc) {
    NodeHandler proxy = (NodeHandler)(node_handlers.remove(loc));

    printRemoveRoute(proxy, loc);

    Vector nodes = (Vector) proxies.get(proxy.LocHandled);
    if (nodes != null)
      nodes.remove(loc);

    removeProxy(loc);
  }

  synchronized boolean hasRoute(Locality loc) {
    return node_handlers.containsKey(loc);
  }

  synchronized NodeHandler getNodeHandler( Locality l ) {
    return ( NodeHandler ) node_handlers.get( l ) ;
  }

  synchronized Enumeration getLocalities() {
    return node_handlers.keys();
  }

  synchronized protected void addProxy(Locality proxy, Locality loc) {
    Vector nodes = (Vector)proxies.get(proxy);
    if (nodes == null) {
      nodes = new Vector();
      proxies.put(proxy, nodes);
    }

    nodes.addElement(loc);
    printAddProxy(proxy, nodes);
  }

  // remove all the localities connected to this proxy
  synchronized protected void removeProxy(Locality proxy) {
    Vector nodes = (Vector)proxies.remove(proxy);
    if (nodes == null)
      return;

    printRemoveProxy(proxy, nodes);

    NodeHandler nh;
    Locality loc;
    Enumeration subnodes = nodes.elements();
    while (subnodes.hasMoreElements()) {
      loc = (Locality) subnodes.nextElement();
      nh = (NodeHandler) node_handlers.remove(loc);
      printRemoveRoute(nh, loc);
    }
  }

  void printRemoveRoute(NodeHandler nh, Locality loc) {
    String msg = "Remove route " + loc +
      " -> " + nh.LocHandled;
    if (message_printer == null)
      node.PrintSystemOutMessage(msg);
    else
      message_printer.Print(msg);
  }

  void printRemoveProxy(Locality proxy, Vector nodes) {
    String msg = "Remove proxy " + proxy +
      " for " + nodes;
    if (message_printer == null)
      node.PrintSystemOutMessage(msg);
    else
      message_printer.Print(msg);
  }

  void printAddRoute(NodeHandler nh, Locality loc) {
    String msg = "Add route " + loc +
      " -> " + nh.LocHandled;
    if (message_printer == null)
      node.PrintSystemOutMessage(msg);
    else
      message_printer.Print(msg);
  }

  void printAddProxy(Locality proxy, Vector nodes) {
    String msg = "Add proxy " + proxy +
      " for " + nodes;
    if (message_printer == null)
      node.PrintSystemOutMessage(msg);
    else
      message_printer.Print(msg);
  }

  void release_handlers() {
    Enumeration handlers = node_handlers.elements();
    NodeHandler handler;

    while (handlers.hasMoreElements()) {
      handler = (NodeHandler) handlers.nextElement();
      node.PrintSystemOutMessage("Removing NodeHandler " + handler);
      handler.stop_route();
    }
  }
}