package Klava ;

import java.util.Vector ;
import java.util.Enumeration ;
import java.util.Hashtable;
import java.util.Date ;

public class Tuple implements Cloneable, java.io.Serializable {

    protected String id ; // the GUID of this tuple
    protected Vector items ; // the elements of this tuple
    protected Hashtable already_retrieved = null;
    // the ids of tuples already retrieved with this template.
    protected boolean handle_retrieved = false;
    // if already retrieved tuples' ids have to be recorded
    protected Tuple matched = null;
    // the tuple we have matched with this tuple
    protected Tuple original_template = null;
    // the original template with which we matched this tuple
    protected static int id_counter = 1;
    // this will be added to id, in order to be sure that two ids are different

  public Tuple() {
    items = new Vector() ;
    initId ();
  }

  public Tuple( Vector v ) {
    items = v ;
    initId ();
  }

  public Tuple( Tuple t ) {
    items = t.items;

    original_template =  t.getOriginalTemplate() ;
    setMatched( t.getMatched() ) ;

    if (handleRetrieved ())
        already_retrieved = t.already_retrieved;

    initId ();
  }

  public Tuple( Object t[] ) {
    items = new Vector( t.length ) ;

    for( int i = 0 ; i < t.length ; i++ ) {
      add( t[ i ] ) ;
    }
    initId ();
  }

    protected synchronized void initId() {
        Date date = new Date() ;
        long longtime = date.getTime() + Tuple.id_counter++ ;
        id = String.valueOf(longtime) ;
    }

  public Tuple( Object o1 ) {
    items = new Vector( 1 ) ;
    add( o1 ) ;
    initId ();
  }

  public Tuple( Object o1, Object o2 ) {
    items = new Vector( 2 ) ;
    add( o1 ) ;
    add( o2 ) ;
    initId ();
  }

  public Tuple( Object o1, Object o2, Object o3 ) {
    items = new Vector( 3 ) ;
    add( o1 ) ;
    add( o2 ) ;
    add( o3 ) ;
    initId ();
  }

  public Tuple( Object o1, Object o2, Object o3, Object o4 ) {
    items = new Vector( 4 ) ;
    add( o1 ) ;
    add( o2 ) ;
    add( o3 ) ;
    add( o4 ) ;
    initId ();
  }

  public void setItem( int index, Object item ) {
    items.setElementAt( item, index ) ;
  }

  public Object getItem( int index ) {
    return items.elementAt( index ) ;
  }

    // reset the tuple without creating a new tuple
    public void reset() {
      items = new Vector ();
      matched = null;
      original_template = null;
    }

    protected Vector getItems() { return items; }

    public String getTupleId() { return id; }

    public void setTupleId(String _id) { id = _id; }

    public Tuple getMatched() { return matched; }

    public void setMatched(Tuple t) { matched = t; }

    public Tuple getOriginalTemplate() { return original_template; }

    public void setOriginalTemplate() {
        original_template = (Tuple) clone ();
        backup_original_template ();
    }

    protected void backup_original_template() {
        for ( int i = 0; i < length(); i++ ) {
            if ( original_template.items.elementAt(i) instanceof TupleItem ) {
                original_template.items.setElementAt
                    ( ((TupleItem)(items.elementAt(i))).duplicate (), i ) ;
            }
        }
        /*
          Notice that we have to use duplicate, because the orginal
          tuple items of the original_template would be overwritten
          anyway, when calling copy inside match.
         */
    }

    // reset the original template
    public void resetOriginalTemplate() {
        if ( original_template != null ) {
          restore_original_template ();
        }

        matched = null;
    }

    protected void restore_original_template() {
        Object elem;
        TupleItem my_elem;

        for ( int i = 0; i < length(); i++ ) {
            elem = original_template.getItem (i);
            if ( elem instanceof TupleItem ) {
                my_elem = ((TupleItem)(getItem (i)));
                my_elem.setValue (elem);
                // we maintain the original reference
            } else {
                setItem (i, elem); // simply copy
            }
        }
        /*
          Notice that we have to perform a manual copy, since we cannot
          lose the original reference to tuple items (clone would
          overwrite them.
         */
    }

    public void resetRetrieved() { already_retrieved = new Hashtable (); }

    public void updateAlreadyRetrieved(Tuple t) {
        already_retrieved = t.already_retrieved;
    }

    public void addRetrieved(String _id) {
        if (! handleRetrieved ())
            return;

        if (already_retrieved == null)
            already_retrieved = new Hashtable ();

        // as a dummy value we put our own id
        already_retrieved.put (_id, id);
    }

    public boolean alreadyRetrieved(String id) {
        return ((already_retrieved != null) &&
                (already_retrieved.containsKey (id)));
    }

    public void setHandleRetrieved(boolean b) { handle_retrieved = b; }

    public boolean handleRetrieved() { return handle_retrieved; }

  public int length() {
    return items.size() ;
  }

  public Enumeration tupleElements() {
    return items.elements() ;
  }

  public String toString() {
    StringBuffer sb = new StringBuffer( "( " ) ;

    for ( int i = 0 ; i < length() ; i++ ) {
      sb.append( getItem( i ) ) ;
      if ( i < length() - 1 ) {
	sb.append( ", " ) ;
      }
    }

    //sb.append( " ) " + getTupleId () ) ;
    sb.append( " ) " ) ;

    /*
    if (handleRetrieved () &&
        already_retrieved != null) {
        Enumeration retrieved = already_retrieved.keys ();
        sb.append (" {");
        while (retrieved.hasMoreElements ()) {
            sb.append (retrieved.nextElement ());
            if (retrieved.hasMoreElements ())
                sb.append (", ");
        }
        sb.append ("}");
    }
    */

    return sb.toString() ;
  }

  public void clear() {
    items = new Vector() ;
  }

  public void add( Object o ) {
    // this is for a probable Java Bug during deseralization
    if ( o instanceof TupleItem && !( ((TupleItem)o).isFormal() ) )
      items.addElement( ((TupleItem)o).duplicate() ) ;
    else
      items.addElement( o ) ;
  }

  public Object clone() {
      Tuple new_tuple = new Tuple( (Vector)items.clone() ) ;
      new_tuple.id = id; // also the id is copied
      return new_tuple;
  }

    public void copy( Tuple t ) {
        copy_elems (t);

        setMatched (t);

        if (handleRetrieved ())
            addRetrieved (t.getTupleId ());
    }

    public void copy_tuple(Tuple t) {
        copy_elems (t);

	original_template =  t.getOriginalTemplate() ;
	setMatched( t.getMatched() ) ;

        if (handleRetrieved ())
            already_retrieved = t.already_retrieved;

        // FIXME: maybe it's better to merge the two tables?
    }

    protected void copy_elems(Tuple t) {
        Object elem;
        Object my_elem;

        for ( int i = 0; i < length(); i++ ) {
            elem = t.getItem (i);
            my_elem = getItem (i);
            if ( my_elem instanceof TupleItem ) {
                ((TupleItem)my_elem).setValue ((TupleItem)elem) ;
            } else {
                // simple copy
                setItem (i, elem);
            }
        }
    }

  public boolean preMatch( Tuple toBeMatched ) {
      if ( length() != toBeMatched.length() ) {
      	  return false ;
      }

      if (toBeMatched.handleRetrieved ())
          return (! toBeMatched.alreadyRetrieved (getTupleId ()));

      return true;
  }

  public boolean match( Tuple toBeMatched ) {
    if ( ! preMatch(toBeMatched) ) {
        return false;
    }

    // we make operations on a copy of the original tuple
    Tuple t ;

    t = (Tuple)toBeMatched.clone() ;
    t.id = id; // we also set the id of this tuple

    for ( int i = 0 ; i < length() ; i++ ) {
      Object MyElement = getItem( i ) ;
      Object ItsElement = t.getItem( i ) ;

      // we skip bad tuple items
      if( MyElement == null )
	continue ;

      if( ItsElement == null )
	return false ;

      if( ItsElement instanceof Class &&
	  ItsElement.equals( MyElement.getClass() ) ) {
	try {
	  if ( ! ( ItsElement.equals( getClass() ) ) ) {
	    // for security fields of tuple type are not allowed
	    t.setItem( i, MyElement ) ;
	  } else {
	    return false ;
	  }
	} catch ( Exception e ) {
	  System.out.println( e ) ;
	  return false ;
	}
      } else if ( ! ItsElement.getClass().equals( MyElement.getClass() ) ) {
	return false ;
      } else if ( ItsElement instanceof TupleItem ) {
        TupleItem itsElement = (TupleItem)ItsElement;
	if ( itsElement.isFormal() ) {
          if (itsElement instanceof MoMiTupleItem) {
            MoMiTupleItem momi_item = (MoMiTupleItem) itsElement;
            if (! ((MoMiTupleItem)MyElement).getType().subtype(momi_item.getType()))
                return false;
          }
	  t.setItem( i, MyElement ) ;
	} else if ( ItsElement.equals( MyElement ) ) {
	  continue ;
	} else
	  return false ;
      } else if (
		 ( ItsElement.equals( MyElement ) )
		 ||
		 ( MyElement instanceof Tuple &&
		   ItsElement instanceof Tuple &&
		   ((Tuple)MyElement).match( (Tuple)ItsElement ) ) ) {
	continue ;
      } else {
	return false ;
      }
    }

    // first we save the orginal template
    toBeMatched.setOriginalTemplate ();
    // we update the original
    toBeMatched.copy(t) ;
    return true ;
  }
}

