/**
 * KCipherLocalMatcher
 * @author Lorenzo Bettini
 * @version 1.0
 *
 * Customized version of LocalMatcher that tries to decrypt the received
 * remote tuple, in order to test if it matches.
 */

package Klava.crypto;

import Klava.LocalMatcher;
import Klava.Tuple;
import Klava.KlavaProcess;

public class KCipherLocalMatcher extends LocalMatcher {

    /**
     * Constructor
     */
    public KCipherLocalMatcher() { super (); }

    /**
     * Do a match with the tuple received from the network.
     * If the tuple is a Tuplex it also tries to decrypt the
     * contents and test if they match.
     * @param source the template used for the match
     * @param received candidate tuple
     * @param requester the KlavaProcess that requested the tuple
     * @return true if it can decrypt the contents and they match
     */
    public boolean localMatch(Tuple source, Tuple received,
                              KlavaProcess requester) {
	// we really need source?
        if (! (source instanceof Tuplex) ||
            ! (received instanceof Tuplex) ||
            ! (requester instanceof KlavaProcessx) )
            return true;

        Tuplex Source = (Tuplex) source;
        Tuplex Received = (Tuplex) received;
        KlavaProcessx Requester = (KlavaProcessx) requester;

        System.err.println ("KCIPHERLOCALMATCHER ===");

        try {
            return Received.matchTuplex (Requester);
        } catch (java.security.GeneralSecurityException e) {
            e.printStackTrace ();
            return false;
        }
    }
    
}
