/**
 * KCipherLocalTupleSpaceMatcher.
 * @author Lorenzo Bettini
 * @version 1.0
 *
 * Extended version to handle ciphered tuples.
 */

package Klava.crypto;

import Klava.*;

public class KCipherLocalTupleSpaceMatcher extends LocalTupleSpaceMatcher {

    /**
     * Constructor
     */
    public KCipherLocalTupleSpaceMatcher() { super (); }

    /**
     * Constructor
     */
    public KCipherLocalTupleSpaceMatcher(TupleSpace ts) { super (ts); }

    public boolean in( Tuple t, boolean blocking, KlavaProcess requester ) {
        if (! super.in (t, blocking, requester))
            return false;

        return handle_KCipherTuple (t, requester);
    }

    public boolean in( Tuple t, long timeout, KlavaProcess requester )
        throws KlavaTimeOutException {
        if (! super.in (t, timeout, requester))
            return false;

        return handle_KCipherTuple (t, requester);
    }

    public boolean read( Tuple t, boolean blocking, KlavaProcess requester ) {
        if (! super.read (t, blocking, requester))
            return false;

        return handle_KCipherTuple (t, requester);
    }

    public boolean read( Tuple t, long timeout, KlavaProcess requester )
        throws KlavaTimeOutException {
        if (! super.read (t, timeout, requester))
            return false;

        return handle_KCipherTuple (t, requester);
    }

    /**
     * Try to decrypt the tuple if it contains crypted fields.
     * @param t the tuple
     * @param requester the process that requested the tuple
     * @return true if it succeeds
     */
    protected boolean handle_KCipherTuple(Tuple t, KlavaProcess requester) {
        if (! (t instanceof Tuplex) ||
            ! (requester instanceof KlavaProcessx) )
            return true;

        Tuplex tuple = (Tuplex) t;
        KlavaProcessx Requester = (KlavaProcessx) requester;

        System.err.println ("KCIPHERLOCALTUPLESPACEMATCHER ===");

        try {
            return tuple.matchTuplex (Requester);
        } catch (java.security.GeneralSecurityException e) {
            e.printStackTrace ();
	    return false;
        }
    }

    public boolean should_retry() { return true ; }
}
