package Klava.crypto;

import java.io.*;
import java.util.Enumeration;


/**
 * Used for asking the user for the keystore and the key through a dialog.
 *
 * @author $author$
 * @version $Revision: 1.1.2.2 $
 */
public class KStoreDialogAccess implements KStoreAccess {
    /** The frame for requesting the keystore and the key */
    private KStoreRequestFrame request_frame;

    /** The keystore */
    private KStore store;

    /**
     * Ask for the key by means of a dialog
     *
     * @param message message that will appear in the dialog
     * @param store the keystore
     *
     * @return return the info about the chosen key
     *
     * @throws IOException
     * @throws java.security.GeneralSecurityException
     */
    public KeyInfo askKey(String message, KStore store)
        throws IOException, java.security.GeneralSecurityException {
        if (request_frame == null) {
            request_frame = new KStoreRequestFrame(this);
        }

        this.store = store;

        String key_alias;
        String key_pass;

        synchronized (this) {
            request_frame.explain_label.setText(message);
            request_frame.pack();
            request_frame.show();

            try {
                wait();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

        key_alias = request_frame.getKeyAlias();
        key_pass = request_frame.getKeyPassword();

        return new KeyInfo(key_alias, key_pass);
    }

    /**
     * Open the chosen keystore
     *
     * @param storeName name of the keystore
     * @param storePass passphrase for the keystore
     *
     * @throws IOException
     * @throws java.security.GeneralSecurityException
     */
    public void openStore(String storeName, String storePass)
        throws IOException, java.security.GeneralSecurityException {
        store.openKeyStore(storeName, storePass);
    }

    /**
     * Return the aliases of the keys stored in the chosen keystore
     *
     * @return the aliases of the keys stored in the chosen keystore
     *
     * @throws IOException
     * @throws java.security.GeneralSecurityException
     */
    public Enumeration getKeyAliases()
        throws IOException, java.security.GeneralSecurityException {
        return store.getKeyAliases();
    }
}
