package Klava.crypto;

import java.io.*;

/**
 * Used for chosing the keystore and the key by reading this information from a
 * file.  This file does not have a specific syntax: each line represents the
 * answer to a request of the KStoreAccess (e.g., the keystore name, the
 * keystore passphrase, etc.). NOTE: this should be used only for automatic
 * test purposes, since the password and keystore and key names are stored in
 * such file in clear.
 *
 * @author $author$
 * @version $Revision: 1.1.2.2 $
 */
public class KStoreFileAccess implements KStoreAccess {
    /** current line number within the file */
    private int lineNumber;

    /** for reading lines of the file */
    private LineNumberReader in;

    /**
     * Creates a new KStoreFileAccess object.
     *
     * @param file_name name of the file containing the answers for the chosing
     *        the keystore and key.
     *
     * @throws IOException
     */
    public KStoreFileAccess(String file_name) throws IOException {
        in = new LineNumberReader(new FileReader(file_name));
    }

    /**
     * Read the keystore name from the file
     *
     * @param store the reference to the actual keystore
     *
     * @throws IOException
     * @throws java.security.GeneralSecurityException
     */
    protected void askStore(KStore store)
        throws IOException, java.security.GeneralSecurityException {
        String storeName = in.readLine();
        System.out.println(lineNumber + "*F* Read Keystore " + storeName);
        lineNumber = in.getLineNumber();

        String password = askPass(storeName);

        store.openKeyStore(storeName, password);
    }

    /**
     * Read the name of the key from the file
     *
     * @param message actually not used
     * @param store the reference to the actual keystore
     *
     * @return info about the chosen key
     *
     * @throws IOException
     * @throws java.security.GeneralSecurityException
     */
    public KeyInfo askKey(String message, KStore store)
        throws IOException, java.security.GeneralSecurityException {
        if (!store.isKeyStoreOpen()) {
            askStore(store);
        }

        System.out.println("*F* Key request:" + message);
        in.setLineNumber(lineNumber);

        String keyAlias = in.readLine();
        System.out.println(lineNumber + "*F* Read Key " + keyAlias);
        lineNumber = in.getLineNumber();

        String password = askPass(keyAlias);

        return new KeyInfo(keyAlias, password);
    }

    /**
     * Read the password from the file
     *
     * @param subject a message that is shown on the screen just for debug
     *
     * @return the password
     *
     * @throws IOException
     */
    public String askPass(String subject) throws IOException {
        in.setLineNumber(lineNumber);

        String keyPass = in.readLine();
        System.out.println(lineNumber + "*F* Read password for " + subject);
        lineNumber = in.getLineNumber();

        return keyPass;
    }
}
