package Klava.crypto;

import java.io.*;

import java.util.*;


/**
 * Used for asking the user for the keystore and the key through the console.
 *
 * @author $author$
 * @version $Revision: 1.1.2.1 $
 */
public class KStoreTextAccess implements KStoreAccess {
    /**
     * List the key aliases of the chosen keystore
     *
     * @param store DOCUMENT ME!
     */
    protected void list(KStore store) {
        Enumeration aliases;

        try {
            aliases = store.getKeyAliases();
        } catch (Exception e) {
            return; // we simply ignore it in this case
        }

        if (aliases == null) {
            return;
        }

        System.out.println("***----- These are the keys in the KeyStore:");

        while (aliases.hasMoreElements()) {
            System.out.println("|  " + aliases.nextElement());
        }

        System.out.println("------------------- End Key List -------------");
    }

    /**
     * Request the keystore file name
     *
     * @param store the reference to the actual keystore
     *
     * @throws IOException
     * @throws java.security.GeneralSecurityException
     */
    public void askStore(KStore store)
        throws IOException, java.security.GeneralSecurityException {
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
        System.out.println("***Which Keystore do you want to use?");

        String storeName = in.readLine();
        String storePass = askPass(storeName);

        store.openKeyStore(storeName, storePass);
    }

    /**
     * Ask for the key by means of a dialog
     *
     * @param message message that will appear in the dialog
     * @param store the keystore
     *
     * @return return the info about the chosen key
     *
     * @throws IOException
     * @throws java.security.GeneralSecurityException
     */
    public KeyInfo askKey(String message, KStore store)
        throws IOException, java.security.GeneralSecurityException {
        if (!store.isKeyStoreOpen()) {
            askStore(store);
        }

        list(store);

        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
        System.out.println(message);

        String alias = in.readLine();
        String pass = askPass(alias);

        return new KeyInfo(alias, pass);
    }

    /**
     * Request a password
     *
     * @param subject for what the passphrase is asked for
     *
     * @return the inserted password
     *
     * @throws IOException
     */
    public String askPass(String subject) throws IOException {
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
        System.out.println("***Password for " + subject + "?");

        return in.readLine();
    }
}
