package Klava.crypto;

import Klava.*;

import java.io.*;

import java.security.*;

import java.util.Enumeration;
import java.util.Vector;

/**
 * A specialized tuple for containing also encrypted tuple fields.
 *
 * @author $author$
 * @version $Revision: 1.1.2.2 $
 */
public class Tuplex extends Tuple {
    /**
     * Creates a new Tuplex object.
     */
    public Tuplex() {
        super();
        setHandleRetrieved(true);
    }

    /**
     * Creates a new Tuplex object.
     *
     * @param v vector containing the initial tuple fields
     */
    public Tuplex(Vector v) {
        super(v);
        setHandleRetrieved(true);
    }

    /**
     * Copy constructor
     *
     * @param t the tuple to copy from
     */
    public Tuplex(Tuple t) {
        super(t);
        setHandleRetrieved(true);
    }

    /**
     * Encrypt all the KCipher tuple fields
     *
     * @throws IOException
     * @throws GeneralSecurityException
     *
     * @see KCipher
     */
    public void encode() throws IOException, GeneralSecurityException {
        encode("DES");
    }

    /**
     * Encrypt all the KCipher tuple fields by using the specified
     * encryption algorithm.
     *
     * @param alg the algorithm to use for the encryption
     *
     * @throws IOException
     * @throws GeneralSecurityException
     */
    public void encode(String alg) throws IOException, GeneralSecurityException {
        KlavaProcessx requester = (KlavaProcessx) Thread.currentThread();
        Key key = requester.useEncKey("Crypting: " + toString() + " alg: " +
                alg);
        _encode(key, alg);
    }

    /**
     * Encrypt all the KCipher tuple fields by using the specified
     * encryption algorithm and the specified key.
     *
     * @param key the key to use for the encryption
     * @param alg the algorithm to use for the encryption
     *
     * @throws IOException
     * @throws GeneralSecurityException
     */
    public void encode(Key key, String alg)
        throws IOException, GeneralSecurityException {
        _encode(key, alg);
    }

    /**
     * Encrypt all the KCipher tuple fields by using the specified key.
     *
     * @param key the key to use for the encryption
     *
     * @throws IOException
     * @throws GeneralSecurityException
     */
    public void encode(Key key) throws IOException, GeneralSecurityException {
        _encode(key, "DES");
    }

    /**
     * The actual encryption method that is called by the other overloaded
     * methods
     *
     * @param key the key to use for the encryption
     * @param alg the algorithm to use for the encryption
     *
     * @throws IOException
     * @throws GeneralSecurityException
     */
    protected void _encode(Key key, String alg)
        throws IOException, GeneralSecurityException {
        for (int i = 0; i < length(); i++) {
            Object Element = getItem(i);

            if (Element instanceof KCipher) {
                ((KCipher) Element).enc(key, alg);
            }
        }
    }

    /**
     * First test whether the other tuple is actually a Tuplex.
     *
     * @param toBeMatched the tuple to match with
     *
     * @return true whether this first test succeeds
     */
    public boolean preMatch(Tuple toBeMatched) {
        if (!super.preMatch(toBeMatched) ||
                !this.getClass().equals(toBeMatched.getClass())) {
            return false;
        }

        return true;
    }

    /**
     * Try to decrypt the tuple, if something fails the original tuple is not
     * touched.
     *
     * @param requester the KlavaProcess who request to decrypt the tuple
     *
     * @return true if the decryption succedded and also the matching.
     *
     * @throws GeneralSecurityException the decryption has failed.
     */
    protected boolean matchTuplex(KlavaProcessx requester)
        throws GeneralSecurityException {
        // il test di lunghezza non serve
        // si prova a decifrare la tupla
        // we act on a copy of the original one
        Tuplex tuplex = (Tuplex) clone();

        try {
            KCipher toDecode;
            Key key = requester.useDecKey("Decrypting: " + tuplex.toString());

            for (int i = 0; i < length(); i++) {
                Object MyElement = getItem(i);

                if (MyElement instanceof KCipher) {
                    // we act on a copy
                    toDecode = new KCipher((KCipher) getItem(i));
                    tuplex.setItem(i, toDecode);

                    // it cannot be called on items where there's nothing
                    // to decrypt
                    if (toDecode.encItem == null) {
                        return false;
                    }

                    byte[] decoded = toDecode.dec(key);
                    ByteArrayInputStream iStream = new ByteArrayInputStream(decoded);
                    ObjectInputStream in = new ObjectInputStream(iStream);
                    Object mystery = in.readObject();

                    if (!toDecode.ref.getClass().equals(mystery.getClass())) {
                        // anche se sono sicuramente TupleItem
                        // devono essere dello stesso tipo
                        return false;
                    } else if (toDecode.ref instanceof TupleItem) {
                        TupleItem decodedItem = (TupleItem) toDecode.ref;

                        if (decodedItem.isFormal()) {
                            // DO NOT decodedItem.setValue( mystery ) ;
                            // we are still acting on a copy
                            toDecode.ref = mystery;
                            toDecode.formal = false;
                            toDecode.encItem = null;
                            toDecode.alg = null;
                        } else if (toDecode.ref.equals(mystery)) {
                            continue;
                        } else {
                            return false;
                        }
                    }

                    toDecode.encItem = null;
                }

                //if
            }

            //for
        } catch (GeneralSecurityException e) {
            throw e;
        } catch (Exception e1) {
            e1.printStackTrace();

            return false;
        }

        // now update our vector (if we are here we successfully decrypted
        // everything :-)
        set_decrypted_fields(tuplex);

        return true;
    }

    /**
     * DOCUMENT ME!
     *
     * @param decrypted DOCUMENT ME!
     */
    protected void set_decrypted_fields(Tuplex decrypted) {
        Object elem;

        for (int i = 0; i < length(); ++i) {
            elem = decrypted.getItem(i);

            if (elem instanceof KCipher) {
                // it is a decrypted field
                ((KCipher) (getItem(i))).setValue(elem);
            }

            // else there's nothing to do
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Object clone() {
        Tuplex new_tuplex = new Tuplex((Vector) items.clone());
        new_tuplex.setTupleId(getTupleId());

        return new_tuplex;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws GeneralSecurityException DOCUMENT ME!
     */
    public boolean decode() throws GeneralSecurityException {
        KlavaProcessx requester = (KlavaProcessx) Thread.currentThread();

        return matchTuplex(requester);
    }

    /**
     * Try to decrypt the tuple, if something fails the original tuple is not
     * touched.
     *
     * @param decoded the decrypted Klava.Tuple
     *
     * @return true if the decryption succedded and also the matching.
     *
     * @throws GeneralSecurityException the decryption has failed.
     */
    public boolean decode_into(Tuple decoded) throws GeneralSecurityException {
        Tuple copy = new Tuple();

        if (decode_to_tuple(copy)) {
            Enumeration en = copy.tupleElements();

            while (en.hasMoreElements())
                decoded.add(en.nextElement());

            return true;
        } else {
            return false;
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param result DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws GeneralSecurityException DOCUMENT ME!
     */
    protected boolean decode_to_tuple(Tuple result)
        throws GeneralSecurityException {
        KlavaProcessx requester = (KlavaProcessx) Thread.currentThread();

        // we act on a copy of the original one
        Tuplex tuplex = (Tuplex) clone();

        try {
            KCipher toDecode;
            Key key = requester.useDecKey("Decrypting: " + tuplex.toString());

            for (int i = 0; i < length(); i++) {
                Object MyElement = getItem(i);

                if (MyElement instanceof KCipher) {
                    // we act on a copy
                    toDecode = new KCipher((KCipher) getItem(i));
                    tuplex.setItem(i, toDecode);

                    // it cannot be called on items where there's nothing
                    // to decrypt
                    if (toDecode.encItem == null) {
                        return false;
                    }

                    byte[] decoded = toDecode.dec(key);
                    ByteArrayInputStream iStream = new ByteArrayInputStream(decoded);
                    ObjectInputStream in = new ObjectInputStream(iStream);
                    Object mystery = in.readObject();

                    if (mystery instanceof TupleItem) {
                        result.add(mystery);
                    } else {
                        return false;
                    }
                } else { /*if not a KCipher */
                    result.add(MyElement);
                }
            }

            //for
        } catch (GeneralSecurityException e) {
            e.printStackTrace();

            return false;
        } catch (Exception e1) {
            e1.printStackTrace();

            return false;
        }

        return true;
    }
}
