// A breakpoint in the debugging session
package Klava.debug ;

import java.io.Serializable ;
import java.util.List ;
import java.util.ArrayList ;
import Klava.debug.event.BreakpointEvent ;

public abstract class Breakpoint implements Serializable {

  private int id ;
  private boolean suspend_enabled = true ;
  private List conditions = null ;

  // Breakpoint conditions provide additional constraints
  // that an agent must satisfy in order to hit a breakpoint
  public interface Condition extends Serializable {
    public boolean isSatisfiedBy( KlavaAgent a ) throws DebugException ;
  }

  public Breakpoint( int id ) {
    this.id = id ;
  }

  // Copy contructor
  public Breakpoint( Breakpoint b ) {
    id = b.id ;
    suspend_enabled = b.suspend_enabled ;
    List conds = b.conditions ;
    if ( conds != null ) {
      for ( int i = 0 ; i < conds.size() ; i++ ) {
        addCondition( ( Condition )conds.get( i ) ) ;
      }
    }
  }

  public int getId() {
    return id ;
  }

  public void setSuspendEnabled( boolean enabled ) {
    suspend_enabled = enabled ;
  }

  public boolean isSuspendEnabled() {
    return suspend_enabled ;
  }

  public void addCondition( Condition cond ) {
    if (cond != null ) {
      if ( conditions == null ) {
        conditions = new ArrayList() ;
      }
      conditions.add( cond ) ;
    }
  }

  protected boolean checkConditions( KlavaAgent a ) throws DebugException {
    if ( conditions == null )
      return true ;

    int condCount = conditions.size() ;
    boolean satisfied = true ;

    for ( int i = 0 ; satisfied && i < condCount ; i++ ) {
      satisfied = ( ( Condition ) conditions.get( i ) ).isSatisfiedBy( a ) ;
    }
    return satisfied ;
  }

  // Template method
  public final BreakpointEvent locationChanged( KlavaAgent a ) throws DebugException {
    if ( isHitBy( a ) && checkConditions( a ) ) {
      return newBreakpointEvent( a ) ;
    } else {
      return null ;
    }
  }

  public String toString() {
    String className = getClass().getName() ;
    String bkptName = className.substring( className.lastIndexOf( '.' ) + 1 ) ;
    return bkptName + " #" + id  ;
  }

  // To be defined in concrete subclasses
  public abstract Breakpoint duplicate() ;
  protected abstract boolean isHitBy( KlavaAgent a ) throws DebugException  ;
  protected abstract BreakpointEvent newBreakpointEvent( KlavaAgent a ) throws DebugException ;

}