// Debug mode process
package Klava.debug ;

import java.util.EventListener ;
import Klava.KlavaProcess ;
import Klava.KlavaException ;
import Klava.NodeInterface ;

public abstract class KlavaDebugProcess extends KlavaProcess {

  // Current location of execution
  private int currentLine = -1 ;
  private int currentColumn = -1 ;
  private transient boolean attached = false ;
  private transient StepListener stepListener = null ;

  // For objs that want to be notified of process' execution steps
  public interface StepListener extends EventListener {
    public void processStepped( KlavaDebugProcess sourceP ) throws KillException ;
  }


  public KlavaDebugProcess() {
    super();
  }

  public KlavaDebugProcess( NodeInterface n ) {
    super( n );
  }

  public void setStepListener( StepListener l ) {
    stepListener = l ;
  }

  final protected void notify_step( int line, int column )
    throws KlavaException
  {
    currentLine = line ;
    currentColumn = column ;
    // This prevents concurrent modification
    StepListener stepL = this.stepListener ;
    if ( stepL != null ) {
      stepL.processStepped( this ) ;
    }
  }

  void setAttached( boolean attached ) {
    this.attached = attached ;
  }

  public boolean isAttached() {
    return attached ;
  }

  public int getCurrentLine() {
    return currentLine ;
  }

  public int getCurrentColumn() {
    return currentColumn ;
  }

  protected final void pre_execution() {
   // To write
  }


}