package klavatest;

import Klava.*;

/**
 * test sending and loading of remote classes. The remote process is then
 * in turn sent to another site.
 */

public class testClassLoading2 extends KlavaTestApplication {
  public testClassLoading2(String name, String suffix,
                                 boolean log_to_file)
                                 throws java.io.IOException, KlavaException {
    super(name, suffix, log_to_file);

    NetNode server = new AcceptNetNode("server", 9999);
    server.start();
    addNode(server);
    server.logLoadedClasses();
    PhysicalLocality server_loc =
      new PhysicalLocality(NetUtils.createLocalNodeAddress(9999));
    addProcessToNode
      (new ReceiveFWDRemoteProcess
        (new PhysicalLocality(NetUtils.createLocalNodeAddress(12000))),
          server);

    NetNode client = new NetNode("client2", 12000);
    client.setAckNonBlocking(false); // to avoid blocking
    client.start();
    addNode(client);
    client.logLoadedClasses();
    addProcessToNode(new LoginNodeCoordinator(server_loc), client);
    ReceiveExecRemoteProcess rem_proc = new ReceiveExecRemoteProcess(server_loc);
    rem_proc.setKlavaProcName(rem_proc.KlavaProcName() + "_client");
    addProcessToNode(rem_proc, client);
    try {
      client.waitToBeConnected();
    } catch (InterruptedException ie) {
      ie.printStackTrace();
    }
  }

  public static void main(String args[]) throws Exception, KlavaException {
    KlavaTestApplication app = new testClassLoading2
      ("testClassLoading2", (args.length > 1 ? args[1] : "cl_msgout"),
        args.length > 0 && ! args[0].equals("0"));

    if (! (args.length > 0 && args[0].equals("0"))) {
      System.out.println("starting the sender application...");
      String envp[] = new String[1];
      envp[0] = "CLASSPATH=" + System.getProperty("java.class.path") +
                System.getProperty( "path.separator" ).charAt(0) + "..";

      Process P = Runtime.getRuntime().exec
        ("java classloading.sendRemoteProcess " +
          (args.length > 0 ? "1" : "") +
          (args.length > 1 ? " " + args[1] : ""), envp);
      System.out.println("sender application started");

      OutputGrabber outGrab = new OutputGrabber(P.getInputStream());
      OutputGrabber errGrab = new OutputGrabber(P.getErrorStream());
      outGrab.start();
      errGrab.start();

      int exit_value = P.waitFor();
    }

    app.waitForProcessesAndTerminate();
  }
}

// wait for a process coming from outside, and forward it to another site.
class ReceiveFWDRemoteProcess extends KlavaProcess {
  Locality remote;

  ReceiveFWDRemoteProcess(Locality l) {
    remote = l;
  }

  public void execute() throws KlavaException {
    KlavaProcessVar P = new KlavaProcessVar();
    in(P, self);
    Print("Received a remote process");
    procCall(P);
    Print("Executed a remote process");
    out(P, remote);
    Print("Forwarded to " + remote);
  }
}

// wait for a process coming from outside, eval it, and wait for it
// to terminate.
class ReceiveExecRemoteProcess extends KlavaProcess {
  Locality remote;

  ReceiveExecRemoteProcess(Locality l) {
    remote = l;
  }

  public void execute() throws KlavaException {
    KlavaProcessVar P = new KlavaProcessVar();
    in(P, self);
    Print("Received a remote process");
    procCall(P);
    Print("Remote process started");
    Print("Remote process terminated");
  }
}