package klavatest;

import Klava.*;

class NodeCreator extends NodeCoordinator {
  public void execute() throws KlavaException {
    Locality newloc = newloc(9997);
    Print("Created new locality " + newloc);
    out(newloc, self); // communicate it to the other process
    in(new KBoolean(true), self);
    try {
      Thread.sleep(1000);
      Print("Now closing " + newloc);
      removeloc(newloc);
    } catch (InterruptedException e) {
      e.printStackTrace();
    }
  }
}

// this process attempts to close a locality it does not own
class NotNodeCreator extends KlavaProcess {
  public void execute() throws KlavaException {
    Locality l = new PhysicalLocality();
    in( l, self );
    try {
      Print("Attempt to remove " + l );
      removeloc(l);
      Print("SUCCEED!!!"); // should never reach this point
    } catch (Exception e) {
      Print("Something went wrong!");
    }
    out( new KBoolean(true), self );
  }
}

public class testCloseNode extends KlavaTestApplication {
  public testCloseNode(String name, String suffix,
                       boolean log_to_file)
                       throws java.io.IOException, KlavaException {
    super(name, suffix, log_to_file);

    Node node = new Node("testCloseNode", "127.0.0.1:9998", "localhost", 9999);
    addProcessToNode(new NodeCreator(), node);
    addProcessToNode(new NotNodeCreator(), node);
  }

  public static void main(String[] args) throws Exception {
    boolean log_to_file = args.length > 0;

    if (log_to_file)
      (new Net()).start();

    KlavaTestApplication app = new testCloseNode
      ("testCloseNode", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    app.start();
    app.waitForProcessesAndTerminate();

    System.exit(0);
  }
}