package klavatest;

/**
 * test the non-automatic closure of processes
 */

import Klava.*;

public class testCloseProcess extends KlavaTestApplication {
  public testCloseProcess(String name, String suffix,
                                 boolean log_to_file)
                                 throws java.io.IOException,
                                 InterruptedException, KlavaException {
    super(name, suffix, log_to_file);

    NetNode server = new AcceptNetNode("server", 9999);
    server.start();
    server.addProcess(new ReceiveCloseProcess());
    addNode(server);

    NetNode client = new NetNode("client", 9998);
    client.addProcess(new LoginNodeCoordinator(server.ThisLocality()));
    client.start();
    client.waitToBeConnected();

    server.logForName("ProcessToClose");

    KlavaProcess P = new ProcessToClose(client.ThisLocality());
    KlavaProcess PC = new ProcessToClose(client.ThisLocality());
    PC.setKlavaProcName(PC.KlavaProcName() + "-closed");

    KlavaProcess SP = new SendProcessToClose(P, server.ThisLocality(), false);
    KlavaProcess SPC = new SendProcessToClose(PC, server.ThisLocality(), true);
    SPC.setKlavaProcName(SPC.KlavaProcName() + "-close");

    addProcessToNode(SP, client);
    addProcessToNode(SPC, client);
  }

  public static void main(String args[]) throws Exception {
    KlavaTestApplication app = new testCloseProcess
      ("testCloseProcess", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    //app.start();
    app.waitForProcessesAndTerminate();
  }
}

class ProcessToClose extends KlavaProcess {
  Locality loc;
  Locality home;

  ProcessToClose(Locality home) {
    this.home = home;
  }

  public void execute() throws KlavaException {
    loc = new PhysicalLocality();
    Print("Trying to retrieve a locality from " + self);
    read(loc, self);
    Print("Retrieved this locality: " + loc);
    Print("done");
    out(new KString("done"), home);
  }
}

class SendProcessToClose extends KlavaProcess {
  KlavaProcess P;
  boolean close;
  Locality remote;

  SendProcessToClose(KlavaProcess P, Locality remote, boolean close) {
    this.P = P;
    this.remote = remote;
    this.close = close;
  }

  public void execute() throws KlavaException {
    out(getPhysicalLocality(self), self);
    Print("Sending process " + P.KlavaProcName());
    Print("to remote site " + remote);
    KlavaProcess SP = (close ? closeProcess(P) : P);
    out( SP, remote);
    SP.setKlavaProcName(SP.KlavaProcName() + "-eval");
    eval(SP, remote);
    in(new KString(), self);
    in(new KString(), self);
    Print("remote process terminated");
  }
}

class ReceiveCloseProcess extends KlavaProcess {
  public void execute() throws KlavaException {
    out(getPhysicalLocality(self), self);
    KlavaProcessVar P;

    while (true) {
      P = new KlavaProcessVar();
      in(P, self);
      eval(P, self);
    }
  }
}
