package klavatest;

/**
 * test local subnodes, using old style nodes
 */

import Klava.*;

public class testHierarchicalSubNodes extends KlavaTestApplication {
  public testHierarchicalSubNodes(String name, String suffix,
                                 boolean log_to_file)
                                 throws java.io.IOException,
                                 InterruptedException, KlavaException {
    super(name, suffix, log_to_file);

    NetNode server = new NetNode("server", 9999);
    server.setNoLogonServer(true); // this way it does not forward messages
    // to any other server

    PhysicalLocality client1_loc =
      new PhysicalLocality(NetUtils.createLocalNodeAddress (9997));
    PhysicalLocality client2_loc =
      new PhysicalLocality(NetUtils.createLocalNodeAddress (9998));

    server.start();

    server.addToEnv("client1", client1_loc.toString());
    server.addToEnv("client2", client2_loc.toString());

    // add node coordinators for accepting sub nodes
    server.addProcess(new AcceptNodeCoordinator(true));
    server.addProcess(new AcceptNodeCoordinator(true));

    Node client1 = new Node("client1", client1_loc, "localhost", 9999);
    Node client2 = new Node("client2", client2_loc, "localhost", 9999);

    client1.start();
    client2.start();

    client1.waitToBeConnected();
    client2.waitToBeConnected();

    //server.addSubNode(client1);
    //server.addSubNode(client2);

    KlavaProcess P1 = new SendAndRetrieve
      (new LogicalLocality ("client1"));
    KlavaProcess P2 = new SendAndRetrieve
      (new LogicalLocality ("client2"));
    P2.setKlavaProcName(P2.KlavaProcName() + "2");

    addProcessToNode(P1, server);
    addProcessToNode(P2, server);
  }

  public static void main(String args[]) throws Exception {
    KlavaTestApplication app = new testHierarchicalSubNodes
      ("testHierarchicalSubNodes", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    //app.start();
    app.waitForProcessesAndTerminate();
  }
}

class SendAndRetrieve extends KlavaProcess {
  protected Locality loc;
  protected KString test_string = new KString("TEST");

  public SendAndRetrieve(Locality loc) {
    this.loc = loc;
  }

  public void execute() throws KlavaException {
    Print("Sending to " + loc + " ...");
    out(test_string, loc);
    Print("Retrieving string \"" + test_string + "\" from " + loc + "...");
    in(test_string, loc);
    Print("Retrieved.");
  }
}