package klavatest;

import Klava.*;

/**
 * the node coordinator synchronizes with a process before connecting
 * and disconnecting (with login/logout) from two different servers
 */

public class testLoginLogoutSyncProc extends KlavaTestApplication {
  public testLoginLogoutSyncProc(String name, String suffix,
                                 boolean log_to_file)
                                 throws java.io.IOException,
                                 KlavaException {
    super(name, suffix, log_to_file);

    NetNode server = new AcceptNetNode("server", 9999);
    addNode(server);

    NetNode server2 = new AcceptNetNode("server2", 9998);
    addNode(server2);

    NetNode client = new NetNode("client", 11000);
    addProcessToNode(new LoginLogoutSynchProcess(), client);
    addProcessToNode(new LoginLogoutCommSynchProcess(null), client);
  }

  public static void main(String args[]) throws Exception {
    KlavaTestApplication app = new testLoginLogoutSyncProc
      ("testLoginLogoutSyncProc", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    app.start();
    app.waitForProcessesAndTerminate();
  }
}

class LoginLogoutSynchProcess extends NodeCoordinator {
  public void execute() throws KlavaException {
    boolean result;

    KString command;
    Locality server;

    while (true) {
      command = new KString();
      server = new PhysicalLocality();
      in(command, server, self);
      if (command.equals("login")) {
        Print("Trying to login to " + server);
        result = login(server);
        Print("Login result: " + result);
        out(server, new KBoolean(result), self);
      } else if (command.equals("logout")) {
        Print("logout from " + server);
        logout(server);
        Print("Logged off.");
        out(server, new KBoolean(true), self);
      } else if (command.equals("end")) {
        Print("Finished." + command);
        break;
      }
    }
  }
}

class LoginLogoutCommSynchProcess extends KlavaProcess {
  PhysicalLocality server;
  PhysicalLocality server2;

  public LoginLogoutCommSynchProcess(String server) throws KlavaException {
    if (server != null) {
      this.server =
        new PhysicalLocality(NetUtils.createNodeAddress
          (server, 9999));
      this.server2 =
        new PhysicalLocality(NetUtils.createNodeAddress
          (server, 9998));
    } else {
      this.server =
          new PhysicalLocality(NetUtils.createNodeAddress
          (NetUtils.getLocalIPAddress(), 9999));
      this.server2 =
          new PhysicalLocality(NetUtils.createNodeAddress
          (NetUtils.getLocalIPAddress(), 9998));
    }
  }

  public void execute() throws KlavaException {
    KString login = new KString("login");
    KString logout = new KString("logout");
    KString end = new KString("end");

    if (comm_login(server))
      comm_loc_rem("Test1", server);

    comm_logout(server);

    if (comm_login(server2))
      comm_loc_rem("Test2", server2);

    comm_logout(server2);

    if (comm_login(server))
      comm_loc_rem("Test3", server);

    Print("OK.");
    out(end, server, self);
  }

  protected void comm_loc_rem(String str, Locality loc) throws KlavaException {
    Tuple t = new Tuple();
    KString s = new KString(str);
    t.add(s);
    t.add(loc);
    Print("Sending " + t + " to " + loc);
    out( t, loc );
    Print("Retrieving " + t + " from " + loc);
    in( t, loc );
    Print("Retrieved " + t );
  }

  protected boolean comm_login(Locality server) throws KlavaException {
    Print("Asking to login to " + server);
    out(new KString("login"), server, self);
    KBoolean result = new KBoolean();
    in(server, result, self);
    return result.booleanValue();
  }

  protected void comm_logout(Locality server) throws KlavaException {
    Print("Asking to logout from " + server);
    out(new KString("logout"), server, self);
    KBoolean result = new KBoolean();
    in(server, result, self);
  }
}