package klavatest;

/**
 * This client will sometimes lose connection to the server.
 * When the user presses ENTER on the keyboard, a NodeCoordinator
 * will try to reconnect.
 * Meanwhile a process outs an incremented number to the remote server
 * and tries to retrieve it.  When the client is not connected this
 * process is blocked, until the client reconnects.
 */

import Klava.*;

public class testLostConnectionAndReconnect {
  public static void main(String args[]) throws Exception {
    String server = null;

    if (args.length > 0)
      server = args[0];

    NetNode n = new NetNode("lost_client", 11000);
    n.addProcess(new ReconnectProcess(server));
    n.addProcess(new CommunicateWithServerThatDisconnect(server));
    n.start();
  }
}

class ReconnectProcess extends NodeCoordinator {
  PhysicalLocality server;

  public ReconnectProcess(String server) throws KlavaException {
    if (server != null)
      this.server =
        new PhysicalLocality(NetUtils.createNodeAddress
          (server, 9999));
    else
      this.server =
      new PhysicalLocality(NetUtils.createNodeAddress
    (NetUtils.getLocalIPAddress(), 9999));
  }

  public void execute() throws KlavaException {
    while (true) {
      Print("PRESS ENTER when ready to login to " + server);
      try { while (System.in.read() != '\n') ; }
      catch(java.io.IOException ioe) {}

      Print("Trying to login to " + server);
      boolean result = login(server);

      Print("Login result: " + result);
    }
  }
}

class CommunicateWithServerThatDisconnect extends KlavaProcess {
  Locality server;

  public CommunicateWithServerThatDisconnect(String server)
  throws KlavaException {
    if (server != null)
      this.server = new PhysicalLocality(NetUtils.createNodeAddress
        (server, 9999));
    else
      this.server = new PhysicalLocality(NetUtils.createNodeAddress
    (NetUtils.getLocalIPAddress(), 9999));
  }

  public CommunicateWithServerThatDisconnect(Locality server) {
    this.server = server;
  }

  public void execute() throws KlavaException {
    int counter = 0;
    while (true) {
      pause(1000);
      KInteger i = new KInteger(++counter);
      out(i, server);
      Print("Sent " + i);
      in(i, server);
      Print("Received " + i);
    }
  }
}