package klavatest;

/**
 * Title:        Klava: Klaim in Java
 * Description:  The implementation of Klaim in Java
 * Copyright:    Copyright (c) 2001, Lorenzo Bettini
 * Company:      Dipartimento Sistemi e Informatica, Universita' di Firenze
 * @author Lorenzo Bettini
 * @version 1.0
 */

 /**
  * test what happens if a physical locality is not present in the
  * net: an exception
  */

import Klava.*;

class testNPL extends KlavaProcess {
  public void execute() throws KlavaException {
    Print ("*** Process Started ***");
    Tuple t = new Tuple (new KString ());
    Locality loc = new PhysicalLocality ("127.0.0.1:9995"); // no loc
    setNotifyOnNonExistentLocalities(false);
    Print("Trying to accessing locality " + loc);
    if (! in_nb(t, loc))
      Print("Retrieval FAILED!");
    else
      Print("Retrieval OK!"); // should not come here

    setNotifyOnNonExistentLocalities(true);
    try {
      in(t, loc);
      Print("Success!"); // should not come here
    } catch (KlavaException e) {
      Print("Received exception " + e.getMessage());
    }
  }
}

public class testNoPhysLoc extends KlavaTestApplication {
  public testNoPhysLoc(String name, String suffix,
                        boolean log_to_file)
                        throws java.io.IOException, KlavaException {
    super(name, suffix, log_to_file);

    Node n1 = new Node( "n1", "127.0.0.1:9998", "localhost", 9999 ) ;

    addProcessToNode( new testNPL (), n1 ) ;
  }

  static public void main( String args[] ) throws Exception {
    boolean log_to_file = args.length > 0;

    if (log_to_file)
      (new Net()).start();

    testNoPhysLoc app = new testNoPhysLoc
      ("testNoPhysLoc", (args.length > 1 ? args[1] : ""),
        args.length > 0);

    app.start();
    app.waitForProcessesAndTerminate();

    System.exit(0);
  }
}