package klavatest;

/**
 * test dispatching of messages in a hierarchical way
 *
 *                       server
 *                    ^           ^
 *          printer_manager      printer_client
 *        ^
 *    printer
 *
 * printer_client issues a tuple for printer and server has to
 * deliver it to printer, though printer is not directly connected to it.
 */

import Klava.*;

public class testPhysLocReach extends KlavaTestApplication {
  public testPhysLocReach(String name, String suffix,
                                 boolean log_to_file)
                                 throws java.io.IOException,
                                 InterruptedException, KlavaException {
    super(name, suffix, log_to_file);

    NetNode server = new RegisterNetNode("server", 9999);

    NetNode printer_manager = new RegisterNetNode("printer_manager", 9998);

    NetNode printer_client = new NetNode("printer_client", 11000);
    NetNode printer = new NetNode("printer", 10001);

    PhysicalLocality server_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 9999));

    PhysicalLocality printer_manager_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 9998));

    PhysicalLocality printer_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 10001));

    server.addToEnv("printer", printer_loc.toString());

    addNode(server);

    server.start();
    printer_manager.start();
    printer.start();
    printer_client.start();

    printer.addProcess(new SubscriberNodeCoordinator(printer_manager_loc,
        new LogicalLocality("printer")));
    printer.waitToBeConnected();

    printer_manager.addProcess(new SubscriberNodeCoordinator(server_loc,
        new LogicalLocality("printer_manager")));
    printer_manager.waitToBeConnected();

    printer_client.addProcess(new SubscriberNodeCoordinator(server_loc,
        new LogicalLocality("printer_client")));
    printer_client.waitToBeConnected();

    addProcessToNode(new PrinterServer(), printer);
    addProcessToNode(new PrinterProcess(), printer_client);
  }

  public static void main(String args[]) throws Exception {
    KlavaTestApplication app = new testPhysLocReach
      ("testPhysLocReach", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    //app.start();
    app.waitForProcessesAndTerminate();
  }
}

class PrinterProcess extends KlavaProcess {
  Locality printer = new LogicalLocality("printer");

  public PrinterProcess() {}

  public PrinterProcess(String printer) {
    this.printer = new LogicalLocality(printer);
  }

  public void execute() throws KlavaException {
    Print("Printing to " + printer);
    PhysicalLocality printer_ploc = getPhysicalLocality(printer);
    Print("Real address " + printer_ploc);

    KString print_string = new KString("print");
    KString file = new KString("article.ps");

    Print("Printing file " + file);

    try {
      Tuple t = new Tuple(print_string, file);
      out( t, printer );

      Tuple result = new Tuple(new KString("done"), file);

      in( result, printer );
      Print("Printing Done!");
    } catch (KlavaPhysicalLocalityException ke) {
      Print("Printing failed...");
      Print("Locality " + printer_ploc + " does not exist!");
    }
  }
}

class PrinterServer extends KlavaProcess {
  public void execute() throws KlavaException {
    KString file = new KString();
    in(new KString("print"), file, self);
    Print("Received for printing: " + file);
    out(new KString("done"), file, self);
    Print("Printing done");
  }
}