package klavatest;

/**
 * test dispatching of messages in a hierarchical way
 *
 *                       server
 *                    ^           ^
 *          printer_manager      printer_client
 *        ^
 *    printer
 *
 * printer_client issues a tuple for printer and server has to
 * deliver it to printer, though printer is not directly connected to it.
 * the printer connects to printer_manager after printer_manager
 * has already logged on the server.
 */

import Klava.*;

public class testPhysLocReach2 extends KlavaTestApplication {
  public testPhysLocReach2(String name, String suffix,
                                 boolean log_to_file)
                                 throws java.io.IOException,
                                 InterruptedException, KlavaException {
    super(name, suffix, log_to_file);

    NetNode server = new RegisterNetNode("server", 9999);
    NetNode printer_manager = new RegisterNetNode("printer_manager", 9998);

    NetNode printer_client = new NetNode("printer_client", 11000);
    NetNode printer = new NetNode("printer", 10001);

    PhysicalLocality server_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 9999));

    PhysicalLocality printer_manager_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 9998));

    PhysicalLocality printer_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 10001));

    server.addToEnv("printer", printer_loc.toString());

    addNode(server);

    server.start();
    printer_manager.start();
    printer.start();
    printer_client.start();

    printer_manager.addProcess(new SubscriberNodeCoordinator(server_loc,
        new LogicalLocality("printer_manager")));
    printer_manager.waitToBeConnected();

    printer_client.addProcess(new SubscriberNodeCoordinator(server_loc,
        new LogicalLocality("printer_client")));
    printer_client.waitToBeConnected();

    printer.addProcess(new SubscriberNodeCoordinator(printer_manager_loc,
        new LogicalLocality("printer")));
    printer.waitToBeConnected();
    // the printer connects to printer_manager after the printer_manager
    // has connected to the server

    addProcessToNode(new PrinterServer(), printer);
    addProcessToNode(new PrinterProcess(), printer_client);
  }

  public static void main(String args[]) throws Exception {
    KlavaTestApplication app = new testPhysLocReach2
      ("testPhysLocReach2", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    //app.start();
    app.waitForProcessesAndTerminate();
  }
}