package klavatest;

/**
 * test dispatching of messages in a hierarchical way
 *
 *                       server
 *                    ^           ^
 *          printer_manager      client_manager
 *        ^                                 ^
 *    printer                               client
 *
 * printer_client issues a tuple for printer and server has to
 * deliver it to printer, though printer is not directly connected to it.
 */

import Klava.*;

public class testPhysLocReach3 extends KlavaTestApplication {
  public testPhysLocReach3(String name, String suffix,
                                 boolean log_to_file)
                                 throws java.io.IOException,
                                 InterruptedException, KlavaException {
    super(name, suffix, log_to_file);

    NetNode server = new RegisterNetNode("server", 9999);
    NetNode printer_manager = new RegisterNetNode("printer_manager", 9998);

    NetNode client_manager = new RegisterNetNode("client_manager", 20000);
    NetNode printer_client = new NetNode("printer_client", 20001);

    NetNode printer = new NetNode("printer", 10001);

    PhysicalLocality server_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 9999));

    PhysicalLocality printer_manager_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 9998));

    PhysicalLocality printer_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 10001));

    PhysicalLocality client_manager_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 20000));

    server.addToEnv("printer", printer_loc.toString());

    addNode(server);

    server.start();
    printer_manager.start();
    printer.start();
    client_manager.start();
    printer_client.start();

    printer.addProcess(new SubscriberNodeCoordinator(printer_manager_loc,
        new LogicalLocality("printer")));
    printer.waitToBeConnected();

    printer_manager.addProcess(new SubscriberNodeCoordinator(server_loc,
        new LogicalLocality("printer_manager")));
    printer_manager.waitToBeConnected();

    client_manager.addProcess(new SubscriberNodeCoordinator(server_loc,
        new LogicalLocality("client_manager")));
    client_manager.waitToBeConnected();

    printer_client.addProcess(new SubscriberNodeCoordinator(client_manager_loc,
        new LogicalLocality("printer_client")));
    printer_client.waitToBeConnected();

    addProcessToNode(new PrinterServer(), printer);
    addProcessToNode(new PrinterProcess(), printer_client);
  }

  public static void main(String args[]) throws Exception {
    KlavaTestApplication app = new testPhysLocReach3
      ("testPhysLocReach3", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    //app.start();
    app.waitForProcessesAndTerminate();
  }
}
