package klavatest;

/**
 * test dispatching of messages in a hierarchical way
 *
 *                       server
 *                    ^           ^
 *          printer_manager      client_manager
 *        ^                                 ^
 *    printer                               client
 *
 * printer_client issues a tuple for printer and server has to
 * deliver it to printer, though printer is not directly connected to it.
 * The printer disconnect, so the client should fail in printing.
 */

import Klava.*;

public class testPhysLocReach4 extends KlavaTestApplication {
  public testPhysLocReach4(String name, String suffix,
                                 boolean log_to_file)
                                 throws java.io.IOException,
                                 InterruptedException, KlavaException {
    super(name, suffix, log_to_file);

    NetNode server = new RegisterNetNode("server", 9999);
    server.setNoLogonServer(true); // this way it does not forward messages
    // to any other server

    NetNode printer_manager = new RegisterNetNode("printer_manager", 9998);

    NetNode client_manager = new RegisterNetNode("client_manager", 20000);
    NetNode printer_client = new NetNode("printer_client", 20001);

    NetNode printer = new NetNode("printer", 10001);

    PhysicalLocality server_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 9999));

    PhysicalLocality printer_manager_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 9998));

    PhysicalLocality printer_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 10001));

    PhysicalLocality client_manager_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 20000));

    server.addToEnv("printer", printer_loc.toString());

    addNode(server);

    server.start();
    printer_manager.start();
    printer.start();
    client_manager.start();
    printer_client.start();

    LogicalLocality printer_log_loc = new LogicalLocality("printer");

    printer.addProcess(new PrinterSubscriber(printer_manager_loc,
        printer_log_loc));
    printer.waitToBeConnected();

    printer_manager.addProcess(new SubscriberNodeCoordinator(server_loc,
        new LogicalLocality("printer_manager")));
    printer_manager.waitToBeConnected();

    printer.addProcess(new PrinterUnsubscriber(printer_manager_loc,
        printer_log_loc));

    client_manager.addProcess(new SubscriberNodeCoordinator(server_loc,
        new LogicalLocality("client_manager")));
    client_manager.waitToBeConnected();

    printer_client.addProcess(new SubscriberNodeCoordinator(client_manager_loc,
        new LogicalLocality("printer_client")));
    printer_client.waitToBeConnected();

    //addProcessToNode(new PrinterServer(), printer);
    KlavaProcess printer_proc = new PrinterProcess();
    printer_proc.setNotifyOnNonExistentLocalities(true);
    addProcessToNode(printer_proc, printer_client);
  }

  public static void main(String args[]) throws Exception {
    KlavaTestApplication app = new testPhysLocReach4
      ("testPhysLocReach4", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    //app.start();
    app.waitForProcessesAndTerminate();
  }
}

class PrinterUnsubscriber extends UnsubscriberNodeCoordinator {
  public PrinterUnsubscriber(PhysicalLocality server,
                             LogicalLocality my_loc) {
    super (server, my_loc);
  }

  public void execute() throws KlavaException {
    in(new KString("SUBSCRIBED"), self);
    in(new KString("UNSUBSCRIBE"), self);
    super.execute();
  }
}

class PrinterSubscriber extends SubscriberNodeCoordinator {
  public PrinterSubscriber(PhysicalLocality server,
                           LogicalLocality my_loc) {
    super (server, my_loc);
  }

  public void subscribe_success(PhysicalLocality server, LogicalLocality loc) {
    try {
      out(new KString("SUBSCRIBED"), self);
      out(new KString("UNSUBSCRIBE"), self);
    } catch (KlavaException e) {
      e.printStackTrace();
    }
  }
}
