package klavatest;

/**
 * test dispatching of messages in a hierarchical way
 *
 *                       server
 *                    ^           ^
 *          printer_manager      printer_client
 *        ^
 *    printer
 *
 * printer_client issues a tuple for printer and server has to
 * deliver it to printer, though printer is not directly connected to it.
 */

import Klava.*;

public class testPhysLocReachLogin extends KlavaTestApplication {
  public testPhysLocReachLogin(String name, String suffix,
                                 boolean log_to_file)
                                 throws java.io.IOException,
                                 InterruptedException, KlavaException {
    super(name, suffix, log_to_file);

    NetNode server = new AcceptNetNode("server", 9999);
    NetNode printer_manager = new AcceptNetNode("printer_manager", 9998);

    NetNode printer_client = new AcceptNetNode("printer_client", 11000);
    NetNode printer = new AcceptNetNode("printer", 10001);

    PhysicalLocality server_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 9999));

    PhysicalLocality printer_manager_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 9998));

    PhysicalLocality printer_loc =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 10001));

    server.addToEnv("printer", printer_loc.toString());

    addNode(server);

    server.start();
    printer_manager.start();
    printer.start();
    printer_client.start();

    printer.addProcess(new LoginNodeCoordinator(printer_manager_loc));
    printer.waitToBeConnected();

    printer_manager.addProcess(new LoginNodeCoordinator(server_loc));
    printer_manager.waitToBeConnected();

    printer_client.addProcess(new LoginNodeCoordinator(server_loc));
    printer_client.waitToBeConnected();

    addProcessToNode(new PrinterServer(), printer);
    addProcessToNode(new PrinterProcess(), printer_client);
  }

  public static void main(String args[]) throws Exception {
    KlavaTestApplication app = new testPhysLocReachLogin
      ("testPhysLocReachLogin", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    //app.start();
    app.waitForProcessesAndTerminate();
  }
}