package klavatest;

/**
 * A NetNode is started and a NodeCoordinator notifies with a tuple
 * when a node registers.
 * Another node subscribes to this node.
 * A process in the RegisterNetNode waits for the notification and communicates
 * with this new node, assuming that its logical locality is l2
 */

import Klava.*;

public class testRegisterNetNodeLocalProc extends KlavaTestApplication {
  public static void main(String args[]) throws Exception {
    KlavaTestApplication app = new testRegisterNetNodeLocalProc
      ("testRegisterNetNodeLocalProc", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    app.start();
    app.waitForProcessesAndTerminate();
  }

  public testRegisterNetNodeLocalProc(String name, String suffix,
                                    boolean log_to_file)
                                    throws java.io.IOException, KlavaException {
    super(name, suffix, log_to_file);

    NetNode n1 = new NetNode("n1", 9999);
    addProcessToNode(new NotifyNodeCoordinator(), n1);
    addProcessToNode(new CommunicateWithRegisteredProcess(), n1);

    NetNode n2 = new NetNode("l2", 11000);
    addProcessToNode(new SubscribeProcess(), n2);
  }
}

class NotifyNodeCoordinator extends NodeCoordinator {
  public void execute() throws KlavaException {
    PhysicalLocality ploc = new PhysicalLocality();
    LogicalLocality lloc = new LogicalLocality();
    if (register(ploc, lloc)) {
      Print("Registered: {" +  lloc + "~" + ploc + "}");
      try { Thread.sleep(1000); } // give time to the node to start
      catch ( InterruptedException e ) { e.printStackTrace(); }
      out( new KString("entered"), self);
    } else
      Print("Error in registering a node!");
  }
}

class CommunicateWithRegisteredProcess extends KlavaProcess {
  LogicalLocality l2 = new LogicalLocality("l2");

  public void execute() throws KlavaException {
    in( new KString("entered"), self );
    Print("New node registered; now try to communicate with it");

    KString hello = new KString("hello");
    out( hello, l2 );
    KString s = new KString();
    in( s, l2 );
    Print("Retrieved from registered node: " + s);
  }
}

class SubscribeProcess extends NodeCoordinator {
  PhysicalLocality server;

  public SubscribeProcess() throws KlavaException {
    server =
        new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), 9999));
  }

  public SubscribeProcess(int port) throws KlavaException {
    server =
      new PhysicalLocality(NetUtils.createNodeAddress
        (NetUtils.getLocalIPAddress(), port));
  }

  public void execute() throws KlavaException {
    Print("Trying to subscribe to " + server);
    boolean result = subscribe(server, new LogicalLocality("l2"));

    Print("Subscribe result: " + result);
  }
}