package klavatest;

/**
 * A NetNode is started and a NodeCoordinator notifies with a tuple
 * when a node registers.
 * Another node subscribes to this node.
 * A process in the RegisterNetNode waits for the notification and communicates
 * with this new node, assuming that its logical locality is l2, then waits
 * for a response that has to be delivered in the RegisterNetNode.
 * This response will be delivered by a process executing at the registered
 * node.
 */

import Klava.*;

public class testRegisterNetNodeRemoteProc extends KlavaTestApplication {
  public static void main(String args[]) throws Exception {
    KlavaTestApplication app = new testRegisterNetNodeRemoteProc
      ("testRegisterNetNodeRemoteProc", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    app.start();
    app.waitForProcessesAndTerminate();
  }

  public testRegisterNetNodeRemoteProc(String name, String suffix,
                                    boolean log_to_file)
                                    throws java.io.IOException, KlavaException {
    super(name, suffix, log_to_file);

    NetNode n1 = new NetNode("n1", 9999);
    addProcessToNode(new NotifyNodeCoordinator(), n1);
    addProcessToNode(new CommunicateWithRegisteredRemoteProcess(), n1);

    NetNode n2 = new NetNode("l2", 11000);
    addProcessToNode(new SubscribeProcess(), n2);
    addProcessToNode(new CommunicateWithServerProcess(), n2);
  }
}

class CommunicateWithRegisteredRemoteProcess extends KlavaProcess {
  LogicalLocality l2 = new LogicalLocality("l2");

  public void execute() throws KlavaException {
    in( new KString("entered"), self );
    Print("New node registered; now try to communicate with it");

    KString hello = new KString("Welcome");
    out( hello, getPhysicalLocality(self), l2 );
    Print("Sent " + hello );
    KString s = new KString();
    in( s, self );
    Print("Retrieved " + s );
  }
}

class CommunicateWithServerProcess extends KlavaProcess {
  public void execute() throws KlavaException {
    KString hello = new KString();
    PhysicalLocality remote = new PhysicalLocality();
    in( hello, remote, self );
    Print("Received " + hello + " from " + remote);
    KString thanks = new KString("Thank you!");
    out( thanks, remote);
    Print("Sent back " + thanks );
  }
}