package klavatest;

/**
 * a registered node tries to translate some localities;
 * some can be solved locally and some remotely
 */

import Klava.*;

public class testRemoteAndLocalLocTrans extends KlavaTestApplication {
  public testRemoteAndLocalLocTrans(String name, String suffix,
                                    boolean log_to_file)
                                    throws java.io.IOException, KlavaException {
    super(name, suffix, log_to_file);

    NetNode n1 = new NetNode("server", 9999);
    n1.addToEnv("server", n1.ThisLocality().toString());
    n1.addToEnv("printer", n1.ThisLocality().toString());
    n1.addToEnv("screen", n1.ThisLocality().toString());
    addProcessToNode(new NotifyNodeCoordinator(), n1);

    NetNode n2 = new NetNode("l2", 11000);
    n2.addToEnv("screen", n2.ThisLocality().toString());
    addProcessToNode(new SubscribeProcess(), n2);
    addProcessToNode(new LocSolverProcess(), n2);
  }

  public static void main(String args[]) throws Exception {
    KlavaTestApplication app = new testRemoteAndLocalLocTrans
      ("testRemoteAndLocalLocTrans", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    app.start();
    app.waitForProcessesAndTerminate();
  }
}

class LocSolverProcess extends KlavaProcess {
  public void execute() throws KlavaException {
    comm_loc_rem("Test1", new LogicalLocality("server"));
    comm_loc_rem("Test2", new LogicalLocality("screen"));
    comm_loc_rem("Test2", new LogicalLocality("printer"));
  }

  protected void comm_loc_rem(String str, Locality loc) throws KlavaException {
    Tuple t = new Tuple();
    KString s = new KString(str);
    KString retr = new KString();
    t.add(s);
    t.add(loc);
    Print("Sending " + t + " to " + loc);
    out( t, loc );
    Print("Retrieving " + t + " from " + loc);
    Tuple retrieved = new Tuple(retr, loc);
    in( retrieved, loc );
    Print("Retrieved " + retrieved );
  }
}