package klavatest;

import Klava.*;
import java.util.*;
import momi.*;

/**
 * test sending and receive of a MoMi class
 */

public class testSimpleMoMi extends KlavaTestApplication {
  public testSimpleMoMi(String name, String suffix,
                                 boolean log_to_file)
                                 throws java.io.IOException, KlavaException {
    super(name, suffix, log_to_file);

    NetNode server = new NetNode("server", 9999);
    server.start();
    addNode(server);

    PhysicalLocality server_loc =
      new PhysicalLocality(NetUtils.createLocalNodeAddress(9999));
    addProcessToNode(new SendMoMiClass(), server);
    addProcessToNode(new SendMoMiMixin(), server);
    addProcessToNode(new ReceiveMoMiClass(), server);
  }

  public static void main(String args[]) throws Exception {
    KlavaTestApplication app = new testSimpleMoMi
      ("testSimpleMoMi", (args.length > 1 ? args[1] : "momi_msgout"),
        args.length > 0 && ! args[0].equals("0"));

    app.waitForProcessesAndTerminate();
  }
}

class SendMoMiClass extends KlavaProcess {
    Locality l = self;

    SimpleType string_type = new SimpleType("string");
    SimpleType int_type = new SimpleType("int");
    SimpleType void_type = new SimpleType("void");

    public SendMoMiClass() {}

    public SendMoMiClass(Locality l) {
        this.l = l;
    }

    public void execute() throws KlavaException {
        Ctest1 C = new Ctest1();
        Ctest2 C2 = new Ctest2();

        Vector m_par = new Vector();
        m_par.addElement(int_type);

        Vector n_par = new Vector();

        MethodType m = new MethodType("m", m_par, int_type);
        MethodType n = new MethodType("n", n_par, int_type);

        RecordType rect1 = new RecordType();
        rect1.addMethod(m);
        rect1.addMethod(n);

        RecordType rect2 = new RecordType();
        rect2.addMethod(n);

        ClassType c1 = new ClassType(rect1);
        ClassType c2 = new ClassType(rect2);

        MoMiTupleItem tc = new MoMiTupleItem(c1, C);
        MoMiTupleItem tc2 = new MoMiTupleItem(c2, C2);

        Print("Inserting the following classes:");
        Print("c1 = " + tc);
        Print("c2 = " + tc2);

        out(tc2, self);
        out(tc, self);

        Print("done");
    }
}

class SendMoMiMixin extends KlavaProcess {
    Locality l = self;

    SimpleType string_type = new SimpleType("string");
    SimpleType int_type = new SimpleType("int");
    SimpleType void_type = new SimpleType("void");

    public SendMoMiMixin() {}

    public SendMoMiMixin(Locality l) {
        this.l = l;
    }

    public void execute() throws KlavaException {
        Mixin M = new Mtest1();

        Vector m_par = new Vector();
        m_par.addElement(int_type);

        Vector n_par = new Vector();

        MethodType m = new MethodType("m", m_par, int_type);
        MethodType n = new MethodType("n", n_par, int_type);

        RecordType old = new RecordType();
        old.addMethod(m);

        RecordType red = new RecordType();
        red.addMethod(m);

        RecordType exp = new RecordType();
        exp.addMethod(n);

        MixinType m1 = new MixinType(null, red, exp, old);

        MoMiTupleItem mc = new MoMiTupleItem(m1, M);

        Print("Inserting the following mixins:");
        Print("M = " + mc);

        out(mc, self);

        Print("done");
    }
}

class ReceiveMoMiClass extends KlavaProcess {
    Locality l = self;

    SimpleType string_type = new SimpleType("string");
    SimpleType int_type = new SimpleType("int");
    SimpleType void_type = new SimpleType("void");

    public ReceiveMoMiClass() {}

    public ReceiveMoMiClass(Locality l) {
        this.l = l;
    }

    public void execute() throws KlavaException {
        Vector m_par = new Vector();
        m_par.addElement(int_type);

        Vector n_par = new Vector();

        MethodType m = new MethodType("m", m_par, int_type);
        MethodType n = new MethodType("n", n_par, int_type);

        RecordType rect1 = new RecordType();
        rect1.addMethod(m);
        rect1.addMethod(n);

        ClassType c1 = new ClassType(rect1);

        MoMiTupleItem tc = new MoMiTupleItem(c1);

        Print("waiting for a class of type " + c1);
        in(tc, self);
        Print("received this class " + tc);

        MoMiClass C = (MoMiClass) tc.getValue();
        MoMiObject ref = C.new_instance();
        MoMiObject pass_self;

        try {
            Print("Created object: " + ref);

            MoMiMethod meth = ref.get_method("m");
            pass_self = ref.get_self("m");
            meth.pass_argument(new Integer(100));
            meth.invoke(pass_self);
            Print("result of m: " + meth.get_result());

            meth.pass_argument(new Integer(200));
            meth.invoke(pass_self);
            Print("result of m: " + meth.get_result());

            meth = ref.get_method("n");
            pass_self = ref.get_self("n");
            meth.invoke(pass_self);
            Print("result of n: " + meth.get_result());

            RecordType old = new RecordType();
            old.addMethod(m);

            RecordType red = new RecordType();
            red.addMethod(m);

            RecordType exp = new RecordType();
            exp.addMethod(n);

            MixinType m1 = new MixinType(null, red, exp, old);

            MoMiTupleItem mc = new MoMiTupleItem(m1);

            Print("waiting for a mixin of type " + m1);
            in(mc, self);
            Print("received this mixin " + mc);
            Mixin M = (Mixin) mc.getValue();

            Print("Applying mixin M to class C");
            MoMiClass MC = Mixin.apply(M, C);
            Print("MC: " + MC);

            ref = MC.new_instance();

            Print("Created object: " + ref);

            meth = ref.get_method("m");
            pass_self = ref.get_self("m");
            meth.pass_argument(new Integer(300));
            meth.invoke(pass_self);
            Print("result of m: " + meth.get_result());

            /**
             * there is a run-time name clash between M2 and MC,
             * since both M2 and C (and thus also MC) defines n.
             * However n is "statically" bound to M2.n in any object
             * instantiated from M2.
             */

            Mixin M2 = new M2test1();

            Print("M2 = " + M2);
            Print("Applying mixin M2 to class MC");
            MoMiClass M2MC = Mixin.apply(M2, MC);
            Print("M2MC: " + M2MC);

            ref = M2MC.new_instance();

            Print("Created object: " + ref);

            meth = ref.get_method("m");
            pass_self = ref.get_self("m");

            meth.pass_argument(new Integer(500));
            meth.invoke(pass_self);
            Print("result of m: " + meth.get_result());

            meth = ref.get_method("n");
            pass_self = ref.get_self("n");
            meth.pass_argument("200");
            meth.pass_argument(new Integer(100));
            meth.invoke(pass_self);
            Print("result of n: " + meth.get_result());
        } catch (NoSuchMethodException nm) {
          nm.printStackTrace();
          Print("got exception: " + nm.getMessage());
        }
    }
}

class Ctest1 extends MoMiClass {
    protected static Hashtable _methods = null;

    public Ctest1() {
        if (_methods == null) {
            _methods = new Hashtable();
            set_methods(_methods);

            set_method("m", new Ctest1_m());
            set_method("n", new Ctest1_n());
        } else
            set_methods(_methods);
    }

    protected MoMiObject _new_instance() {
        return new Ctest1Object();
    }
}

class Ctest2 extends MoMiClass {
    protected static Hashtable _methods = null;

    public Ctest2() {
        if (_methods == null) {
            _methods = new Hashtable();
            set_methods(_methods);

            set_method("n", new Ctest1_n());
        } else
            set_methods(_methods);
    }
}

class Ctest1Object extends MoMiObject {
    protected Integer i = new Integer(10);
    protected String foo = "foo";

    public Integer get_i() {
        return i;
    }

    public void set_i(Integer i) {
        this.i = i;
    }

    public String get_foo() {
        return foo;
    }

    public void set_foo(String foo) {
        this.foo = foo;
    }
}

class Ctest1_m extends MoMiMethod {
    public Ctest1_m() {
        super("m", DEFINED);
    }

    public void invoke(MoMiObject _self) {
        Ctest1Object self = (Ctest1Object) _self;
        Integer j = (Integer) stack.pop();

        self.i = j;

        stack.push(self.i);
    }
}

class Ctest1_n extends MoMiMethod {
    public Ctest1_n() {
        super("n", DEFINED);
    }

    public void invoke(MoMiObject _self) {
        Ctest1Object self = (Ctest1Object) _self;

        System.out.println("Ctest1.n()");

        stack.push(self.i);
    }
}

class Mtest1 extends Mixin {
    public Mtest1() {
        set_method("m", new Mtest1_m());
        set_method("n", new MoMiMethod("n", MoMiMethod.EXPECTED));
    }

    public MoMiObject _new_instance() {
        return new Mtest1Object();
    }
}

class Mtest1Object extends MoMiObject {
    protected Boolean b = new Boolean(false);

    public Boolean get_b() {
        return b;
    }

    public void set_Boolean(Boolean b) {
        this.b = b;
    }
}

class Mtest1_m extends MoMiMethod {
    public Mtest1_m() {
        super("m", REDEFINED);
    }

    public void invoke(MoMiObject _self) throws NoSuchMethodException {
        Mtest1Object self = (Mtest1Object) _self;
        Integer j = (Integer) stack.pop();

        MoMiObject next = self.get_next();
        MoMiMethod next_m = self.get_next_method("m");
        next_m.pass_argument(j);
        next_m.invoke(next);

        System.out.println("Mtest1.m()");

        MoMiMethod meth = self.get_method("n");
        MoMiObject pass_self = self.get_self("n");
        meth.invoke(pass_self);

        stack.push(meth.get_result());
    }
}

// another mixin, M2

class M2test1 extends Mixin {
    public M2test1() {
        set_method("m", new MoMiMethod("m", MoMiMethod.EXPECTED)); // expected
        set_method("n", new M2test1_n()); // defined
    }

    public MoMiObject _new_instance() {
        return new M2test1Object();
    }
}

class M2test1Object extends MoMiObject {
    protected Integer i;
    protected Integer k;
}

class M2test1_n extends MoMiMethod {
    public M2test1_n() {
        super("n", DEFINED);
    }

    /**
     * n(int j, String j2) {
     *      i = j;
     *      k = (int) j2;
     *      return (i + k);
     * }
     */
    public void invoke(MoMiObject _self) throws NoSuchMethodException {
        M2test1Object self = (M2test1Object) _self;
        Integer j = (Integer) stack.pop();
        String j2 = (String) stack.pop();

        System.out.println("M2test1.n()");

        self.i = j;
        self.k = new Integer(Integer.parseInt(j2));

        stack.push(new Integer(self.i.intValue() + self.k.intValue()));
    }
}