package klavatest;

/**
 * Title:        Klava: Klaim in Java
 * Description:  The implementation of Klaim in Java
 * Copyright:    Copyright (c) 2001, Lorenzo Bettini
 * Company:      Dipartimento Sistemi e Informatica, Universita' di Firenze
 * @author Lorenzo Bettini
 * @version 1.0
 */

 /**
 * Test a subnode of a node: the subnode will connect afterwards
 * The process waiting for a tuple at the subnode is blocked, until
 * the tuple is added, but no exception is thrown even if it's not
 * connected, since everything takes place locally with no
 * network communication.
 */

import Klava.*;

class testSBP extends KlavaProcess {
  public void execute() throws KlavaException {
    Print("*** Process Started ***");
    KString s = new KString();
    in( s, new PhysicalLocality ("127.0.0.1:9996")); // subNode
    Print("Retrieved: " + s);
  }
}

public class testSubNode extends KlavaTestApplication {
  Node node1;
  Node node2;

  public testSubNode(String name, String suffix,
                    boolean log_to_file)
                    throws java.io.IOException, InterruptedException, KlavaException {
    super(name, suffix, log_to_file);

    node1 = new Node("testSubNode", "127.0.0.1:9998", "localhost", 9999);
    node2 = new Node("subNode", "127.0.0.1:9996", "localhost", 9999);

    node1.start();
    node1.waitToBeConnected();

    node1.addSubNode (node2);
    node2.out(new Tuple(new KString("hi")));
    node2.start ();
    node2.waitToBeConnected();

    addNode(node2);

    addProcessToNode (new testSBP (), node1);
  }

  static public void main( String args[] ) throws Exception {
    boolean log_to_file = args.length > 0;

    if (log_to_file)
      (new Net()).start();

    testSubNode app = new testSubNode
      ("testSubNode", (args.length > 1 ? args[1] : ""),
        args.length > 0);

    System.out.println
      ("node1's connection status: " +
        app.node1.getConnectionStatus()); // this returns when connected

    app.waitForProcessesAndTerminate();

    System.exit(0);
  }
}