package klavatest;

import Klava.*;

/**
 * the node coordinator synchronizes with a process before connecting
 * and disconnecting (with subscribe/unsubscribe) from two different servers
 */

public class testSubUnsubSyncProc extends KlavaTestApplication {
  public testSubUnsubSyncProc(String name, String suffix,
                                 boolean log_to_file)
                                 throws java.io.IOException, KlavaException {
    super(name, suffix, log_to_file);

    NetNode server = new RegisterNetNode("server", 9999);
    addNode(server);

    NetNode server2 = new RegisterNetNode("server2", 9998);
    addNode(server2);

    NetNode client = new NetNode("client", 11000);
    addProcessToNode(new SubUnsubSynchProcess(), client);
    addProcessToNode(new SubUnsubCommSynchProcess(null), client);
  }

  public static void main(String args[]) throws Exception {
    KlavaTestApplication app = new testSubUnsubSyncProc
      ("testSubUnsubSyncProc", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    app.start();
    app.waitForProcessesAndTerminate();
  }
}

class SubUnsubSynchProcess extends NodeCoordinator {
  public void execute() throws KlavaException {
    boolean result;

    KString command;
    PhysicalLocality server;
    LogicalLocality client = new LogicalLocality("client");

    while (true) {
      command = new KString();
      server = new PhysicalLocality();
      in(command, server, self);
      if (command.equals("subscribe")) {
        Print("Trying to subscribe to " + server);
        result = subscribe(server, client);
        Print("Subscribe result: " + result);
        out(server, new KBoolean(result), self);
      } else if (command.equals("unsubscribe")) {
        Print("unsubscribe from " + server);
        unsubscribe(server, client);
        Print("Logged off.");
        out(server, new KBoolean(true), self);
      } else if (command.equals("end")) {
        Print("Finished." + command);
        break;
      }
    }
  }
}

class SubUnsubCommSynchProcess extends KlavaProcess {
  PhysicalLocality server;
  PhysicalLocality server2;

  public SubUnsubCommSynchProcess(String server) throws KlavaException {
    if (server != null) {
      this.server =
        new PhysicalLocality(NetUtils.createNodeAddress
          (server, 9999));
      this.server2 =
        new PhysicalLocality(NetUtils.createNodeAddress
          (server, 9998));
    } else {
      this.server =
          new PhysicalLocality(NetUtils.createNodeAddress
          (NetUtils.getLocalIPAddress(), 9999));
      this.server2 =
          new PhysicalLocality(NetUtils.createNodeAddress
          (NetUtils.getLocalIPAddress(), 9998));
    }
  }

  public void execute() throws KlavaException {
    KString subscribe = new KString("subscribe");
    KString unsubscribe = new KString("unsubscribe");
    KString end = new KString("end");

    if (comm_subscribe(server))
      comm_loc_rem("Test1", server);

    comm_unsubscribe(server);

    if (comm_subscribe(server2))
      comm_loc_rem("Test2", server2);

    comm_unsubscribe(server2);

    if (comm_subscribe(server))
      comm_loc_rem("Test3", server);

    Print("OK.");
    out(end, server, self);
  }

  protected void comm_loc_rem(String str, Locality loc) throws KlavaException {
    Tuple t = new Tuple();
    KString s = new KString(str);
    t.add(s);
    t.add(loc);
    Print("Sending " + t + " to " + loc);
    out( t, loc );
    Print("Retrieving " + t + " from " + loc);
    in( t, loc );
    Print("Retrieved " + t );
  }

  protected boolean comm_subscribe(Locality server) throws KlavaException {
    Print("Asking to subscribe to " + server);
    out(new KString("subscribe"), server, self);
    KBoolean result = new KBoolean();
    in(server, result, self);
    return result.booleanValue();
  }

  protected void comm_unsubscribe(Locality server) throws KlavaException {
    Print("Asking to unsubscribe from " + server);
    out(new KString("unsubscribe"), server, self);
    KBoolean result = new KBoolean();
    in(server, result, self);
  }
}