package com;

import orc.error.runtime.TokenException;
import orc.runtime.sites.EvalSite;
import orc.runtime.sites.DotSite;
import orc.runtime.sites.Site;
import orc.runtime.sites.ThreadedSite;
import orc.runtime.*;
import Klava.*;


public class orcNode extends EvalSite{
	public Object evaluate (Args args) throws TokenException{
		return new orcNodeInstance(args.stringArg(0),
                                           args.intArg(1),
                                           args.stringArg(2),
                                           args.intArg(3));
	}
}

class orcNodeInstance extends DotSite {
	private String loc;
	private NetNode klavaNode;
	private Integer nport;
	private String server;
	
	
	orcNodeInstance (String loc,Integer nport,String server,Integer port){
	   try{
		this.loc = loc;
		this.nport = nport;
		this.server = server;

		System.out.println("Create Node" + server+":"+loc+"-"+port);
		klavaNode = new Node( loc, nport, server, port );
		
		addMember("in", new InMethod());
		addMember("out", new OutMethod());
		addMember("read", new ReadMethod());
		addMember("inp",new InpMethod());
		addMember("readp",new ReadpMethod());
		addMember("eval", new EvalMethod());
		addMember("getLocality", new GetLocalityMethod());
		addMember("getPort",new GetPortMethod());
		addMember("addEnv",new AddEnvironmentMethod());
		
		klavaNode.start();
		
	   } catch(Exception e) {
		System.out.println("Error in the Klava node creation");
		e.printStackTrace();
	   }
	}
	
	
	private class InMethod extends ThreadedSite{
		public Object evaluate(Args args) throws TokenException{
			Tuple t= null; 
			try {
				t = (Tuple) args.getArg(0);
				Locality l = (Locality) args.getArg(1);
				klavaNode.in(t,l);
			} catch (KlavaException e) {
				e.printStackTrace();
			}
			return t;
		}
	}
	
	private class InpMethod extends ThreadedSite{
		public Object evaluate(Args args) throws TokenException{
			Tuple t = null;
			try {
				t = (Tuple) args.getArg(0);
				Locality l = (Locality) args.getArg(1);
		    	Integer timeout = args.intArg(2);
		    	
				boolean a = klavaNode.in_t(t,l,timeout);
		    	if (a == false){
		    		return new Tuple(new KBoolean(false));
		    	}
		    	
			} catch (KlavaException e) {
				e.printStackTrace();
			}
			return t;
		}
	}
	
	private class ReadMethod extends ThreadedSite {
		public Object evaluate(Args args)throws TokenException{
			Tuple t = null;
			try {
				t = (Tuple) args.getArg(0);
		    	Locality l = (Locality) args.getArg(1);
		    	
		    	klavaNode.read(t,l);
		    	
			} catch (KlavaException e) {
				e.printStackTrace();
			}
			return t;
		}
	}
	
	private class ReadpMethod extends ThreadedSite {
		public Object evaluate(Args args)throws TokenException{
			Tuple t = null;
			try {
				t = (Tuple) args.getArg(0);
		    	Locality l = (Locality) args.getArg(1);
		    	Integer timeout = args.intArg(2);
		    	
				boolean a = klavaNode.read_t(t,l,timeout);
		    	if (a == false){
		    		return new Tuple(new KBoolean(false));
		    	}
			} catch (KlavaException e) {
				e.printStackTrace();
			}
			return t;
		}
	}
	
	private class OutMethod extends EvalSite{
		public Object evaluate(Args args) throws TokenException{
		    try {
		    	Tuple t = (Tuple) args.getArg(0);
		    	Locality l = (Locality) args.getArg(1);
		    	
		    	klavaNode.out(t,l);
		    	
			} catch (KlavaException e) {
				e.printStackTrace();
			}
			return signal();
		}	
	}
	
	private class EvalMethod extends EvalSite{
		public Object evaluate(Args args) throws TokenException{
			try {
						
				KlavaProcess p = (KlavaProcess) args.getArg(0);
				
				Locality loc = (Locality) args.getArg(1);
				klavaNode.eval(p,loc);
				
		    	
			} catch (KlavaException e) {
				e.printStackTrace();
			}
			return signal();
		}
	}

	private class GetLocalityMethod extends Site{
		public void callSite(Args args, Token token) throws TokenException{
			token.resume(loc);
		}
	}
	
	private class GetPortMethod extends Site{
		public void callSite(Args args, Token token) throws TokenException{
			token.resume(nport);
		}
	}
	
	private class AddEnvironmentMethod extends EvalSite{
		public Object evaluate(Args args) throws TokenException{
			try{
			klavaNode.addToEnv(args.stringArg(0),
                                           NetUtils.createNodeAddress(server+":"+args.intArg(1)));
			}catch(KlavaException e){
				e.printStackTrace();
			}
			return signal();
		}
	}
	
	protected void addMembers() {
	}
	
}