package com;

import java.lang.reflect.Constructor;

import orc.error.runtime.TokenException;
import orc.runtime.sites.EvalSite;
import orc.runtime.sites.DotSite;
import orc.runtime.sites.Site;
import orc.runtime.*;
import Klava.*;

public class orcTuple extends EvalSite{
	public Object evaluate (Args args) throws TokenException{
		return new orcTupleInstance();
	}
}


class orcTupleInstance extends DotSite{

	orcTupleInstance(){
		addMember("tuple", new TupleMethod() );
		addMember("stringFormal", new StringFormalMethod());
		addMember("intFormal", new IntFormalMethod());
		addMember("boolFormal", new BooleanFormalMethod());
		addMember("get",new getMethod());
		addMember("loadProcess", new loadProcessMethod());
		addMember("locality", new localityMethod());
		addMember("equals", new EqualsMethod() );
	}

	private class localityMethod extends Site{
		public void callSite(Args args, Token token)throws TokenException{
			try{
				token.resume(new LogicalLocality(args.stringArg(0)));
			}catch(Exception e){
				e.printStackTrace();
			}
		}
	}

	private class TupleMethod extends Site{
		public void callSite(Args args, Token token) throws TokenException{
			try {
				Tuple t = new Tuple();
				for (int i = 0; i< args.size(); i++){
					if (args.getArg(i) instanceof java.lang.String){
						KString g = new KString(args.stringArg(i));
						t.add(g);
					}
					if (args.getArg(i) instanceof java.lang.Integer ){ 
						KInteger g = new KInteger(args.intArg(i));
						t.add(g);
					}
					if (args.getArg(i)instanceof java.math.BigInteger){
						KInteger g = new KInteger(((java.math.BigInteger)
                                                                           args.getArg(i)).intValue());
						t.add(g);
					}
					if (args.getArg(i) instanceof java.lang.Boolean){
						KBoolean g = new KBoolean(args.boolArg(i));
						t.add(g);
					}
					if (args.getArg(i)instanceof Klava.TupleItem){
						t.add(args.getArg(i));
					}
				}
				token.resume(t);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private class StringFormalMethod extends Site{
		public void callSite(Args args, Token token) throws TokenException{
			try {
				token.resume(new KString());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	private class BooleanFormalMethod extends Site{
		public void callSite(Args args, Token token) throws TokenException{
			try {
				token.resume(new KBoolean());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	private class IntFormalMethod extends Site{
		public void callSite(Args args, Token token) throws TokenException{
			try {
				token.resume(new KInteger());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private class loadProcessMethod extends Site{
		public void callSite(Args args, Token token)throws TokenException{
			try{
				Object o;
				if (args.size() == 1){
					Class process = Class.forName(args.stringArg(0));
					o = process.newInstance();
				}else{
					Class process = Class.forName(args.stringArg(0));
					Class paramtype[] = new Class[args.size()-1];
					Object arguments[] = new Object[args.size()-1];					
					for (int i = 1; i < args.size(); i++){
						if (args.getArg(i) instanceof java.math.BigInteger ){
							paramtype[i-1] = KInteger.class;
							arguments[i-1] = new KInteger(
                                                                              ((java.math.BigInteger)
                                                                               args.getArg(i)).intValue());
						}
						if (args.getArg(i) instanceof java.lang.Integer ){
							paramtype[i-1] = KInteger.class;
							arguments[i-1] = new KInteger(args.intArg(i));
						}
						if (args.getArg(i) instanceof java.lang.String ){
							paramtype[i-1] = KString.class;
							arguments[i-1] = new KString(args.stringArg(i));
						}
						if (args.getArg(i) instanceof java.lang.Boolean ){
							paramtype[i-1] = KBoolean.class;
							arguments[i-1] = new KBoolean(args.boolArg(i));
						}
						if (args.getArg(i) instanceof KInteger 
                                                    || args.getArg(i) instanceof KBoolean 
                                                    || args.getArg(i) instanceof KString){
							paramtype[i-1] = args.getArg(i).getClass();
							arguments[i-1] = args.getArg(i);
						}
					}
					Constructor ct = process.getConstructor(paramtype);
					o = ct.newInstance(arguments);	
				}
				token.resume(o);

			}catch(Exception e){
				e.printStackTrace();
			}
		}
	}

	private class EqualsMethod extends Site{
		public void callSite (Args args, Token token) throws TokenException{
			try{
				TupleItem tupleitem = (TupleItem)args.getArg(0);
				Object val = null;
				if (args.getArg(1) instanceof java.lang.Integer 
                                    || args.getArg(1) instanceof java.math.BigInteger){
					val = new KInteger(args.intArg(1));
				}
				if (args.getArg(1) instanceof java.lang.Integer 
                                    || args.getArg(1) instanceof java.math.BigInteger){
					val = new KInteger(args.intArg(1));
				}
				if (args.getArg(1) instanceof java.lang.String){
					val = new KString(args.stringArg(1));
				}
				if (args.getArg(1) instanceof java.lang.Boolean){
					val = new KBoolean(args.boolArg(1));
				}
				token.resume(tupleitem.equals(val));
			}catch(Exception e){
				e.printStackTrace();
			}
		}
	}

	private class getMethod extends Site {
		public void callSite (Args args, Token token) throws TokenException{
			try{
				Tuple tuple = (Tuple)args.getArg(0);
				token.resume(tuple.getItem(args.intArg(1)));
			}catch(Exception e){
				e.printStackTrace();
			}
		}
	}

	protected void addMembers() {
	}
}


