package momi;

import java.util.Hashtable;
import java.util.Enumeration;


/**
 * A container of methods' table.  It is used as the base class for
 * Mixin and MoMiClass and also for MoMiObject.
 *
 * @version $Revision: 1.10.2.2 $
 * @author $author$
 */
public class WithMethods implements java.io.Serializable {
    /** The table of methods.  The key is the name of the method
     *  and the value is a MoMiMethod object
     *  @see momi.MoMiMethod */
    protected Hashtable methods = new Hashtable();

    public WithMethods() {}

    public WithMethods(Hashtable methods) {
        this.methods = methods;
    }

    /**
     * Clone the table of methods.  This is a shallow copy, so the
     * MoMiMethod objects are not cloned.
     *
     * @return a (shallow) copy of the method table
     */
    public final Hashtable clone_methods() {
        return (Hashtable) methods.clone();
    }

    public final int size() {
        return methods.size();
    }

    /**
     * Return the method table
     *
     * @return the method table
     */
    public final Hashtable get_methods() {
        return (Hashtable) methods;
    }

    /**
     * Set the method table
     *
     * @param methods the new method table
     */
    public final void set_methods(Hashtable methods) {
        this.methods = methods;
    }

    /**
     * Set a method in the method table with a specific name
     *
     * @param name the name of the method
     * @param meth the MoMiMethod object representing the method
     */
    public final void set_method(String name, MoMiMethod meth) {
        methods.put(name, meth);
    }

    /**
     * Retrieve a method given its name
     *
     * @param name the name of the method to retrieve
     *
     * @return the MoMiMethod object representing the requested method
     *
     * @throws NoSuchMethodException if the requested method is not found
     */
    public final MoMiMethod get_method(String name) throws NoSuchMethodException {
        MoMiMethod method = (MoMiMethod) methods.get(name);

        if (method == null) {
            throw new NoSuchMethodException(name);
        }

        return method;
    }

    /**
     * Retrieve a method given its name
     *
     * @param name the name of the method to retrieve
     *
     * @return the MoMiMethod object representing the requested method or null
     * if it is not found
     */
    final MoMiMethod retrieve_method(String name)  {
        MoMiMethod method = (MoMiMethod) methods.get(name);

        return method;
    }

    /**
     * Test the presence of a method
     *
     * @param name the name of the method to look for
     *
     * @return true if the method is found, false otherwise
     */
    public final boolean contains_method(String name) {
        return methods.containsKey(name);
    }

    /**
     * Turn the method table into a String
     *
     * @return the string representation of this method table
     */
    public String toString() {
        return _with_methods_string() + " = \n" + _toString(2);
    }

    /**
     * Return a string denoting the kind of this method table
     * (e.g. class or mixin)
     *
     * @return the string denoting the kind of this method table
     */
    protected String _with_methods_string() {
        return "";
    }

    /**
     * Turn the method table into a String
     *
     * @param indent the indentation level
     *
     * @return the string representation of this method table
     */
     String _toString(int indent) {
        String indent_str = MoMiPrinter.build_indent(indent);

        StringBuffer buffer = new StringBuffer();

        Enumeration meths = methods.elements();
        while (meths.hasMoreElements()) {
            buffer.append(((MoMiMethod)meths.nextElement())._toString(indent));
            if (meths.hasMoreElements())
                buffer.append('\n');
        }

        return buffer.toString();
    }
}