/**
 * Agent
 *
 * retrieves tuplex with a mobile agent
 * extending from KlavaProcess
 * It stores the tuplex in a TupleSpace delivered
 * to the AgentHome for decryption
 */

package test_crypto;

import java.io.* ;
import java.util.Vector ;
import Klava.* ;
import Klava.crypto.*;

class AgentProc1 extends KlavaProcessx {

    public AgentProc1() throws Exception {
        super (new KStoreFileAccess("Agent1.answers"));
        //super (new KStoreDialogAccess());
    }

    public void execute() throws KlavaException {
        try {
            setEncKey("1") ;
            Tuplex tx1 = new Tuplex() ;
            tx1.add(new KString("lorenzo")) ;
            KString s1 = new KString("chiave des1") ;
            tx1.add(new KCipher(s1)) ;
            tx1.encode();
            out(tx1,self) ;
            Print("first encoded tuple inserted");

            setEncKey("2");
            Tuplex tx2 = new Tuplex() ;
            tx2.add(new KString("bettini")) ;
            KString s2 = new KString("chiave des2") ;
            tx2.add(new KCipher(s2)) ;
            tx2.encode();
            out(tx2,self) ;
            Print("second encoded tuple inserted");
            unsetEncKey();
        } catch (Exception e) {
          Print("problems while encoding");
          e.printStackTrace ();
        }
    }
}

class AgentProc2 extends KlavaProcessx {

  public AgentProc2() throws Exception {
        super (new KStoreFileAccess("Agent1.answers"));
        //super (new KStoreDialogAccess());
    }

    public void execute() throws KlavaException {
        try {
            setEncKey("1") ;
            Tuplex tx1 = new Tuplex() ;
            tx1.add(new KString("daniele")) ;
            KInteger s1 = new KInteger(1) ;
            tx1.add(new KCipher(s1)) ;
            tx1.encode();
            out(tx1,self) ;
            Print("first encoded tuple inserted");

            setEncKey("2");
            Tuplex tx2 = new Tuplex() ;
            tx2.add(new KString("becchi")) ;
            KInteger s2 = new KInteger(2) ;
            tx2.add(new KCipher(s2)) ;
            tx2.encode();
            out(tx2,self) ;
            Print("second encoded tuple inserted");
            unsetEncKey();
        } catch (Exception e) {
          Print("problems while encoding");
          e.printStackTrace ();
        }
    }
}

class AgentAgentHome extends KlavaProcessx {

    protected Locality dati1 = new LogicalLocality("data1");
    protected Locality dati2 = new LogicalLocality("data2");

    public AgentAgentHome() throws Exception {
        super (new KStoreFileAccess("Agent2.answers"));
        //super (new KStoreDialogAccess());
    }

    public void execute() throws KlavaException {

        PhysicalLocality myLoc = getPhysicalLocality(self);

        KlavaProcess agent = new AgentAgent(myLoc, dati2);
        agent.setKlavaProcName(agent.KlavaProcName() + "-" +
                               dati1);
        eval( agent, dati1 ) ;
        in( new KString("DONE"), self ) ;
        TupleSpace collectedData = new TupleSpace();
        in( collectedData, self );

        while ( true ) {
            Tuplex txf1 = new Tuplex() ;
            KString s1 = new KString() ;
            txf1.add( s1 ) ;
            KString fString1 = new KString() ;
            KCipher k = new KCipher( fString1 ) ;
            txf1.add( k ) ;
            try {
                collectedData.in(txf1, 10) ;
            } catch ( KlavaTimeOutException e ) {
                Print("No more collected data" ) ;
                break ;
            }
            boolean badkey = true ;
            while ( badkey ) {
                try {
                    if ( txf1.decode() ) {
                      Print("decoded: " + txf1);
                      Print("s1 = " + s1);
                      Print("fString1 = " + fString1);
                      out( s1, self ) ;
                      out( fString1, self );
                      out( txf1 ) ;
                    } else {
                      Print("decription failed for " + s1);
                      txf1.add("DECRIPTION FAILED") ;
                      out( txf1 ) ;
                    }
                    badkey = false ;
                } catch (java.security.InvalidKeyException e1) {
                    e1.printStackTrace ();
                } catch (java.security.UnrecoverableKeyException e2) {
                    e2.printStackTrace ();
                } catch (javax.crypto.BadPaddingException e4) {
                    e4.printStackTrace ();
                } catch ( Exception e3 ) {
                  e3.printStackTrace ();
                  Print("decription failed for " + s1);
                  txf1.add("DECRIPTION FAILED") ;
                  out( txf1 ) ;
                  badkey = false ;
                }
            }
        }
    }
}

class AgentAgent extends KlavaProcess {

    protected Locality owner ;
    protected Locality secondStep ;
    protected boolean done = false ;
    //private TupleSpace collectedData ;
    private TupleSpace collectedData ;

    public AgentAgent( Locality from, Locality to ) {
        owner = from ;
        secondStep = to ;
        collectedData = new TupleSpace() ;
    }

    public void execute() throws KlavaException {
        Tuplex txf1 = new Tuplex() ;
        KString s1 = new KString() ;
        txf1.add( s1 ) ;
        KString fString1 = new KString() ;
        KCipher k = new KCipher( fString1 ) ;
        txf1.add( k ) ;

        Print("arrived at " + getPhysicalLocality(self));
        in(txf1, self) ;
        Print("Retrieved encrypted data");
        collectedData.out(txf1) ;

        if ( ! done ) {
            done = true ;
            Print("migration to " + secondStep);
            setKlavaProcName(KlavaProcName() + "-" + secondStep);
            eval(this, secondStep) ;
        } else {
            Print("itinerary is finished");
            out(collectedData, owner) ;
            out(new KString("DONE"), owner) ;
        }
    }
}

public class Agent extends KlavaTestApplication {
  public Agent(String name, String suffix,
               boolean log_to_file)
      throws Exception {
    super(name, suffix, log_to_file);

    Node Node1 = new Node ("node1", 9990, "localhost", 9999 );
    Node Node2 = new Node ("node2", 9980, "localhost", 9999 );
    Node Node3 = new Node ("node3", 9970, "localhost", 9999 );

    Node1.setLocalMatcher (new KCipherLocalMatcher ());
    Node1.setLocalTupleSpaceMatcher
        (new KCipherLocalTupleSpaceMatcher ());
    Node1.addToEnv("data2", 9980);
    Node1.start();
    Node1.waitToBeConnected();
    addProcessToNode (new AgentProc1(), Node1);

    Node2.setLocalMatcher (new KCipherLocalMatcher ());
    Node2.setLocalTupleSpaceMatcher
        (new KCipherLocalTupleSpaceMatcher ());
    Node2.start ();
    Node2.waitToBeConnected();
    addProcessToNode (new AgentProc2(), Node2);

    Node3.setLocalMatcher (new KCipherLocalMatcher ());
    Node3.setLocalTupleSpaceMatcher
        (new KCipherLocalTupleSpaceMatcher ());
    Node3.addToEnv("data1", 9990);
    Node3.start ();
    Node3.waitToBeConnected();
    addProcessToNode (new AgentAgentHome(), Node3);
  }

  static public void main  (String args[]) throws Exception {
    boolean log_to_file = args.length > 0;

    if (log_to_file)
      (new Net()).start();

    KlavaTestApplication app = new Agent
      ("Agent", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    app.waitForProcessesAndTerminate();
  }
}
