/**
 * Agentk
 *
 * retrieves tuplex with a mobile agent
 * extending from KlavaProcessx
 * There are no KCipherMatcher setted: this version uses ink
 * to decode automatically the tuples
 */

package test_crypto;

import java.io.* ;
import java.util.Vector ;
import Klava.* ;
import Klava.crypto.*;


class AgentkProc1 extends KlavaProcessx {

    //protected Locality dati2 = new LogicalLocality("data2");

  public AgentkProc1() throws Exception {
        super (new KStoreFileAccess("Agent1.answers"));
        //super (new KStoreDialogAccess());
    }

    public void execute() throws KlavaException {
        try {
            setEncKey("1") ;
            Tuplex tx1 = new Tuplex() ;
            tx1.add(new KString("lorenzo")) ;
            KString s1 = new KString("chiave des1") ;
            tx1.add(new KCipher(s1)) ;
            tx1.encode();
            out(tx1,self) ;
            Print("first encoded tuple inserted");

            setEncKey("2");
            Tuplex tx2 = new Tuplex() ;
            tx2.add(new KString("bettini")) ;
            KString s2 = new KString("chiave des2") ;
            tx2.add(new KCipher(s2)) ;
            tx2.encode();
            out(tx2,self) ;
            Print("second encoded tuple inserted");
            unsetEncKey();
        } catch (Exception e) {
          Print("problems while encoding");
          e.printStackTrace ();
        }
    }
}

class AgentkProc2 extends KlavaProcessx {

  public AgentkProc2() throws Exception {
        super (new KStoreFileAccess("Agent1.answers"));
        //super (new KStoreDialogAccess());
    }

    public void execute() throws KlavaException {
        try {
            setEncKey("1") ;
            Tuplex tx1 = new Tuplex() ;
            tx1.add(new KString("daniele")) ;
            KInteger s1 = new KInteger(1) ;
            tx1.add(new KCipher(s1)) ;
            tx1.encode();
            out(tx1,self) ;
            Print("first encoded tuple inserted");

            setEncKey("2");
            Tuplex tx2 = new Tuplex() ;
            tx2.add(new KString("becchi")) ;
            KInteger s2 = new KInteger(2) ;
            tx2.add(new KCipher(s2)) ;
            tx2.encode();
            out(tx2,self) ;
            Print("second encoded tuple inserted");
            unsetEncKey();
        } catch (Exception e) {
          Print("problems while encoding");
          e.printStackTrace ();
        }
    }
}

class AgentkAgentHome extends KlavaProcessx {

    protected Locality dati1 = new LogicalLocality("data1");
    protected Locality dati2 = new LogicalLocality("data2");

    public AgentkAgentHome() throws Exception {
        super (new KStoreFileAccess("Agent2.answers"));
        //super (new KStoreDialogAccess());
    }

    public void execute() throws KlavaException {

        PhysicalLocality myLoc = getPhysicalLocality(self);

        try {
          KlavaProcess agent = new AgentkAgent(myLoc, dati2);
          agent.setKlavaProcName(agent.KlavaProcName() + "-" + dati1);
          eval( agent, dati1 ) ;
        } catch (Exception e) {
            e.printStackTrace ();
        }
        in( new KString("DONE"), self ) ;
        TupleSpace collectedData = new TupleSpace();
        in( collectedData, self );

        while ( true ) {
            Tuplex txf1 = new Tuplex() ;
            KString s1 = new KString() ;
            txf1.add( s1 ) ;
            KString fString1 = new KString() ;
            KCipher k = new KCipher( fString1 ) ;
            txf1.add( k ) ;
            try {
                collectedData.in(txf1, 10) ;
                out(txf1, self) ;
            } catch ( KlavaTimeOutException e ) {
                Print("No more collected data" ) ;
                break ;
            }
        }

        Tuplex txf1 = new Tuplex() ;
        KString s1 = new KString() ;
        txf1.add( s1 ) ;
        KString fString1 = new KString() ;
        KCipher k = new KCipher( fString1 ) ;
        txf1.add( k ) ;
        while ( true ) {
            txf1.resetOriginalTemplate();
            try {
                readk(txf1,self, 10) ;
                Print("decoded: " + txf1);
                Print("s1 = " + s1);
                Print("fString1 = " + fString1);
                out( s1, self ) ;
                out( fString1, self );
                out( txf1 ) ;
            } catch ( KlavaTimeOutException e ) {
                Print("All possible data decoded" ) ;
                break ;
            }
        }
    }
}

class AgentkAgent extends KlavaProcessx {

    protected Locality owner ;
    protected Locality secondStep ;
    protected boolean done = false ;
    private TupleSpace collectedData ;

    public AgentkAgent( Locality from, Locality to ) throws Exception {
        owner = from ;
        secondStep = to ;
        collectedData = new TupleSpace() ;
    }

    public void execute() throws KlavaException {
        Tuplex txf1 = new Tuplex() ;
        KString s1 = new KString() ;
        txf1.add( s1 ) ;
        KString fString1 = new KString() ;
        KCipher k = new KCipher( fString1 ) ;
        txf1.add( k ) ;
        try {
          Print("arrived at " + getPhysicalLocality(self));
          in(txf1, self) ;
          Print("retrieved encrypted data");
          //collectedData.add(txf1) ;
          collectedData.out(txf1) ;
        } catch (Exception e) {
            e.printStackTrace ();
        }
        if ( ! done ) {
            done = true ;
            Print("migration to " + secondStep);
            setKlavaProcName(KlavaProcName() + "-" + secondStep);
            eval(this, secondStep) ;
        } else {
            Print("itinerary is finished");
            out(collectedData, owner) ;
            out(new KString("DONE"), owner) ;
        }
    }
}

public class Agentk extends KlavaTestApplication {
  public Agentk(String name, String suffix,
               boolean log_to_file)
      throws Exception {
    super(name, suffix, log_to_file);

    Node Node1 = new Node ("node1", 9990, "localhost", 9999 );
    Node Node2 = new Node ("node2", 9980, "localhost", 9999 );
    Node Node3 = new Node ("node3", 9970, "localhost", 9999 );

    Node1.addToEnv("data2", 9980);
    Node1.start();
    Node1.waitToBeConnected();
    addProcessToNode (new AgentkProc1(), Node1);

    Node2.start ();
    Node2.waitToBeConnected();
    addProcessToNode (new AgentkProc2(), Node2);

    Node3.addToEnv("data1", 9990);
    Node3.start ();
    Node3.waitToBeConnected();
    addProcessToNode (new AgentkAgentHome(), Node3);
  }

  static public void main  (String args[]) throws Exception {
    boolean log_to_file = args.length > 0;

    if (log_to_file)
      (new Net()).start();

    KlavaTestApplication app = new Agentk
      ("Agentk", (args.length > 1 ? args[1] : ""),
        args.length > 0);
    app.waitForProcessesAndTerminate();
  }
}
