/**
 * crypto_agent
 *
 * retrieves tuplex with a mobile agent
 * extending from KlavaProcessx
 * There are no KCipherMatcher setted: this version uses ink
 * to decode automatically the tuples
 */

package cryptoagent;

import java.io.* ;
import java.util.Vector ;
import Klava.* ;
import Klava.crypto.*;


class Node1Proc extends KlavaProcessx {

    public Node1Proc() throws Exception {
        super (new KStoreFileAccess("Agent1.answers"));
    }

    public void execute() throws KlavaException {
        try {
            setEncKey("1") ;
            Tuplex tx1 = new Tuplex() ;
            tx1.add(new KString("item1")) ;
            KString s1 = new KString("10000") ;
            tx1.add(new KCipher(s1)) ;
            tx1.encode();
            out(tx1,self) ;

            setEncKey("2");
            Tuplex tx2 = new Tuplex() ;
            tx2.add(new KString("item2")) ;
            KString s2 = new KString("20000") ;
            tx2.add(new KCipher(s2)) ;
            tx2.encode();
            out(tx2,self) ;
            unsetEncKey();
        } catch (Exception e) {
            e.printStackTrace ();
        }
    }
}

class Node2Proc extends KlavaProcessx {

    public Node2Proc() throws Exception {
        super (new KStoreFileAccess("Agent1.answers"));
    }

    public void execute() throws KlavaException {
        try {
            setEncKey("1") ;
            Tuplex tx1 = new Tuplex() ;
            tx1.add(new KString("item3")) ;
            KInteger s1 = new KInteger(1) ;
            tx1.add(new KCipher(s1)) ;
            tx1.encode();
            out(tx1,self) ;

            setEncKey("2");
            Tuplex tx2 = new Tuplex() ;
            tx2.add(new KString("item4")) ;
            KInteger s2 = new KInteger(2) ;
            tx2.add(new KCipher(s2)) ;
            tx2.encode();
            out(tx2,self) ;
            unsetEncKey();
        } catch (Exception e) {
            e.printStackTrace ();
        }
    }
}

class AgentHome extends KlavaProcessx {

    protected Locality node1 = new LogicalLocality("data1");
    protected Locality node2 = new LogicalLocality("data2");

    public AgentHome() throws Exception {
        super (new KStoreFileAccess("Agent2.answers"));
    }

    public void execute() throws KlavaException {
        // retrieve my own physical locality
        PhysicalLocality myLoc = getNodeLocality ();

        eval( new AgentCrypto(myLoc, node2), node1 ) ;

        TupleSpace collectedData = new TupleSpace();
        in( collectedData, self );

        Tuplex txf1 = new Tuplex() ;

        // retrieve all the collected data and store them in
        // the local tuple space
        while ( true ) {
            KString s1 = new KString() ;
            txf1.add( s1 ) ;
            KString fString1 = new KString() ;
            KCipher k = new KCipher( fString1 ) ;
            txf1.add( k ) ;
            if (collectedData.in_nb(txf1)) {
                out(new Tuplex(txf1), self) ;
                // put a copy, 'cos afterwards it is reset
                txf1.reset(); // for the next in
            } else {
                Print("No more collected data" ) ;
                break ;
            }
        }

        // now decrypt the collected data store in the
        // the local tuple space
        Tuplex txf2 = new Tuplex() ;
        while ( true ) {
            KString s1 = new KString() ;
            txf2.add( s1 ) ;
            KString fString1 = new KString() ;
            KCipher k = new KCipher( fString1 ) ;
            txf2.add( k ) ;
            if (readk_nb( txf2, self )) {
                out( "decoded", s1, fString1, self ) ;
                txf2.reset();
            } else {
                Print("All possible data decoded" ) ;
                break ;
            }
        }
    }
}

class AgentCrypto extends KlavaProcessx {
    protected Locality owner ;
    protected Locality secondStep ;
    protected boolean done = false ;
    private TupleSpace collectedData ;

    public AgentCrypto( Locality from, Locality to ) {
        owner = from ;
        secondStep = to ;
        collectedData = new TupleSpace() ;
    }

    public void execute() throws KlavaException {
        Tuplex txf1 = new Tuplex() ;
        KString s1 = new KString() ;
        txf1.add( s1 ) ;
        KString fString1 = new KString() ;
        KCipher k = new KCipher( fString1 ) ;
        txf1.add( k ) ;

        in(txf1, self) ;
        collectedData.out(txf1) ;

        if ( ! done ) {
            done = true ;
            Print("second migration");
            eval(this, secondStep) ;
        } else {
            out(collectedData, owner) ;
        }
    }
}

public class crypto_agent {
    static public void main  (String args[]) throws Exception {
        Node Node1 = new NodeG ("node1", 9990, "localhost", 9999 );
        Node Node2 = new NodeG ("node2", 9980, "localhost", 9999 );
        Node Node3 = new NodeG ("node3", 9970, "localhost", 9999 );

        Node1.addToEnv("data2", 9980);
        Node1.start();
        Node1.addProcess (new Node1Proc());

        Node2.start ();
        Node2.addProcess (new Node2Proc());

        Node3.addToEnv("data1", 9990);
        Node3.start ();
        Node3.addProcess (new AgentHome());
    }
}
