/**
 * crypto_agent2
 *
 * retrieves tuples with a mobile agent
 * the agent encrypts tuples collected in remote sites.
 * The agent brings its key (intended to be a public key)
 * extending from KlavaProcessx
 * There are no KCipherMatcher setted: this version uses ink
 * to decode automatically the tuples
 */

package cryptoagent;

import java.io.* ;
import java.util.Vector ;
import java.security.Key;
import Klava.* ;
import Klava.crypto.*;


class Node1Proc2 extends KlavaProcess {
    public void execute() throws KlavaException {
        out(new KString("item1"), new KString("10000"), self) ;
        out(new KString("item2"), new KString("20000"), self) ;
    }
}

class Node2Proc2 extends KlavaProcess {
    public void execute() throws KlavaException {
        out(new KString("item3"), new KInteger(1), self) ;
        out(new KString("item4"), new KString("40000"), self) ;
    }
}

class AgentHome2 extends KlavaProcessx {

    protected Locality node1 = new LogicalLocality("data1");
    protected Locality node2 = new LogicalLocality("data2");

    public AgentHome2() throws Exception {
        super (new KStoreFileAccess("Agent2.answers"));
    }

    public void execute() throws KlavaException {
        // retrieve my own physical locality
        PhysicalLocality myLoc = getNodeLocality ();

        try {
          Key agent_key = useEncKey ("Key for the agent");
          eval( new AgentCrypto2(agent_key, myLoc, node2), node1 ) ;
        } catch (Exception e) {
          e.printStackTrace();
          return;
        }

        TupleSpace collectedData = new TupleSpace();
        in( collectedData, self );

        Tuplex txf1 = new Tuplex() ;

        // retrieve all the collected data and store them in
        // the local tuple space
        while ( true ) {
            KString s1 = new KString() ;
            txf1.add( s1 ) ;
            KString fString1 = new KString() ;
            KCipher k = new KCipher( fString1 ) ;
            txf1.add( k ) ;
            if (collectedData.in_nb(txf1)) {
                out(new Tuplex(txf1), self) ;
                // put a copy, 'cos afterwards it is reset
                txf1.reset(); // for the next in
            } else {
                Print("No more collected data" ) ;
                break ;
            }
        }

        // now decrypt the collected data store in the
        // the local tuple space
        Tuplex txf2 = new Tuplex() ;
        while ( true ) {
            KString s1 = new KString() ;
            txf2.add( s1 ) ;
            KString fString1 = new KString() ;
            KCipher k = new KCipher( fString1 ) ;
            txf2.add( k ) ;
            if (ink_nb( txf2, self )) {
                out( "decoded", s1, fString1, self ) ;
                txf2.reset();
            } else {
                Print("All possible data decoded" ) ;
                break ;
            }
        }
    }
}

class AgentCrypto2 extends KlavaProcessx {
    protected Locality owner ;
    protected Locality secondStep ;
    private Key my_key;
    protected boolean done = false ;
    private TupleSpace collectedData ;

    public AgentCrypto2( Key key, Locality from, Locality to ) throws Exception {
        super (new KStoreFileAccess("Agent2.answers"));
        owner = from ;
        secondStep = to ;
        my_key = key;
        collectedData = new TupleSpace() ;
    }

    public void execute() throws KlavaException {
        KString s1 = new KString() ;
        KString fString1 = new KString() ;
        // retrieve clear text data...
        in(s1, fString1, self, 1000);
        // ... encrypt it
        try {
          Tuplex txf1 = new Tuplex();
          txf1.add(s1);
          txf1.add(new KCipher(fString1));
          txf1.encode(my_key);
          collectedData.out(txf1) ;
        } catch (Exception e) {
          e.printStackTrace();
        }

        if ( ! done ) {
            done = true ;
            Print("second migration");
            eval(this, secondStep) ;
        } else {
            out(collectedData, owner) ;
        }
    }
}

public class crypto_agent2 {
    static public void main  (String args[]) throws Exception {
        Node Node1 = new NodeG ("node1", 9990, "localhost", 9999 );
        Node Node2 = new NodeG ("node2", 9980, "localhost", 9999 );
        Node Node3 = new NodeG ("node3", 9970, "localhost", 9999 );

        Node1.addToEnv("data2", 9980);
        Node1.start();
        Node1.addProcess (new Node1Proc2());

        Node2.start ();
        Node2.addProcess (new Node2Proc2());

        Node3.addToEnv("data1", 9990);
        Node3.start ();
        Node3.addProcess (new AgentHome2());
    }
}
