package cryptomarket;

import java.io.*;
import java.util.Vector	;
import java.util.Enumeration ;

import Klava.* ;
import Klava.crypto.* ;

/*
    CameraClient
    We want to buy a camera using an agent to retrieve camera prices
    visiting some shops

    java CameraClient ClientLoc	CameraMake MarketPlace distance

    CameraMake : CameraX | CameraY

    Decoding of the price in done manually
*/
public class CameraClientx extends KlavaProcessx {
    protected KString CameraMake ;
    protected Locality MarketPlaceDir ;
    protected KInteger distance ;

    protected KInteger CurrentPrice ;
    protected KString CurrentShop ;

    protected MarketPlaceAgent mAgent ;

    public CameraClientx( KString c, Locality m, KInteger d ) throws Exception {
        super (new KStoreFileAccess("Camera.answers")) ;
        CameraMake = c ;
        MarketPlaceDir = m ;
        distance = d ;
    }

    public void execute() throws KlavaException {
        Locality newLoc ;
        PhysicalLocality myLoc = new PhysicalLocality();

        newLoc = newloc() ;
        out( self, newLoc ) ;
        in( myLoc, newLoc ) ;

        /* sends the agent to marketplacedir */
        try {
            mAgent = new MarketPlaceAgent( CameraMake, myLoc, distance ) ;
            eval( mAgent, MarketPlaceDir ) ;
        } catch (Exception e) {
            e.printStackTrace ();
        }
        /* now waits for results */
        TupleSpace CollectedData = new TupleSpace() ;
        in( CollectedData, self) ;
        CurrentPrice = new KInteger(0) ; // initial price
        CurrentShop = new KString( "NO SHOP" ) ;
        while ( true ) {
            Tuplex tx = new Tuplex();
            tx.add( CameraMake ) ;
            KInteger NewCost = new KInteger() ;
            tx.add( new KCipher( NewCost ) ) ;
            KString NewShop = new KString() ;
            tx.add( NewShop );
            try {
                CollectedData.in(tx, 10) ;
            } catch ( KlavaTimeOutException e ) {
                Print("No more collected data" ) ;
                break ;
            }
            Tuple t = new Tuple() ;
            try {
                tx.decode() ;
                if ( CurrentPrice.intValue() == 0 ||
                     NewCost.intValue() < CurrentPrice.intValue() ) {
                    CurrentPrice = NewCost ;
                    CurrentShop = NewShop ;
                }
            } catch (Exception e) {
                e.printStackTrace ();
            }
        }

        Print( CameraMake + " at " + CurrentShop + " costs " + CurrentPrice ) ;
        KString bestBuy = new KString( CameraMake + " at " + CurrentShop + " costs " + CurrentPrice ) ;
        out( new Tuple(bestBuy) ) ;
    }

    synchronized public void Print( String s ) {
        super.Print( "CAMERA CLIENT : " + s ) ;
    }

    public static void main( String args[] ) throws Exception {
        if ( args.length != 6 )	{
            throw new IOException
            ( "args : ClientPort CameraMake MarketLoc " +
            "distance nethost netport" ) ;
        }

        Node node ;
        PhysicalLocality ClientLoc =
            new PhysicalLocality(
            NetUtils.createLocalNodeAddress(Integer.parseInt(args[0])) ) ;
        KString	CameraMake = new KString( args[1] ) ;
        Locality MarketLoc =
            new PhysicalLocality(
            NetUtils.createNodeAddress(args[2]) ) ;
        KInteger distance = new KInteger( Integer.parseInt( args[3] ) ) ;

        node = new NodeG( "CameraClient", ClientLoc, args[4],
                          Integer.parseInt( args[5] ) ) ;
        KlavaProcess P	= new CameraClientx( CameraMake,
            MarketLoc, distance ) ;

        node.start() ;
        node.addProcess( P ) ;
    }
}

class MarketPlaceAgent extends KlavaProcessx {
    protected KString CameraMake ;
    protected Locality ClientLoc ; // to send results
    protected KInteger distance ;

    protected KVector ShopList = new KVector() ; // formal

    protected ShopAgentx sAgent ;

    public MarketPlaceAgent( KString c, Locality l, KInteger d )
        throws Exception {
        CameraMake = c ;
        ClientLoc = l ;
        distance = d ;
    }

    public void execute() throws KlavaException {
        KString cshop = new KString( "cshop" ) ;

        Print( "Arrived!" ) ;

        // asks for shop list
        out( cshop, distance, self ) ;
        // gets shop list
        in( cshop, ShopList, self ) ;

        if ( ShopList.empty() ) {
            out( CameraMake, new KString( "NO SHOPS" ),
                 new KInteger( -1 ), ClientLoc ) ;
        } else {
            sAgent = new ShopAgentx (CameraMake, ShopList, ClientLoc ) ;
            eval( sAgent,
                  (PhysicalLocality)ShopList.vector.firstElement() ) ;
        }
        Print( "Done !" ) ;
    }

    synchronized public void Print( String s ) {
        super.Print( "MARKET PLACE AGENT : " + s ) ;
    }
}

class ShopAgentx extends KlavaProcess {
    protected KString CameraMake ;
    protected Locality ClientLoc ; // to send results
    protected KVector ShopList ;
    private TupleSpace CollectedData ;

    public ShopAgentx( KString c, KVector v, Locality l ) {
        CameraMake = c ;
        ShopList = v ;
        ClientLoc = l ;
        CollectedData = new TupleSpace() ;
    }

    public void execute() throws KlavaException {
        KInteger NewCost = new KInteger() ;
        PhysicalLocality thisShop ;

        Print( "Arrived!" ) ;
        thisShop = (PhysicalLocality)ShopList.vector.firstElement() ;
        ShopList.vector.removeElement(
                                      ShopList.vector.firstElement() ) ;
        try {
            // we don't want to wait for ever, so we use timeout
            Tuplex t =new Tuplex();
            t.add( CameraMake ) ;
            t.add( new KCipher( NewCost ) ) ;
            read(t, self, 10) ;
            t.add( new KString( thisShop.toString() ) );
            CollectedData.out( t ) ;

        } catch ( KlavaTimeOutException e ) {
            Print( CameraMake + " not found here !" ) ;
        }

        if( ! ShopList.empty() ) {
            eval( this, (PhysicalLocality)
                  ShopList.vector.firstElement() );
        } else {
            out(CollectedData, ClientLoc ) ;
        }

        Print( "Done !" ) ;
    }

    synchronized public void Print( String s ) {
        super.Print( "SHOP AGENT : " + s ) ;
    }
}


